/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiFrame;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;
import javax.swing.MenuElement;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import org.apache.log4j.Logger;
import org.gvsig.gui.beans.controls.IControl;

import com.iver.andami.Launcher;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.Messages;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.ExtensionDecorator;
import com.iver.andami.plugins.PluginClassLoader;
import com.iver.andami.plugins.config.generate.ActionTool;
import com.iver.andami.plugins.config.generate.Label;
import com.iver.andami.plugins.config.generate.Menu;
import com.iver.andami.plugins.config.generate.PopupMenu;
import com.iver.andami.plugins.config.generate.SelectableTool;
import com.iver.andami.plugins.config.generate.SkinExtensionType;
import com.iver.andami.plugins.config.generate.ToolBar;
import com.iver.andami.ui.mdiManager.MDIManager;
import com.iver.andami.ui.mdiManager.MDIManagerFactory;


/**
 * Frame principal de la aplicacin.
 *
 * @version $Revision: 1.45 $
 */
public class MDIFrame extends JFrame implements ComponentListener,
	ContainerListener, ActionListener, MainFrame {
	/** DOCUMENT ME! */
	private static Logger logger = Logger.getLogger(MDIFrame.class.getName());
	private MDIManager mdiManager = MDIManagerFactory.createManager();

	/** Elementos de la aplicacin */
	private JMenuBar menuBar = new JMenuBar();

	/** DOCUMENT ME! */
	private JPanel toolBars = new JPanel();

	/** DOCUMENT ME! */
	private NewStatusBar bEstado = null;

	/** Asocia los nombres con las barras de herramientas */
	private HashMap toolBarMap = new HashMap();

    /**
     * actionCommand del tool seleccionado
     */
    private String selectedTool;

	/** Asocia los nombres con los popupMenus */
	private HashMap popupMap = new HashMap();

	/** Asocia controles con la clase de la extension asociada */
	private HashMap controlClass = new HashMap();

	/**
	 * Asocia la informacin sobre las etiquetas que van en la status bar con
	 * cada extension
	 */
	private HashMap classLabels = new HashMap();

	//private HashMap classControls = new HashMap();

	/** ProgressListeners (ver interfaz com.iver.mdiApp.ui.ProgressListener) */
	private ArrayList progressListeners = new ArrayList();

	/** Timer para invocar los enventos de la interfaz anterior */
	private Timer progressTimer = null;

	/** Tabla hash que asocia las clases con las extensiones */
	private HashMap classesExtensions = new HashMap();

	/** ltima clase que activ etiquetas */
	private Class lastLabelClass;

	/** Instancia que pone los tooltip en la barra de estado */
	private TooltipListener tooltipListener = new TooltipListener();
	private HashMap infoCodedMenus = new HashMap();

	private String titlePrefix;
	private ButtonGroup buttonGroup=new ButtonGroup();
	/**
	 * Realiza tareas para poner en marcha la aplicacin
	 *
	 * @throws RuntimeException DOCUMENT ME!
	 */
	public void init() {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("Not Event Dispatch Thread");
		}

		//Se aaden los listeners del JFrame
		this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					/*JOptionPane dlgSalir = new JOptionPane(Utilities.getMessage(this,"quiere_salir"),
					   JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
					 */
					int option = JOptionPane.showConfirmDialog(MDIFrame.this,
							Messages.getString("MDIFrame.quiere_salir"),
							Messages.getString("MDIFrame.salir"),
							JOptionPane.YES_NO_OPTION);

					if (option == JOptionPane.YES_OPTION) {
						Launcher.closeApplication();
					}
				}
			});
		this.addComponentListener(this);
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

		//Se configura la barra de menu
		setJMenuBar(menuBar);

		//Se configura el layout del JFrame principal
		this.getContentPane().setLayout(new BorderLayout());

		/*
		 * Se configura y se aade el JPanel de las barras de
		 * herramientas
		 */
		FlowLayout layout = new FlowLayout(FlowLayout.LEFT);
		layout.setHgap(0);
		layout.setVgap(0);
		toolBars.setLayout(layout);
		getContentPane().add(toolBars, BorderLayout.PAGE_START);

		// Se aade la barra de estado a la aplicacin
		bEstado = new NewStatusBar();
		bEstado.setInfoText(Messages.getString("StatusBar.Aplicacion_iniciada"));
		getContentPane().add(bEstado, BorderLayout.SOUTH);

		this.toolBars.addContainerListener(this);

		pack();

		// TODO LWS Aqui debera cargar los valores salvados de la ltima ejecucin.
		setSize(700, 580);
		setLocation(10,10);
		setExtendedState(MAXIMIZED_BOTH);

		mdiManager.init(this);
	}

	public void setTitle(String title) {
		super.setTitle(titlePrefix + ":" + title);
	}

	/**
	 * Aade un modo de operacin a la caja de herramientas
	 *
	 * @param ext Texto del boton, si es null no aparece texto
	 * @param ext Icono del boton, si es null no aparece icono
	 * @param ext Extensin asociada al control
	 * @param selectableTool Enable text del control
	 *
	 * @throws ClassNotFoundException
	 * @throws RuntimeException DOCUMENT ME!
	 */
	public void addTool(PluginClassLoader loader, SkinExtensionType ext,
		ToolBar toolBar, SelectableTool selectableTool)
		throws ClassNotFoundException {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("No Event Dispatch Thread");
		}

		// Para traducir
		PluginServices ps = PluginServices.getPluginServices(loader.getPluginName());

		JToolBarToggleButton btn;
		URL image = loader.getResource(selectableTool.getIcon());

		if (image != null) {
			btn = new JToolBarToggleButton(selectableTool.getText(),
					new ImageIcon(image));
		} else {
			btn = new JToolBarToggleButton(selectableTool.getText());
		}

		btn.setMargin(new Insets(0, 0, 0, 0));
		btn.addMouseListener(tooltipListener);
		btn.addActionListener(this);
		btn.setFocusable(false);
		btn.setActionCommand(selectableTool.getActionCommand());
		btn.setToolTipText(selectableTool.getTooltip());
		btn.setEnabled(false);
		btn.setVisible(false);

		if (selectableTool.getIsDefault()) {
			btn.setSelected(true);
            selectedTool = btn.getActionCommand();
		}

		String name = getName(toolBar.getName(), loader);

		SelectableToolBar jtb = (SelectableToolBar) toolBarMap.get(name);

		if (jtb == null) {
			jtb = new SelectableToolBar(toolBar.getName());
			jtb.setRollover(true);
			toolBarMap.put(name, jtb);
			toolBars.add(jtb);

		}

		jtb.addButton(buttonGroup, btn);
		controlClass.put(btn, loader.loadClass(ext.getClassName()));

		if (selectableTool.getTooltip() != null) {
			btn.setToolTip(ps.getText(selectableTool.getTooltip()));
		}

		if (selectableTool.getEnableText() != null) {
			btn.setEnableText(ps.getText(selectableTool.getEnableText()));
		}

		if (selectableTool.getLast() == true) {
			jtb.addSeparator();
		}
	}

	/**
	 * Aade un botn a la barra de herramientas
	 *
	 * @param ext Texto del boton, si es null no aparece texto
	 * @param ext Extensin asociada al control
	 * @param toolBar Icono del boton, si es null no aparece texto
	 * @param actionTool Tooltip de la barra de herramientas
	 *
	 * @throws ClassNotFoundException
	 * @throws RuntimeException DOCUMENT ME!
	 */
	public void addTool(PluginClassLoader loader, SkinExtensionType ext,
		ToolBar toolBar, ActionTool actionTool) throws ClassNotFoundException {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("No Event Dispatch Thread");
		}

		// Para traducir los textos que vengan
		PluginServices ps = PluginServices.getPluginServices(loader.getPluginName());

		JToolBarButton btn;
		URL image = loader.getResource(actionTool.getIcon());

		if (image != null) {
			btn = new JToolBarButton(actionTool.getText(), new ImageIcon(image));
		} else {
			btn = new JToolBarButton(actionTool.getText());
		}

		btn.setMargin(new Insets(0, 0, 0, 0));
		btn.addMouseListener(tooltipListener);
		btn.addActionListener(this);
		btn.setFocusable(false);
		btn.setActionCommand(actionTool.getActionCommand());
		btn.setEnabled(false);
		btn.setVisible(false);

		String name = getName(toolBar.getName(), loader);

		SelectableToolBar jtb = (SelectableToolBar) toolBarMap.get(name);

		if (jtb == null) {
			jtb = new SelectableToolBar(toolBar.getName());
			jtb.setRollover(true);
			toolBarMap.put(name, jtb);
			toolBars.add(jtb);
		}

		jtb.add(btn);

		controlClass.put(btn, loader.loadClass(ext.getClassName()));

		if (actionTool.getTooltip() != null) {
			btn.setToolTip(ps.getText(actionTool.getTooltip()));
		}

		if (actionTool.getEnableText() != null) {
			btn.setEnableText(ps.getText(actionTool.getEnableText()));
		}

		if (actionTool.getLast() == true) {
			jtb.addSeparator();
		}
	}

	/**
	 * Creates the needed menu structure to add the menu to the bar.
	 * Returns the father which must hold the menu which was
	 * provided as parameter.
	 *
	 * Crea la estructura de mens necesaria para aadir el menu a la barra.
	 * Devuelve el padre del cual debe colgar el menu que se pasa como
	 * parmetro.
	 *
	 * @param menu The Menu whose support is going to be added
	 * @param loader The plugin's class loader
	 *
	 * @return The proper father for the menu which was provided as parameter
	 */
	private JMenu createMenuAncestors(Menu menu, PluginClassLoader loader) {
		JMenu menuPadre = null;

		String[] menues = menu.getText().split("/");
		PluginServices ps = PluginServices.getPluginServices(loader.getPluginName());

		for (int i = 0; i < menues.length; i++) {
			menues[i] = ps.getText(menues[i]);
		}

		//Se busca el padre en la menuBar
		for (int i = 0; i < menuBar.getMenuCount(); i++) {
			if (menuBar.getMenu(i).getText().compareToIgnoreCase(menues[0]) == 0) {
				menuPadre = (JMenu) menuBar.getMenu(i);
			}
		}

		//Si no se encuentra el padre se crea
		if (menuPadre == null) {
			menuPadre = new JMenu(menues[0]);
			menuBar.add(menuPadre);
		}

		//Se crea el resto de menus
		Vector temp = new Vector();

		for (int i = 1; i < (menues.length - 1); i++) {
			temp.add(menues[i]);
		}

		menuPadre = createMenus(temp, menuPadre);

		return menuPadre;
	}

	/**
	 * Aade la informacin del menu al framework. Debido a que los menes se
	 * pueden introducir en un orden determinado por el usuario, pero los
	 * plugins se instalan en un orden arbitrario, primero se almacena la
	 * informacin de todos los menus para luego ordenarlos y posteriormente
	 * aadirlos al interfaz
	 *
	 * @param loader Posicion del menu. Se ordena por este campo
	 * @param ext Array con los nombres de los padres del menu
	 * @param menu Texto del menu
	 *
	 * @throws ClassNotFoundException
	 * @throws RuntimeException DOCUMENT ME!
	 */
	public void addMenu(PluginClassLoader loader, SkinExtensionType ext,
		Menu menu) throws ClassNotFoundException {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("No Event Dispatch Thread");
		}

		JMenu menuPadre = createMenuAncestors(menu, loader);

		//Se registra y aade el menu
        /* String[] aux = menu.getText().split("/");

        if (aux.length == 2)
            if (aux[1].equals("----"))
            {
                menuPadre.addSeparator();
                return;
            } */
        if (menu.getIs_separator())
        {
            menuPadre.addSeparator();
            return;
        }

		JMenuItem nuevoMenu = createJMenuItem(loader, menu);
		nuevoMenu.addMouseListener(tooltipListener);
		nuevoMenu.addActionListener(this);
		menuPadre.add(nuevoMenu);
        controlClass.put(nuevoMenu, loader.loadClass(ext.getClassName()));
	}

	/**
	 * Dado un array de nombres de menu traducidos, encuentra el  men
	 *
	 * @param nombres DOCUMENT ME!
	 * @param padre DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private javax.swing.JMenuItem getMenu(Vector nombres, JMenu padre) {
		javax.swing.JMenuItem buscado = null;
		javax.swing.JMenuItem hijo;

		for (int i = 0; i < padre.getMenuComponentCount(); i++) {
			if (padre.getMenuComponent(i) instanceof javax.swing.JMenuItem) { //we need to test that, as there are also separators
				hijo = (javax.swing.JMenuItem) padre.getMenuComponent(i);

				if (hijo.getText().compareToIgnoreCase((String) nombres.get(0)) == 0) {
					buscado = hijo;
				}
			}
		}

		//Si no se encuentra el menu
		if (buscado == null) {
			return null;
		}

		nombres.remove(0);

		//Si se ha llegado al fin de la ruta completa
		if (nombres.isEmpty()) {
			return buscado;
		} else {
			if (buscado instanceof JMenu) {
				return getMenu(nombres, (JMenu) buscado);
			} else {
				return null;
			}
		}
	}

	/**
	 * Crea la estructura de menus recursivamente. Por ejemplo, si se le pasa
	 * en el parmetro nombres el array {"Search", "References", "Workspace"}
	 * crear un men Search, un submen del anterior que se llamar
	 * References y debajo de ste ltimo otro menu llamado Workspace
	 *
	 * @param nombres Array con los nombres de los mens que se quieren crear
	 * @param padre Menu padre de los mens creados. Es til porque es un
	 * 		  algoritmo recursivo
	 *
	 * @return Devuelve el men creado. Al final de toda la recursividad,
	 * 		   devolver el men de ms abajo en la jerarqua
	 *
	 * @throws RuntimeException DOCUMENT ME!
	 */
	private JMenu createMenus(Vector nombres, JMenu padre) {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("No Event Dispatch Thread");
		}

		//si no quedan nombres de menu por crear se vuelve: caso base
		if (nombres.size() == 0) {
			return padre;
		}

		//Se busca el menu por si ya existiera para no crearlo otra vez
		JMenu buscado = null;

		for (int i = 0; i < padre.getMenuComponentCount(); i++) {
			try {
				JMenu hijo = (JMenu) padre.getMenuComponent(i);

				if (hijo.getText().compareToIgnoreCase((String) nombres.get(0)) == 0) {
					buscado = hijo;
				}
			} catch (ClassCastException e) {
				/*
				 * Se ha encontrado un elemento hoja del arbol de menes
				 */
			}
		}

		if (buscado != null) {
			//Si lo hemos encontrado creamos el resto
			nombres.remove(0);

			return createMenus(nombres, buscado);
		} else {
			//Si no lo hemos encontrado se crea el menu, se aade al padre
			//y se crea el resto
			JMenu menuPadre = new JMenu((String) nombres.get(0));
			padre.add(menuPadre);

			nombres.remove(0);

			return createMenus(nombres, menuPadre);
		}
	}

	/**
	 * Mtodo invocado en respuesta a ciertos eventos de la interfaz que pueden
	 * ocultar botones de las barras de herramientas y que redimensiona sta
	 * de manera conveniente para que no se oculte ninguno
	 */
	private void ajustarToolBar() {
		int margen = 8;
		int numFilas = 1;
		double acum = margen;

		int toolHeight = 0;

		for (int i = 0; i < toolBars.getComponentCount(); i++) {
			Component c = toolBars.getComponent(i);

			if (!c.isVisible()) {
				continue;
			}

			double width = c.getPreferredSize().getWidth();
			acum = acum + width;

			if (acum > this.getWidth()) {
				numFilas++;
				acum = width + margen;
			}

			if (c.getPreferredSize().getHeight() > toolHeight) {
				toolHeight = c.getPreferredSize().height;
			}
		}

		toolBars.setPreferredSize(new Dimension(this.getWidth(),
				(int) (numFilas * toolHeight)));

		toolBars.updateUI();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param classesExtensions
	 */
	public void setClassesExtensions(HashMap classesExtensions) {
		this.classesExtensions = classesExtensions;
	}

	/**
	 * Mtodo de callback invocado cuando se selecciona un men o un botn de
	 * la barra de herramientas. Se busca la extensin asociada y se ejecuta
	 *
	 * @param e Evento producido
	 */
	public void actionPerformed(ActionEvent e) {
		Object control = e.getSource();
		com.iver.andami.plugins.IExtension ext = (com.iver.andami.plugins.IExtension) classesExtensions.get((Class) controlClass.get(
				control));
		String actionCommand = e.getActionCommand();
		try {
			logger.debug(Messages.getString("Ejecutando comando: ") + actionCommand);
			ext.execute(actionCommand);
	        if (control instanceof JToolBarToggleButton)
	        {
	            selectedTool = actionCommand;
	        }
		} catch (RuntimeException t) {
			if (ext==null) {
				logger.error(Messages.getString("No_extension_associated_with_this_event_")+ e.getActionCommand());
			}
			NotificationManager.addError(Messages.getString(
					"MDIFrame.Error_no_capturado_por_el_usuario"), t);
		}

		enableControls();
		showMemory();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param name DOCUMENT ME!
	 * @param loader DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private String getName(String name, PluginClassLoader loader) {
		if (name.indexOf('.') == -1) {
			return loader.getPluginName() + "." + name;
		} else {
			return name;
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param loader DOCUMENT ME!
	 * @param menu DOCUMENT ME!
	 *
	 * @throws RuntimeException DOCUMENT ME!
	 */
	public void addPopupMenu(PluginClassLoader loader, PopupMenu menu) {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("No Event Dispatch Thread");
		}

		String name = getName(menu.getName(), loader);

		//Se crea el control popupmenu
		JPopUpMenu popupMenu = (JPopUpMenu) popupMap.get(name);

		if (popupMenu == null) {
			popupMenu = new JPopUpMenu(menu.getName());
			popupMap.put(name, popupMenu);
		}

		// Se aaden las entradas
		Menu[] menues = menu.getMenu();

		for (int i = 0; i < menues.length; i++) {
			//Se registra y aade el menu
			JMenuItem nuevoMenu = createJMenuItem(loader, menues[i]);

			popupMenu.add(nuevoMenu);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param loader
	 * @param menu
	 *
	 * @return
	 *
	 * @throws RuntimeException DOCUMENT ME!
	 */
	private JMenuItem createJMenuItem(PluginClassLoader loader, Menu menu) {
		JMenuItem nuevoMenu = null;

		PluginServices ps = PluginServices.getPluginServices(loader.getPluginName());
		String texto = menu.getText();
		texto = texto.substring(texto.lastIndexOf('/') + 1);
		texto = ps.getText(texto);

		if (menu.getIcon() != null) {
			URL url = loader.getResource(menu.getIcon());
			if (url != null) {
				nuevoMenu = new JMenuItem(texto, new ImageIcon(url));
			} else {
				NotificationManager.addError(PluginServices.getText(this, "Unable_to_find_icon") +": "+menu.getIcon(), new FileNotFoundException());
			}
		} else {
			nuevoMenu = new JMenuItem(texto);
		}

		if (menu.getMnemonic() != null) {
			if (menu.getMnemonic().length() != 1) {
				throw new RuntimeException(
					"Mnemonic must be 1 character length");
			}

			nuevoMenu.setMnemonic(KeyMapping.getKey(menu.getMnemonic().charAt(0)));
		}

		if (menu.getKey() != null) {
			nuevoMenu.setAccelerator(KeyStroke.getKeyStroke(KeyMapping.getKey(
						menu.getKey().charAt(0)), ActionEvent.ALT_MASK));
		}

		nuevoMenu.setActionCommand(menu.getActionCommand());

		if (menu.getTooltip() != null) {
			nuevoMenu.setToolTip(ps.getText(menu.getTooltip()));
		}

		if (menu.getEnableText() != null) {
			nuevoMenu.setEnableText(ps.getText(menu.getEnableText()));
		}

		nuevoMenu.setEnabled(true);
		nuevoMenu.setVisible(true);

		return nuevoMenu;
	}

	/**
	 * Muestra u oculta el menu de nombre 'name'
	 *
	 * @param name Nombre del menu que se quiere mostrar
	 * @param x Evento de raton
	 * @param y DOCUMENT ME!
	 * @param c DOCUMENT ME!
	 */
	private void showPopupMenu(String name, int x, int y, Component c) {
		JPopupMenu menu = (JPopupMenu) popupMap.get(name);

		if (menu != null) {
			menu.show(c, x, y);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param name DOCUMENT ME!
	 * @param listener DOCUMENT ME!
	 */
	public void removePopupMenuListener(String name, ActionListener listener) {
		JPopupMenu menu = (JPopupMenu) popupMap.get(name);

		if (menu != null) {
			Component[] jmenuitems = menu.getComponents();

			for (int i = 0; i < jmenuitems.length; i++) {
				if (jmenuitems[i] instanceof JMenuItem) {
					((JMenuItem) jmenuitems[i]).removeActionListener(listener);
				}
			}
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param popupName
	 * @param c DOCUMENT ME!
	 * @param listener
	 * @param loader
	 */
	public void addPopupMenuListener(String popupName, Component c,
		ActionListener listener, PluginClassLoader loader) {
		final String name = getName(popupName, loader);

		JPopupMenu menu = (JPopupMenu) popupMap.get(name);

		if (menu != null) {
			Component[] jmenuitems = menu.getComponents();

			for (int i = 0; i < jmenuitems.length; i++) {
				if (jmenuitems[i] instanceof JMenuItem) {
					((JMenuItem) jmenuitems[i]).addActionListener(listener);
				}
			}
		}

		c.addMouseListener(new MouseAdapter() {
				public void mousePressed(MouseEvent e) {
					if (e.isPopupTrigger()) {
						showPopupMenu(name, e.getX(), e.getY(), e.getComponent());
					}
				}

				public void mouseReleased(MouseEvent e) {
					if (e.isPopupTrigger()) {
						showPopupMenu(name, e.getX(), e.getY(), e.getComponent());
					}
				}
			});
	}

	/**
	 * Itera por los controles preguntando a las extensiones si estos estn
	 * habilitados y visibles
	 *
	 * @throws RuntimeException DOCUMENT ME!
	 */
	public void enableControls() {
		if (!SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("No Event Dispatch Thread");
		}

		Iterator e = classesExtensions.values().iterator();
		HashMap estadoExtensiones = new HashMap();
		HashMap visibilidadExtensiones = new HashMap();

		while (e.hasNext()) {
			ExtensionDecorator ext = (ExtensionDecorator) e.next();

			try {
				if (estadoExtensiones.get(ext) == null) {
					boolean b;
					if (ext.getVisibility() == ExtensionDecorator.ALWAYS_VISIBLE)
						b = true;
					else if (ext.getVisibility() == ExtensionDecorator.ALWAYS_INVISIBLE)
						b = false;
					else b = ext.isVisible();
					Boolean visible = new Boolean(b);
					Boolean enabled = new Boolean(false);

					if (visible.booleanValue()) {
						enabled = new Boolean(ext.isEnabled());
					}

					estadoExtensiones.put(ext, enabled);
					visibilidadExtensiones.put(ext, visible);
				}
			} catch (Throwable e1) {
				NotificationManager.addError(Messages.getString(
						"MDIFrame.Error_no_capturado_por_el_usuario"), e1);
				estadoExtensiones.put(ext, Boolean.FALSE);
			}
		}

		//Se habilitan y deshabilitan los controles
		e = controlClass.keySet().iterator();

		while (e.hasNext()) {
			JComponent control = (JComponent) e.next();

			try {
				com.iver.andami.plugins.IExtension ext = (com.iver.andami.plugins.IExtension) classesExtensions.get((Class) controlClass.get(
							control));
				boolean enabled = ((Boolean) estadoExtensiones.get(ext)).booleanValue();
				boolean visible = ((Boolean) visibilidadExtensiones.get(ext)).booleanValue();
				control.setEnabled(enabled);
				control.setVisible(visible);
			} catch (Exception ex) {
				control.setEnabled(false);
				control.setVisible(false);
			}
		}

		//Se itera por los menes para ocultar los que no tengan hijos visibles
		for (int i = 0; i < menuBar.getMenuCount(); i++) {
			MenuElement menu = menuBar.getMenu(i);
			ocultarMenus(menu);
		}

		//Se ocultan las barras que no tienen herramientas
		Iterator it = toolBarMap.values().iterator();

		while (it.hasNext()) {
			SelectableToolBar t = (SelectableToolBar) it.next();
			boolean todosOcultos = true;

			for (int i = 0; i < t.getComponentCount(); i++) {
				if (!(t.getComponent(i) instanceof JSeparator) // separators doesn't matter
						&& t.getComponent(i).isVisible()) {
					todosOcultos = false;
				}
			}

			if (todosOcultos) {
				t.setVisible(false);
			} else {
				t.setVisible(true);
			}
		}

		if (mdiManager != null) {
			JPanel f = (JPanel) mdiManager.getActiveWindow();

			if (f != null) {
				if (lastLabelClass != f.getClass()) {
					lastLabelClass = f.getClass();

					Label[] lbls = (Label[]) classLabels.get(lastLabelClass);

					if (lbls != null) {
						bEstado.setLabelSet(lbls);
					}
				}
			}
		}

		ajustarToolBar();

		showMemory();
	}

	/**
	 * Establece la visibilidad de un menu y todos sus descendientes en la
	 * jerarquia teniendo en cuenta la visibilidad de todos los submenus.
	 *
	 * @param menu Menu que se quiere visualizar
	 *
	 * @return Devuelve true si el menu es visible y false en caso contrario
	 */
	private boolean ocultarMenus(MenuElement menu) {
		MenuElement[] submenus = menu.getSubElements();

		//Si no tiene hijos se devuelve su visibilidad
		if (submenus.length == 0) {
			return menu.getComponent().isVisible();
		}

		/*
		 * Si tiene hijos se devuelve true si algno de ellos es visible,
		 * pero se itera por todos ellos
		 */
		boolean visible = false;

		for (int i = 0; i < submenus.length; i++) {
			if (ocultarMenus(submenus[i])) {
				if (!(menu instanceof JPopupMenu)) {
					menu.getComponent().setVisible(true);
				}

				visible = true;
			}
		}

		if (visible) {
			return true;
		}

		menu.getComponent().setVisible(false);

		return false;
	}

	/**
	 * Muestra la memoria consumida por el programa
	 */
	private void showMemory() {
		Runtime r = Runtime.getRuntime();
		long mem = r.totalMemory() - r.freeMemory();
		logger.debug("Memoria total: " + mem);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return
	 */
	public MDIManager getMDIManager() {
		return mdiManager;
	}

	/**
	 * Establece el mensaje en la barra de estado asociado a una etiqueta
	 *
	 * @return DOCUMENT ME!
	 */
	public NewStatusBar getStatusBar() {
		return bEstado;
	}
	/**
     * You can use this function to select the appropiate
     * tool inside the toolbars
     */
    public void setSelectedTool(String actionCommand)
    {
        Iterator it = toolBarMap.values().iterator();
        while (it.hasNext()) {
            SelectableToolBar t = (SelectableToolBar) it.next();
            t.setSelectedTool(actionCommand);
        }
        selectedTool = actionCommand;

    }

	/**
	 * DOCUMENT ME!
	 *
	 * @param clase
	 * @param label
	 */
	public void setLabels(Class clase, Label[] label) {
		classLabels.put(clase, label);
	}

	public void addStatusBarControl(Class extensionClass, IControl control) {
		control.addActionListener(this);
		bEstado.addControl(control.getName(), (Component)control);
		controlClass.put(control, extensionClass);
	}

	/**
	 * @see com.iver.andami.ui.mdiFrame.MainFrame#removeMenu(com.iver.andami.plugins.config.generate.Menu)
	 */
	public void removeMenu(Menu menu) {
		JMenuItem delete = (JMenuItem) infoCodedMenus.get(menu);

		if (delete == null) {
			throw new NoSuchElementException(menu.getText());
		}

		delete.getParent().remove(delete);
		infoCodedMenus.remove(menu);
	}

	/**
	 * @see com.iver.andami.ui.mdiFrame.MainFrame#addMenu(com.iver.andami.plugins.config.generate.Menu,
	 * 		java.awt.event.ActionListener, PluginClassLoader)
	 */
	public void addMenu(Menu menu, ActionListener listener,
		PluginClassLoader loader) {
		JMenu menuPadre = createMenuAncestors(menu, loader);

		//Se registra y aade el menu
		JMenuItem nuevoMenu = createJMenuItem(loader, menu);
		nuevoMenu.addMouseListener(tooltipListener);
		nuevoMenu.addActionListener(listener);
		menuPadre.add(nuevoMenu);

		infoCodedMenus.put(menu, nuevoMenu);
	}

	/**
	 * @see com.iver.andami.ui.mdiFrame.MainFrame#changeMenuName(java.lang.String[],
	 * 		String, com.iver.andami.plugins.PluginClassLoader)
	 */
	public void changeMenuName(String[] menu, String newName,
		PluginClassLoader loader) {
		PluginServices ps = PluginServices.getPluginServices(loader.getPluginName());

		for (int i = 0; i < menu.length; i++) {
			menu[i] = ps.getText(menu[i]);
		}

		JMenu menuPadre = null;

		//Se busca el padre en la menuBar
		for (int i = 0; i < menuBar.getMenuCount(); i++) {
			if (menuBar.getMenu(i).getText().compareToIgnoreCase(menu[0]) == 0) {
				menuPadre = (JMenu) menuBar.getMenu(i);
			}
		}

		if (menuPadre == null) {
			throw new NoSuchMenuException(menu[0]);
		}

		Vector nombres = new Vector();

		for (int i = 1; i < menu.length; i++) {
			nombres.add(menu[i]);
		}

		javax.swing.JMenuItem m = getMenu(nombres, menuPadre);

		if (m != null) {
			m.setText(newName);
		}else{
			throw new NoSuchMenuException();
		}
	}

	/**
	 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
	 */
	public void componentHidden(ComponentEvent arg0) {
	}

	/**
	 * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
	 */
	public void componentMoved(ComponentEvent arg0) {
	}

	/**
	 * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
	 */
	public void componentResized(ComponentEvent arg0) {
		ajustarToolBar();
		bEstado.setFixedLabelWidth(this.getWidth() * 0.3);
	}

	/**
	 * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
	 */
	public void componentShown(ComponentEvent arg0) {
	}

	/**
	 * @see java.awt.event.ContainerListener#componentAdded(java.awt.event.ContainerEvent)
	 */
	public void componentAdded(ContainerEvent arg0) {
		ajustarToolBar();
	}

	/**
	 * @see java.awt.event.ContainerListener#componentRemoved(java.awt.event.ContainerEvent)
	 */
	public void componentRemoved(ContainerEvent arg0) {
		ajustarToolBar();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @author $author$
	 * @version $Revision: 1.45 $
	 */
	public class TooltipListener extends MouseAdapter {
		/**
		 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		 */
		public void mouseEntered(MouseEvent e) {
			JComponent control = (JComponent) e.getSource();
			EnableTextSupport ets = (EnableTextSupport) e.getSource();

			String texto = null;
			texto = control.getToolTipText();

			if (texto != null) {
				bEstado.setInfoTextTemporal(texto);
			}
		}

		/**
		 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		 */
		public void mouseExited(MouseEvent arg0) {
			bEstado.restaurarTexto();
		}

		/**
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
		public void mousePressed(MouseEvent e) {
			bEstado.restaurarTexto();
		}
	}
	public String getTitlePrefix() {
		return titlePrefix;
	}
	public void setTitlePrefix(String titlePrefix) {
		this.titlePrefix = titlePrefix;
	}

    public String getSelectedTool() {
        return selectedTool;
    }


    /**
     * Get a previously added JComponent by name. For example
     * you can use it if you need to obtain a JToolBar to
     * add some customized component.
     * @param name
     * @return the JComponent or null if none has been found
     */
    public JComponent getComponentByName(String name)
    {
        Iterator e = controlClass.keySet().iterator();

        while (e.hasNext()) {
            JComponent control = (JComponent) e.next();
            String nameCtrl = control.getName();
            if (nameCtrl != null)
            	if (nameCtrl.compareTo(name) == 0)
            		return control;
        }
        Iterator it = toolBarMap.values().iterator();
        while (it.hasNext()) {
            SelectableToolBar t = (SelectableToolBar) it.next();
            String nameCtrl = t.getName();
            if (nameCtrl != null)
            	if (nameCtrl.compareTo(name) == 0)
            		return t;

        }

        return null;
    }
}
