
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.metadataXML;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.Writer;
import java.util.Hashtable;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Esta clase representa un XMLNode simplificado
 * Contiene una lista de subnodos y una lista de atributos.
 * Tambin tiene una cadena.
 * Soporta la lectura y escritura de y desde un fichero XML.
 * Tambin soporta la lectura desde Internet
 * Modificado por jaume
 * 
 */
public class XMLNode {
	public static final String ISNOTXML = "NOTXML";

/**
 * 
 * 
 */
    private Vector subNodes = new Vector();

/**
 * 
 * 
 */
    private String cdata = null;
// atributos

/**
 * 
 * 
 */
    private Hashtable attr = new Hashtable();

/**
 * 
 * 
 */
    private String nodeName;

/**
 * 
 * 
 */
    private String text = null;
// los nombres de los atributos

/**
 * 
 * 
 */
    private Vector attrKeys = new Vector();
// cabecera XML

/**
 * 
 * 
 */
    private String header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";

/**
 * Constructor, lee de un fichero
 * 
 * 
 * @param file 
 * @author jaume - jaume.dominguez@iver.es
 * Modified by Jorge Piera Llodra - piera_jor@gva.es
 */
    public  XMLNode(File file) throws Exception {        
      this(factory.newDocumentBuilder().parse(file));
    } 

/**
 * Constructor. Usando url.openStream() se puede usar
 * para leer info descargada de internet.
 * 
 * 
 * @param inputstream 
 * @throws Exception
 */
    public  XMLNode(InputStream inputstream) throws Exception {        
       this(factory.newDocumentBuilder().parse(inputstream));
    } 

/**
 * Contructor, constructor desde un documento DOM
 * 
 * 
 * @param dom 
 * @author jaume - jaume.dominguez@iver.es
 */
    public  XMLNode(Document dom) throws Exception {        
      this((Element)dom.getFirstChild());
    } 

/**
 * Contructor, crea un nodo con su nombre
 * 
 * 
 * @param name 
 */
    public  XMLNode(String name) throws Exception {        
      nodeName = name;
    } 

/**
 * Contructor, crea un nodo con su nombre y el texto
 * 
 * 
 * @param name 
 * @param text 
 */
    public  XMLNode(String name, String text) throws Exception {        
      nodeName = name;
      this.text = text;
    } 

/**
 * Contructor, desde un elemento DOM
 * 
 * 
 * @param dom 
 */
    public  XMLNode(Element dom) throws Exception {        
      nodeName = dom.getNodeName();
      NamedNodeMap map = dom.getAttributes();
      for (int i = 0; i < map.getLength(); i++) {
         Node att = map.item(i);
         addAtrribute(att.getNodeName(), att.getNodeValue());
         
      }
      NodeList nodeList = dom.getChildNodes();
      for (int i = 0; i < nodeList.getLength(); i++) {
         Node sub = nodeList.item(i);
         if (sub.getNodeType() == Node.ELEMENT_NODE) {
            addSubNode(new XMLNode((Element)sub));
         }else if (sub.getNodeType() == Node.CDATA_SECTION_NODE){
             String sCdata = sub.getNodeValue().trim();   
             if (sCdata.length() > 0) {
                 cdata = sCdata; 
             }
         }else if (sub.getNodeType() == Node.TEXT_NODE) {
            String s = sub.getNodeValue().trim();
            if (s.length() > 0) {
               if (text != null) {
                  throw new Exception("XMLNode '" + nodeName + "' has 2 Textblocks");
               }
               text = s;
            }
         }
      }
    } 

/**
 * 
 * 
 * 
 * @param s 
 */
    public void setText(String s) {        
      text = s;
    } 

/**
 * 
 * 
 * 
 * @param s 
 */
    public void addSubNode(XMLNode s) {        
      subNodes.add(s);
    } 

/**
 * 
 * 
 * 
 * @param name 
 * @param value 
 */
    public void addAtrribute(String name, String value) throws Exception {        
      if (attr.containsKey(name)) {
         throw new Exception(
               "XMLNode '" + nodeName + "' already contains Attribute '" + name + "'");
      }
      attr.put(name, value);
      attrKeys.add(name);
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public int getNumSubNodes() {        
      return subNodes.size();
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public String getName() {        
      return nodeName;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public String getText() {        
      return text;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public String getCdata() {        
       return cdata;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param index 
 */
    public XMLNode getSubNode(int index) {        
      return (XMLNode)subNodes.get(index);
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public XMLNode[] getSubnodes() {        
       XMLNode[] xmlNodes = new XMLNode[getNumSubNodes()];
       for (int i=0 ; i<getNumSubNodes() ; i++){
           xmlNodes[i] = getSubNode(i);
       }
       return xmlNodes;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public Vector getAttributeNames() {        
       return attrKeys;
    } 

/**
 * 
 * 
 * 
 * @param wr 
 */
    public void write(Writer wr) throws Exception {        
      wwrite("", wr);
    } 

/**
 * Escribe el cdigo XML de este objeto con su identacin
 * 
 * 
 * @param pre 
 * @param wr 
 */
    private void wwrite(String pre, Writer wr) throws Exception {        
      wr.write(pre + "<" + nodeName);
      for (int i = 0; i < attrKeys.size(); i++) {
         String name = (String)attrKeys.get(i);
         String val = (String)attr.get(name);
         wr.write(" " + name + "='" + val + "'");
      }
      if (getNumSubNodes() == 0 && text == null) {
         wr.write("/>\n");
      }
      else {
         wr.write(">");
         if (text != null) {
            wr.write(text);
         }
         if (getNumSubNodes() > 0) {
            wr.write("\n");
            for (int i = 0; i < subNodes.size(); i++) {
               if (getSubNode(i) != null) {
                  getSubNode(i).wwrite(pre + "  ", wr);
               }
            }
            wr.write(pre + "</" + nodeName + ">\n");
         }
         else {
            wr.write("</" + nodeName + ">\n");
         }
      }
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param key 
 */
    public String getAttribute(String key) {        
      return (String)attr.get(key);
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param key 
 */
    public double getDoubleAttribute(String key) {        
      return Double.parseDouble((String)attr.get(key));
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param key 
 */
    public boolean getBoolAttribute(String key) {        
      if (!hasAttribute(key)) {
         return false;
      }
      return new Boolean((String)attr.get(key)).booleanValue();
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param key 
 */
    public int getIntAttribute(String key) {        
      return Integer.parseInt((String)attr.get(key));
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param key 
 */
    public boolean hasAttribute(String key) {        
      return attr.containsKey(key);
    } 
// escribe al fichero

/**
 * 
 * 
 * 
 * @param f 
 */
    public void write(File f) throws Exception {        
      FileWriter fw = new FileWriter(f);
      fw.write(header);
      write(fw);
      fw.flush();
      fw.close();
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public String toString() {        
      return this.getName();
      /*
       StringWriter wr = new StringWriter();
      try {
         write(wr);
      }
      catch (Exception e) {
         System.err.println("Exception in StringWriter " + e);
         e.printStackTrace();
         System.exit(1);
      }
      return wr.toString();
      */
    } 
/**
 * 
 * 
 */
    static DocumentBuilderFactory factory;

/**
 * 
 * 
 */
    static {        
      factory = DocumentBuilderFactory.newInstance();
      factory.setValidating(false);
      factory.setNamespaceAware(false);
      factory.setIgnoringComments(true);
    }  

/**
 * 
 * 
 * 
 * @param header 
 */
    public void setHeader(String header) {        
      this.header = header;
    } 

/**
 * This method prints all the child nodes. It is used only for
 * fixing bugs.
 * 
 */
    public void printSubNodes() {        
       for (int i=0 ; i<getNumSubNodes() ; i++){
           System.out.println(getSubNode(i).getName() + " = " + getSubNode(i).getText());
       }
    } 
 }
