
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.ui.showResults;
import es.gva.cit.catalogClient.CatalogClient;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTreeNumberOfRecordsAnswer;
import es.gva.cit.catalogClient.schemas.discoverer.Resource;
import es.gva.cit.catalogClient.ui.chooseResource.ChooseResourceDialog;
import es.gva.cit.catalogClient.ui.showTree.ShowTreeDialog;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * 
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class ShowResultsDialogPanel extends JPanel implements ActionListener {
//It is needed to close the frame
/**
 * 
 * 
 */
    private JDialog parent;
/**
 * 
 * 
 */
    protected ShowResultsPanel controlsPanel = null;
/**
 * 
 * 
 */
    private CatalogClient client = null;

/**
 * 
 * 
 */
    private int currentRecord = 0;
/**
 * 
 * 
 */
    private JButton nextButton = null;
/**
 * 
 * 
 */
    private JButton lastButton = null;
/**
 * 
 * 
 */
    private JButton descriptionButton = null;
/**
 * 
 * 
 */
    private JButton mapButton = null;
/**
 * 
 * 
 */
    private JButton closeButton = null;

/**
 * 
 * 
 */
    protected Collection nodes = new java.util.ArrayList();

/**
 * 
 * 
 * 
 * @param client 
 * @param nodes 
 * @param currentRecord 
 * @param translator 
 */
    public  ShowResultsDialogPanel(CatalogClient client, Collection nodes, int currentRecord) {        
        this.nodes = nodes;
        this.client = client;
        this.currentRecord = currentRecord;
        int numRecords = XMLTreeNumberOfRecordsAnswer.getNumberOfRecords((XMLNode)nodes.toArray()[0]);
        controlsPanel = new ShowResultsPanel((XMLNode)nodes.toArray()[currentRecord],
                client.getProtocol(), numRecords,client.getUrl());
        
  
        setDefaultButtonListeners();
        enableMapButton();
        add(controlsPanel);
    } 

/**
 * 
 * 
 */
    public void enableMapButton() {        
    	getMapButton().setEnabled(false);
    	Resource[] resources = controlsPanel.getTags().getResources();
        if (resources == null)
            return;
    	for (int i = 0; i < resources.length; i++) {
        	String protocol = resources[i].getProtocol();
                	
			if (protocol != null){        	
			    if ((protocol.toUpperCase().indexOf(Resource.WMS) >= 0) ||
			            (protocol.toUpperCase().indexOf(Resource.WFS) >=0) ||
			            (protocol.toUpperCase().indexOf(Resource.WCS) >=0) ||
			            (protocol.toUpperCase().indexOf(Resource.POSTGIS) >=0) ||
			            (protocol.toUpperCase().indexOf(Resource.WEBSITE) >=0) ||
			            (protocol.toUpperCase().indexOf(Resource.DOWNLOAD) >=0)){
			        getMapButton().setEnabled(true);
			        return;
			    }
			}
        }	
    	
    } 

/**
 * 
 * 
 */
    public void setDefaultButtonListeners() {        
        nextButton = controlsPanel.getNextButton();
        lastButton = controlsPanel.getLastButton();
        descriptionButton = controlsPanel.getDescriptionButton();
        mapButton = controlsPanel.getMapButton();
        closeButton = controlsPanel.getCloseButton();
        nextButton.addActionListener(this);
        lastButton.addActionListener(this);
        descriptionButton.addActionListener(this);
        mapButton.addActionListener(this);
        closeButton.addActionListener(this);
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JButton getDescriptionButton() {        
        return descriptionButton;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JButton getMapButton() {        
        return mapButton;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JButton getCloseButton() {        
        return closeButton;
    } 

/**
 * 
 * 
 * 
 * @param firstRecord 
 */
    public void createNewSearch(int firstRecord) {        
        java.util.Collection nodesRecords = new java.util.ArrayList();
        nodesRecords = client.getLnkICatalogServerDriver().getRecords(client.getUrl(),
                null, firstRecord);
        this.nodes = nodesRecords;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public int getCurrentNode() {        
        if ((currentRecord % 10) == 0) {
            return 10;
        } else {
            return (currentRecord % 10);
        }
    } 

/**
 * 
 * 
 * 
 * @param e 
 */
    public void actionPerformed(ActionEvent e) {        
        if (e.getActionCommand().equals("next")) {
            nextButtonActionPerformed();
        } 
        if (e.getActionCommand().equals("last")) {
            lastButtonActionPerformed();
        } 
        if (e.getActionCommand().equals("description")) {
            descriptionButtonActionPerformed();
        } 
        if (e.getActionCommand().equals("layer")) {
            mapButtonActionPerformed();
        } 
        if (e.getActionCommand().equals("close")) {
            closeButtonActionPerformed();
        }
       
    } 

/**
 * 
 * 
 */
    public void nextButtonActionPerformed() {        
        this.currentRecord = this.currentRecord + 1;
        if (this.currentRecord == XMLTreeNumberOfRecordsAnswer.getNumberOfRecords(
                    (XMLNode)nodes.toArray()[0])) {
            nextButton.setEnabled(false);
        } else {
            nextButton.setEnabled(true);
        }
        lastButton.setEnabled(true);
        if ((this.currentRecord % 10) == 1) {
            createNewSearch(currentRecord);
        }
        controlsPanel.loadTextNewRecord((XMLNode)nodes.toArray()[getCurrentNode()],
            client.getProtocol());
        controlsPanel.actualizaLabel(currentRecord);
        
        enableMapButton();
    } 

/**
 * 
 * 
 */
    public void lastButtonActionPerformed() {        
        this.currentRecord = this.currentRecord - 1;
        if (this.currentRecord == 1) {
            lastButton.setEnabled(false);
        } else {
            lastButton.setEnabled(true);
        }
        nextButton.setEnabled(true);
        if ((this.currentRecord % 10) == 0) {
            createNewSearch(currentRecord - 10 + 1);
        }
        controlsPanel.loadTextNewRecord((XMLNode)nodes.toArray()[getCurrentNode()],
            client.getProtocol());
        controlsPanel.actualizaLabel(currentRecord);
        
        enableMapButton();
    } 

/**
 * 
 * 
 */
    public void descriptionButtonActionPerformed() {        
    	XMLNode node = (XMLNode)nodes.toArray()[getCurrentNode()];
    	if (node == null){
    		//Impossible to parse
    	}else{
    		new ShowTreeDialog(node);
    	}
    } 

/**
 * 
 * 
 */
    public void mapButtonActionPerformed() {        
        Resource[] resources = controlsPanel.getTags().getResources();
        Collection col = new java.util.ArrayList();
        for (int i=0 ; i<resources.length ; i++){
            col.add(resources[i]);
        }
        new ChooseResourceDialog(col);
               
    } 

/**
 * 
 * 
 */
    public void closeButtonActionPerformed() {        
        parent.setVisible(false);
    } 

/**
 * 
 * 
 * 
 * @param parent The parent to set.
 */
    public void setParent(JDialog parent) {        
        this.parent = parent;
    } 
 }
