
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Collection;

import com.iver.utiles.swing.jcomboServer.ServerData;
import es.gva.cit.gazetteer.adl.drivers.ADLGazetteerServiceDriver;
import es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver;
import es.gva.cit.gazetteer.idec.drivers.IDECGazetteerServiceDriver;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.wfsg.drivers.WFSGazetteerServiceDriver;

/**
 * This class represents a gazetteer client. It must be created to
 * use the gazetteer service
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class GazetteerClient {

/**
 * 
 * 
 */
    private URL url = null;

/**
 * 
 * 
 */
    private String sURL = null;

/**
 * 
 * 
 */
    private String protocol = null;
/**
 * 
 * 
 */
    private IGazetteerServiceDriver lnkIGazetteerDriver;

/**
 * 
 * 
 */
    private String serverStatus = null;

/**
 * 
 * 
 * 
 * @param url 
 * @param protocol 
 */
    public  GazetteerClient(String url, String protocol) {        
        setProtocol(protocol);
        setSURL(url);
        
        try {
            setUrl(new URL(url));
            
                      
            if (getUrl().getPort() == -1) {
                setUrl(new URL("http", getUrl().getHost(), 80, getUrl().getFile()));
            } else {
                setUrl(new URL("http", getUrl().getHost(), getUrl().getPort(),getUrl().getFile()));
            }
            
            
            
        } catch (MalformedURLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
     
        //Casting to the respective driver
        if (protocol.equals(new String(ServerData.SERVER_SUBTYPE_GAZETTEER_WFSG)) ||
                protocol.equals(new String(ServerData.SERVER_SUBTYPE_GAZETTEER_WFS))) {
            lnkIGazetteerDriver = new WFSGazetteerServiceDriver();
        }
        if (protocol.equals(new String(ServerData.SERVER_SUBTYPE_GAZETTEER_ADL))) {
            lnkIGazetteerDriver = new ADLGazetteerServiceDriver();
        }
        
        if (protocol.equals(new String(ServerData.SERVER_SUBTYPE_GAZETTEER_IDEC))) {
            lnkIGazetteerDriver = new IDECGazetteerServiceDriver();
        }
        
      
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public boolean getCapabilities() {        
           if (!serverReady()) {
               serverStatus = ("errorServerNotFound");
           } else if (!getLnkIGazetteerDriver().isProtocolSupported(getUrl())) {
               serverStatus = "errorNotSupportedProtocol";
           } else {
               //getCapabilities
               Collection nodesCapabilities = getLnkIGazetteerDriver()
                                                 .getCapabilities(getUrl());
               if (nodesCapabilities == null) {
                   serverStatus = "errorNotSupportedCapabilities";
               } else {
                   //Configure the client
                   if (!getLnkIGazetteerDriver().setParameters(nodesCapabilities)) {
                       if (!(getLnkIGazetteerDriver()
                                        .getServerAnswerReady().equals(""))) {
                           serverStatus = getLnkIGazetteerDriver().getServerAnswerReady();
                       } else {
                           serverStatus = "errorNotParsedReply";
                       }
                   } else {
                       serverStatus = getLnkIGazetteerDriver().getServerAnswerReady();
                       return true;
                   }
               }
           }
           
           return false;           
    } 

/**
 * This method implements the getFeature operation
 * 
 * 
 * @return 
 * @param url 
 * @param query 
 */
    public Collection getFeature(URL url, Query query) {        
            return lnkIGazetteerDriver.getFeature(url, query);
    } 

/**
 * 
 * 
 * 
 * @return Returns the lnkIGazetteerDriver.
 */
    public IGazetteerServiceDriver getLnkIGazetteerDriver() {        
        return lnkIGazetteerDriver;
    } 

/**
 * 
 * 
 * 
 * @param lnkIGazetteerDriver The lnkIGazetteerDriver to set.
 */
    public void setLnkICatalogServerDriver(IGazetteerServiceDriver lnkIGazetteerDriver) {        
        this.lnkIGazetteerDriver = lnkIGazetteerDriver;
    } 

/**
 * This method is used to create a new Query
 * 
 * 
 * @return 
 */
    public Query createNewQuery() {        
        return new Query();   
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public String getProtocol() {        
        return protocol;
    } 

/**
 * 
 * 
 * 
 * @param protocol The protocol to set.
 */
    public void setProtocol(String protocol) {        
        this.protocol = protocol;
    } 

/**
 * 
 * 
 * 
 * @return Returns the url.
 */
    public URL getUrl() {        
        return url;
    } 

/**
 * 
 * 
 * 
 * @param url The url to set.
 */
    public void setUrl(URL url) {        
        this.url = url;
    } 

/**
 * It tries if the server is ready
 * 
 * 
 * @return boolean
 * true --> server is ready
 * false --> server is not ready
 */
    public boolean serverReady() {        
        try {
            new Socket(this.getUrl().getHost(),
                    this.getUrl().getPort());
        } catch (UnknownHostException e) {
            // TODO Auto-generated catch block
            return false;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            return false;
        }
        return true;
    } 

/**
 * 
 * 
 * 
 * @return Returns the sURL.
 */
    public String getSURL() {        
        return sURL;
    } 

/**
 * 
 * 
 * 
 * @param surl The sURL to set.
 */
    public void setSURL(String surl) {        
        sURL = surl;
    } 

/**
 * 
 * 
 * 
 * @return Returns the serverStatus.
 */
    public String getServerStatus() {        
            return serverStatus;
    } 
 }
