
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.drivers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import java.net.URL;
import java.util.Collection;
import java.util.Vector;
/**
 * This interface contains the common methods that have to be
 * implemented by all the gazetteer drivers.
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public interface IGazetteerServiceDriver {
/**
 * It have to prove if the specified protocol is supported.
 * 
 * 
 * @return true if is supported, false if it isn't supported
 * @param url Server URL
 */
    public boolean isProtocolSupported(URL url);
/**
 * It try to discover the server capabilities.
 * 
 * 
 * @return Node with the server answer.
 * @param url Server URL
 */
    public Collection getCapabilities(URL url);
/**
 * It returns feature properties
 * 
 * 
 * @return Node with the server answer
 * @param url Server URL
 * @param feature Feature name
 */
    public Collection describeFeatureType(URL url, String feature);
/**
 * It returns the records finded
 * 
 * 
 * @return 
 * @param url Server URL
 * @param query Query with the search parameters
 */
    public Collection getFeature(URL url, Query query);
/**
 * This method is used to update the client parameters
 * 
 * 
 * @return true if the it's possible to update the server
 * @param nodes Node with the parameters to update
 */
    public boolean setParameters(Collection nodes);
/**
 * It parses the getFeature opearation
 * 
 * 
 * @return Array of features
 * @param node XMLNode
 * @param thesaurus The typename selected
 * @param attribute The attribute to do the search
 */
    public Feature[] parseFeatures(XMLNode node, ThesaurusName thesaurus, String attribute);
/**
 * It return a message to write in the server connection frame
 * 
 * 
 * @return It is/isn't the supported protocol
 */
    public String getServerAnswerReady();
/**
 * This function returns a vector with the features retrieved from
 * the get Capabilities file
 * 
 * 
 * @return A vector
 */
    public Vector getVectorFeatures();
}


