
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.wfsg.drivers;
import es.gva.cit.gazetteer.drivers.IProtocolMessages;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.wfsg.filters.WFSGFilter;
import java.net.URL;
import org.apache.commons.httpclient.NameValuePair;

/**
 * This class has methods that return the WFSG opearions messages.
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class WFSGMessages implements IProtocolMessages {
/**
 * 
 * 
 */
    private WFSGazetteerServiceDriver driver;

/**
 * 
 * 
 */
    private URL url;

/**
 * 
 * 
 * 
 * @param driver 
 * @param url 
 */
    public  WFSGMessages(WFSGazetteerServiceDriver driver, URL url) {        
        this.driver = driver;
        this.url = url;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getHTTPGETCapabilities(boolean)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param upper 
 */
    public NameValuePair[] getHTTPGETCapabilities(boolean upper) {        
        NameValuePair nvp1;
        
        if (upper){
            nvp1 = new NameValuePair("request", "GetCapabilities");
        }else{
            nvp1 = new NameValuePair("request", "getCapabilities");
        }
        
          return new NameValuePair[] { nvp1 };
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getHTTPGETDescribeFeature()
     */

/**
 * 
 * 
 * 
 * @return 
 * @param feature 
 */
    public NameValuePair[] getHTTPGETDescribeFeature(String feature) {        
        NameValuePair nvp1 = new NameValuePair("request", "DescribeFeatureType");
        NameValuePair nvp2 = new NameValuePair("typename", feature);
         
        return new NameValuePair[] {
            nvp1, nvp2
        };
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getHTTPGETFeatures(es.gva.cit.catalogClient.querys.Query, int)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param query 
 * @param firstRecord 
 */
    public NameValuePair[] getHTTPGETFeatures(Query query, int firstRecord) {        
        
        NameValuePair nvp1 = new NameValuePair("request", "getFeature");
        NameValuePair nvp2 = new NameValuePair("typename", new WFSGFilter().getQuery(query));
         
        return new NameValuePair[] {
            nvp1, nvp2
        };
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getHTTPPOSTCapabilities()
     */

/**
 * 
 * 
 * 
 * @return 
 */
    public String getHTTPPOSTCapabilities() {        
        return "<?xml version=\"1.0\" ?>" +
     	"<GetCapabilities " +
     	"service=\"WFS\" " +
     	"xmlns:csw=\"http://www.opengis.net/wfs\" " +
     	"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
			"xsi:schemaLocation=\"http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.0.0/WFS-basic.xsd\">" +
     	"</GetCapabilities>";
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getHTTPPOSTDescribeFeature()
     */

/**
 * 
 * 
 * 
 * @return 
 * @param feature 
 */
    public String getHTTPPOSTDescribeFeature(String feature) {        
        return "<DescribeFeatureType " +
			"version=\"1.0.0\" " +
			"service=\"WFS\" " +
			"xmlns=\"http://www.opengis.net/wfs\" " +
			"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
			"xsi:schemaLocation=\"http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.0.0/WFS-basic.xsd\">" +
			"<TypeName>" + feature + "</TypeName>" +
			"</DescribeFeatureType>"; 	   
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getHTTPPOSTFeature(es.gva.cit.catalogClient.querys.Query, int)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param query 
 * @param firstRecord 
 */
    public String getHTTPPOSTFeature(Query query, int firstRecord) {        
        
        
     String message = "<wfs:GetFeature service=\"WFS\" version=\"1.0.0\" " +
      	"outputFormat=\"GML2\" " +
      	"xmlns:topp=\"http://www.openplans.org/topp\" " +
      	"xmlns:wfs=\"http://www.opengis.net/wfs\" " +
      	"xmlns:ogc=\"http://www.opengis.net/ogc\" " +
      	"xmlns:gml=\"http://www.opengis.net/gml\" " +
      	"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
      	"xsi:schemaLocation=\"http://www.opengis.net/wfs " +
      	"http://schemas.opengis.net/wfs/1.0.0/WFS-basic.xsd\">" +
      	"<wfs:Query typeName=\"" + query.getFeatures()[0].getName() + "\">" +
      	    new WFSGFilter().getQuery(query) +
      	"</wfs:Query>" +
      	"</wfs:GetFeature>"; 
       
        /* String message = "<GetFeature " +
      		"xmlns:wfs=\"http://www.opengis.net/wfs\" " +
      		"xmlns:ogc=\"http://www.opengis.net/ogc\" " +
      		"xmlns:cv300=\"http://sercartlin:8080/geoserver/wfs/DescribeFeatureType?typeName=cv300:hidro_txt_300k,cv300:poblacion_pnt_300k,cv300:poblacion50_pol_300k,cv300:orogsecun_txt_300k,cv300:orogsecun_pnt_300k,cv300:comunic_txt_300k,cv300:comunic_lin_300k,cv300:hidro_lin_300k,cv300:embalses_pol_300k,cv300:poblacion50_txt_300k,cv300:orogpral_txt_300k,cv300:limites_pol_300k\" " +
      		"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
      		"xsi:schemaLocation=\"http://www.opengis.net/wfs ../wfs/1.1.0/WFS.xsd\" " +
      		"version=\"1.1.0\" " +
      		"service=\"WFS\" " +
      		"outputFormat=\"text/xml\">" +
      		"<Query typename=\"" + query.getFeature().getName() + "\">" +
      		wfsQuery.getQuery() + 
      		"</Query>" +
      		"</GetFeature>";         	
     */
         
         /*String message = "<wfs-g:GetFeatureRequest " +
         	"xmlns:wfs-g=\"http://www.opengis.net/wfs-g\" " +
         	"xmlns:ogc=\"http://www.opengis.net/ogc\" " +
         	"xmlns:gml=\"http://www.opengis.net/gml\" " +
         	"xmlns:wfs=\"http://www.opengis.net/wfs\" " +
         	"xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
         	"xsi:schemaLocation=\"http://www.opengis.net/wfs-g ../../gazetteer/0.9/WFS-G_GetFeatureRequest.xsd\" " +
         	"version=\"1.0.0\" service=\"WFS\">" +
         	"<wfs:Query typename=\"" + query.getFeature().getName() + "\">" +
         	wfsQuery.getQuery() + 
         	"</wfs:Query>" +
         	"</wfs-g:GetFeatureRequest>";         	
        */
         
        /* String message = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>" +
                "<GetFeature outputFormat=\"GML2\" " +
         		"xmlns:gml=\"http://www.opengis.net/gml\" " +
         		"xmlns:wfs=\"http://www.opengis.net/wfs\" " +
         		"xmlns:ogc=\"http://www.opengis.net/ogc\">" +
         		"<Query typeName=\"" + query.getFeature().getName() + "\">" +
         		"<ogc:Filter>" + 
         			"<ogc:And>" +
    				"<ogc:BBOX>" +
    				 "<ogc:PropertyName>the_geom</ogc:PropertyName>" +
    			        "<gml:Box srsName=\"http://www.opengis.net/gml/srs/epsg.xml#4326\">" +
    			           "<gml:coordinates>-75.102613,40.212597 -72.361859,41.512517</gml:coordinates>" +
    			        "</gml:Box>" +
    				"</ogc:BBOX>" +
    				"<ogc:PropertyIsLike wildCard=\"%\" singleChar=\"?\" escape=\"/\">" +
    					"<ogc:PropertyName>geographicIdentifier</ogc:PropertyName>" +
    					"<ogc:Literal>Bo%</ogc:Literal>" +
    				"</ogc:PropertyIsLike>" +
    			"</ogc:And>" +
    		   "</ogc:Filter>" +
         		"</Query>" +
         		"</GetFeature>";
         	*/	
         System.out.println(message);
             
         return message;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getSOAPCapabilities()
     */

/**
 * 
 * 
 * 
 * @return 
 */
    public String getSOAPCapabilities() {        
        // TODO Auto-generated method stub
        return null;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getSOAPDescribeFeature()
     */

/**
 * 
 * 
 * 
 * @return 
 * @param feature 
 */
    public String getSOAPDescribeFeature(String feature) {        
        // TODO Auto-generated method stub
        return null;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.messages.IProtocolMessages#getSOAPFeature(es.gva.cit.catalogClient.querys.Query, int)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param query 
 * @param firstRecord 
 */
    public String getSOAPFeature(Query query, int firstRecord) {        
        // TODO Auto-generated method stub
        return null;
    } 
 }
