
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.wfsg.drivers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.protocols.HTTPGetProtocol;
import es.gva.cit.catalogClient.protocols.HTTPPostProtocol;
import es.gva.cit.gazetteer.drivers.AsbtractGazetteerServiceDriver;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import es.gva.cit.gazetteer.wfsg.parsers.WfsgCapabilitiesParser;
import es.gva.cit.gazetteer.wfsg.parsers.WfsgDescribeFeatureTypeParser;
import es.gva.cit.gazetteer.wfsg.parsers.WfsgFeatureParser;
import es.gva.cit.gazetteer.wfsg.parsers.WfsgProtocolsOperations;
import java.net.URL;
import java.util.Collection;

/**
 * Driver for the WFS-Gazetteer protocol
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 * @see https://portal.opengeospatial.org/files/?artifact_id=7175
 * @see https://portal.opengeospatial.org/files/?artifact_id=8339
 */
public class WFSGazetteerServiceDriver extends AsbtractGazetteerServiceDriver {
/**
 * 
 * 
 */
    private WfsgProtocolsOperations operations = null;

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public Collection getCapabilities(URL url) {        
        Collection nodes = new java.util.ArrayList();
        WFSGMessages messages = new WFSGMessages(this,url);
        System.out.println("**************GET*************");
        nodes = new HTTPGetProtocol().doQuery(url, messages.getHTTPGETCapabilities(true), 0);
        if (nodes == null){ 
            nodes = new HTTPGetProtocol().doQuery(url, messages.getHTTPGETCapabilities(false), 0);
        }
        
        if (nodes == null) {
            System.out.println("**************POST*************");
            nodes = new HTTPPostProtocol().doQuery(url,
                    messages.getHTTPPOSTCapabilities(), 0);
        }
         
      return nodes;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param query 
 */
    public Collection getFeature(URL url, Query query) {        
         Collection nodes = new java.util.ArrayList();
         WFSGMessages messages = new WFSGMessages(this,url);
         setQuery(query);
         
         for (int i=0 ; i<getOperations().getGetFeature().length; i++){
             if (getOperations().getGetFeature()[i].equals("SOAP")){
        		 /*
                 System.out.println("**************SOAP*************");
        		 nodes = new SOAPProtocol().doQuery(url,
                        getSOAPMessageRecords(getQuery(), 0), 0);
        		 
        		 setCommunicationProtocol("SOAP");
                 //We must cut the SOAP Head
                 nodes[0] = SOAPMessageParser.cutHead(nodes[0]);
        		 break;
        		 */
                 }
            if (getOperations().getGetFeature()[i].equals("GET")){
             	/*System.out.println("**************GET*************");
             	nodes = new HTTPGetProtocol().doQuery(url,
                         getMessageGetFeature(getQuery()), 0);
             	             	      	
                 break;
                 */ 
             }   
            if (getOperations().getGetFeature()[i].equals("POST")){
            	nodes = new HTTPPostProtocol().doQuery(url,
            	        messages.getHTTPPOSTFeature(getQuery(),0),0);
            	 		
                break;
            }
            
         }
         return nodes;
         
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param feature 
 */
    public Collection describeFeatureType(URL url, String feature) {        
        Collection nodes = new java.util.ArrayList();
        WFSGMessages messages = new WFSGMessages(this,url);
                
        for (int i=0 ; i<getOperations().getDescribeFeatureType().length; i++){
            
            if (getOperations().getDescribeFeatureType()[i].equals("SOAP")){
       		 /*
                System.out.println("**************SOAP*************");
       		 nodes = new SOAPProtocol().doQuery(url,
                       getSOAPMessageRecords(getQuery(), 0), 0);
       		 
       		 setCommunicationProtocol("SOAP");
                //We must cut the SOAP Head
                nodes[0] = SOAPMessageParser.cutHead(nodes[0]);
       		 break;
       		 */
       	}
           if (getOperations().getDescribeFeatureType()[i].equals("GET")){
            	/*
               System.out.println("**************GET*************");
            	nodes = new HTTPGetProtocol().doQuery(url,
                        getMessageDescribeFeatureType(feature), 0);
            	             	      	
                break;
                 */
            }  
           if (getOperations().getDescribeFeatureType()[i].equals("POST")){
           	
               System.out.println("**************POST*************");
               System.out.println(messages.getHTTPPOSTDescribeFeature(feature));      
               nodes = new HTTPPostProtocol().doQuery(url,
                             messages.getHTTPPOSTDescribeFeature(feature), 0);
           	   break;
           }           
        }
        return nodes;
    } 
/* (non-Javadoc)
	 * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#isProtocolSupported(java.net.URL)
	 */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public boolean isProtocolSupported(URL url) {        
	    // TODO Auto-generated method stub
	    return true;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#setParameters(org.w3c.dom.Node[])
     */

/**
 * 
 * 
 * 
 * @return 
 * @param nodes 
 */
    public boolean setParameters(Collection nodes) {        
        return new WfsgCapabilitiesParser(this).parse((XMLNode)nodes.toArray()[0]);
    } 

/**
 * 
 * 
 * 
 * @return Returns the operations.
 */
    public WfsgProtocolsOperations getOperations() {        
        return operations;
    } 

/**
 * 
 * 
 * 
 * @param operations The operations to set.
 */
    public void setOperations(WfsgProtocolsOperations operations) {        
        this.operations = operations;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#parseFeatures(org.w3c.dom.Node)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param node 
 * @param currentThesaurus 
 * @param attribute 
 */
    public Feature[] parseFeatures(XMLNode node, ThesaurusName currentThesaurus, String attribute) {        
        return new WfsgFeatureParser().parse(node,currentThesaurus,attribute);
    } 
 }
