
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.wfsg.parsers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import java.awt.geom.Point2D;

/**
 * This class is used to parse the getFeature request
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class WfsgFeatureParser {

/**
 * 
 * 
 */
    private String geomType = null;

/**
 * It parses the answer
 * 
 * 
 * @return Array of features
 * @param node XML tree that contains the getFeature Answer
 * @param thesaurus FEature selected in the thesaurus list
 * @param attribute Attribute to do the search
 */
    public Feature[] parse(XMLNode node, ThesaurusName thesaurus, String attribute) {        
        XMLNode[] nodeFeatures = XMLTree.searchMultipleNode(node,"gml:featureMember");
        
       String prefix = thesaurus.getName().split(":")[0];
          String geomField = getGeomField(prefix,thesaurus);
            
        if ((node.getName().equals("ResultCollection")) ||
        	(node.getName().equals("feature_collection"))){
            return parseWFS(nodeFeatures,thesaurus,"geographicIdentifier",geomField);
        }else{
            return parseWFS(nodeFeatures,thesaurus,prefix + ":" + attribute,geomField);
        }  
    } 

/**
 * It parses the a WFS answer
 * 
 * 
 * @return Array of features
 * @param nodeFeatures XML tree that contains the Features
 * @param thesaurusName FEature selected in the thesaurus list
 * @param attribute Attribute to do the search
 * @param geomField Field that contains the geometry
 */
    public Feature[] parseWFS(XMLNode[] nodeFeatures, ThesaurusName thesaurusName, String attribute, String geomField) {        
        Feature[] features = new Feature[nodeFeatures.length];
                
        for (int i=0 ; i<nodeFeatures.length ; i++){
            String id = XMLTree.searchNodeAtribute(nodeFeatures[i],thesaurusName.getName(),"fid");
            
            XMLNode nodeName = XMLTree.searchNode(nodeFeatures[i],thesaurusName.getName() + "->" + attribute);
            String name = "";
            
            if (nodeName != null){
                if ((nodeName.getText() != null) && (!(nodeName.getText().equals("")))){
                    name = nodeName.getText();
                } else if ((nodeName.getCdata() != null) && (!(nodeName.getCdata().equals("")))){
                    name = nodeName.getCdata();                   
                }
            }           
            String description = name;
            Point2D point = getCoordinates(XMLTree.searchNodeValue(nodeFeatures[i],
                    geomField));
              
            features[i] = new Feature(id,name,description,point);
        }
        
        return features;
    } 
//cv300:entity:layer:level:elevation:color:text:clave:the_geom->gml:MultiLineString->gml:lineStringMember->gml:LineString->gml:coordinates

/**
 * If returns the Geom route of the XML tree
 * 
 * 
 * @return 
 * @param prefix Typename
 * @param thesaurus Feature selected in the thesaurus list
 */
    private String getGeomField(String prefix, ThesaurusName thesaurus) {        
        if ((thesaurus.getFields() == null) || (thesaurus.getFields().length == 0)){
            geomType = "Point";  
            return thesaurus.getName() + "->" + "position->gml:Point->gml:coordinates";
        }    
        
        for (int i=0 ; i<thesaurus.getFields().length ; i++){
            if (thesaurus.getFields()[i].getType().equals("gml:MultiLineStringPropertyType")){
                prefix = thesaurus.getName() + "->" + prefix + ":" + thesaurus.getFields()[i].getName();
                geomType = "MultiLine";
                return prefix +
                "->gml:MultiLineString->gml:lineStringMember->gml:LineString->" +
                "gml:coordinates";
            }
            
            if (thesaurus.getFields()[i].getType().equals("gml:MultiPolygonPropertyType")){
                prefix = thesaurus.getName() + "->" + prefix + ":" + thesaurus.getFields()[i].getName();
                geomType = "MultiPolygon";
                return prefix +
                "->gml:MultiPolygon->gml:polygonMember->gml:Polygon->gml:outerBoundaryIs->" +
                "gml:LinearRing->gml:coordinates";
            }
            
            if (thesaurus.getFields()[i].getType().equals("gml:PointPropertyType")){
                prefix = thesaurus.getName() + "->" + prefix + ":" + thesaurus.getFields()[i].getName();
                geomType = "Point";
                return prefix +
                "->gml:Point->gml:coordinates";
            }           
        }        
      return "";
    } 

/**
 * It returns a pair of coordinates of the Feature
 * 
 * 
 * @return 
 * @param sCoordinates String that contains the coordinates
 */
    private Point2D getCoordinates(String sCoordinates) {        
        if (geomType.equals("MultiLine")){
            return getCoordinatesMultiline(sCoordinates);
        }
        if (geomType.equals("MultiPolygon")){
            return getCoordinatesMultiPolygon(sCoordinates);
        }
        if (geomType.equals("Point")){
            return getCoordinatesPoint(sCoordinates);
        }
        return null;
    } 

/**
 * It returns a pair of coordinates from a Multiline string
 * 
 * 
 * @return 
 * @param sCoordinates String
 */
    private java.awt.geom.Point2D.Double getCoordinatesMultiline(String sCoordinates) {        
        return getAverage(sCoordinates.split(" "));      
    } 

/**
 * It returns a pair of coordinates from a MultiPolygon string
 * 
 * 
 * @return 
 * @param sCoordinates String
 */
    private java.awt.geom.Point2D.Double getCoordinatesMultiPolygon(String sCoordinates) {        
         return getAverage(sCoordinates.split(" "));
    } 

/**
 * It returns a pair of coordinates from a Point
 * 
 * 
 * @return 
 * @param sCoordinates String
 */
    private java.awt.geom.Point2D.Double getCoordinatesPoint(String sCoordinates) {        
          return getAverage(sCoordinates.split(" "));
    } 

/**
 * It returns the average of a set of coordinates
 * 
 * 
 * @return Average
 * @param sCoordinates Coordinates: X,Y
 */
    private java.awt.geom.Point2D.Double getAverage(String[] sCoordinates) {        
         double x =         0.0;
         double y =         0.0;
         for (int i=0 ; i<sCoordinates.length ; i++){
             x = x + Double.parseDouble(sCoordinates[i].split(",")[0]);
             y = y + Double.parseDouble(sCoordinates[i].split(",")[1]);
         }
         x = x / sCoordinates.length;
         y = y / sCoordinates.length;
         return new Point2D.Double(x,y);
    } 
 }
