package com.iver.cit.gvsig;

import java.math.BigDecimal;
import java.sql.Types;
import java.util.BitSet;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.project.documents.table.gui.Statistics;
import com.iver.cit.gvsig.project.documents.table.gui.Table;

/**
 * @author Fernando Gonzlez Corts
 */
public class TableNumericFieldOperations extends Extension{

	/**
	 * @see com.iver.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#execute(java.lang.String)
	 */
	public void execute(String actionCommand) {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v != null) {
			if (v.getClass() == Table.class) {

				Table table = (Table) v;

				doExecute(table);
			}
		}
	}
	/**
	 * "execute" method acction
	 * @param actionCommand
	 * The acction command that executes this method
	 * @param table
	 * Table to operate
	 */

	protected void doExecute(Table table){
		int fieldIndex = table.getSelectedFieldIndices().nextSetBit(0);

		SelectableDataSource ds=null;
		try {
			ds = (SelectableDataSource)table.getModel().getModelo().getRecordset();
		} catch (DriverLoadException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		BitSet selectedRows = (BitSet)ds.getSelection().clone();
		try {
			int rc = (int) ds.getRowCount();
			//Si no hay seleccin se usan todos los registros
			if (selectedRows.cardinality() == 0){
				selectedRows.set(0, rc);
			}else{
				rc = selectedRows.cardinality();
			}
			BigDecimal suma = new BigDecimal(0);
			BigDecimal min = new BigDecimal(Double.MAX_VALUE);
			BigDecimal max = new BigDecimal(Double.MIN_VALUE);
			for(int i=selectedRows.nextSetBit(0); i>=0; i=selectedRows.nextSetBit(i+1)) {
				double d = ((NumericValue) ds.getFieldValue(i, fieldIndex)).doubleValue();
				suma = suma.add(new BigDecimal(d));
				if (d < min.doubleValue()) min = new BigDecimal(d);
				if (d > max.doubleValue()) max = new BigDecimal(d);
			}
			BigDecimal media = suma.divide(new BigDecimal(rc), BigDecimal.ROUND_HALF_DOWN);
			BigDecimal varianza = new BigDecimal(0);
			for(int i=selectedRows.nextSetBit(0); i>=0; i=selectedRows.nextSetBit(i+1)) {
				double d = ((NumericValue) ds.getFieldValue(i, fieldIndex)).doubleValue();
				BigDecimal dif = new BigDecimal(d).subtract(media);
				varianza = dif.multiply(dif).add(varianza);
			}
			varianza = varianza.divide(new BigDecimal(rc), BigDecimal.ROUND_HALF_DOWN);
			double desviacion = Math.sqrt(varianza.doubleValue());

			Statistics st = new Statistics();
			st.setStatistics(media.doubleValue(), max.doubleValue(), min.doubleValue(), varianza.doubleValue(), desviacion, rc, new BigDecimal(max.doubleValue()).subtract(min).doubleValue(), suma.doubleValue());
			PluginServices.getMDIManager().addWindow(st);

		} catch (DriverException e) {
			NotificationManager.addError("No se pudo acceder a los datos", e);
		}
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v == null) {
			return false;
		}

		if (v instanceof Table) {
			Table table = (Table) v;
			return doIsEnabled(table);
		}
		return false;
	}

	protected boolean doIsEnabled(Table table){
		try{
			BitSet indices = table.getSelectedFieldIndices();

			// System.out.println("TableNumericFieldOperations.isEnabled: Tabla: " + table.getModel().getModelo().getRecordset().getName() );

			if (indices.cardinality() == 1){
				try {
					int type = table.getModel().getModelo().getRecordset().getFieldType(indices.nextSetBit(0));
					if ((type == Types.BIGINT) ||
							(type == Types.DECIMAL) ||
							(type == Types.DOUBLE) ||
							(type == Types.FLOAT) ||
							(type == Types.INTEGER) ||
							(type == Types.SMALLINT) ||
							(type == Types.TINYINT) ||
							(type == Types.REAL) ||
							(type == Types.NUMERIC)){
						return true;
					}
				} catch (DriverException e) {
					return false;
				}
			}
		}  catch (DriverLoadException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		return false;
	}


	/**
	 * @see com.iver.andami.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v == null) {
			return false;
		}

		if (v instanceof Table) {
			return true;
		}
		return false;
	}

}
