package com.iver.cit.gvsig.project.documents.view;

import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;

import org.cresques.cts.IProjection;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ErrorEvent;
import com.iver.cit.gvsig.fmap.ErrorListener;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.documents.ProjectDocument;
import com.iver.cit.gvsig.project.documents.exceptions.OpenException;
import com.iver.cit.gvsig.project.documents.table.ProjectTable;
import com.iver.cit.gvsig.project.documents.table.ProjectTableFactory;
import com.iver.cit.gvsig.project.documents.view.info.gui.HTMLInfoToolPanel;
import com.iver.utiles.XMLEntity;

public abstract class ProjectViewBase extends ProjectDocument implements ErrorListener,
		IProjectView {

	protected MapContext mapOverViewContext;
	protected MapContext mapContext;
	protected int m_typeLink = 0;
	protected String m_extLink;
	protected String m_selectedField = null;

	// OVERRIDE THESE
	public IWindow createWindow() {	return null;}
	public IWindow getProperties() { return null;}

	/**
	 * Gets the FMap's contexts of the main map in the view.
	 *
	 * @return
	 */
	public MapContext getMapContext() {
		return mapContext;
	}

	/**
	 * Gets the FMap's context from the locator, which is the
	 * small map in the left-bottom corner of the View.
	 *
	 * @return
	 */
	public MapContext getMapOverViewContext() {
		return mapOverViewContext;
	}

	/**
	 * @see com.iver.cit.gvsig.project.documents.view.ProjectView#setMapContext(com.iver.cit.gvsig.fmap.MapContext)
	 */
	public void setMapContext(MapContext fmap) {
		mapContext = fmap;
		fmap.addErrorListener(this);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param fmap DOCUMENT ME!
	 */
	public void setMapOverViewContext(MapContext fmap) {
		mapOverViewContext = fmap;
		mapOverViewContext.setProjection(mapContext.getProjection());
	}

	public void setXMLEntity(XMLEntity xml, Project p) throws XMLException,
			DriverException, DriverIOException, OpenException {	}

	public void setXMLEntity03(XMLEntity xml, Project p) throws XMLException,
			DriverException, DriverIOException { }

	public void showErrors(){
		if (mapContext.getLayersError().size()>0){
			String layersError="";
			for (int i=0;i<mapContext.getLayersError().size();i++){
				layersError=layersError+"\n"+(String)mapContext.getLayersError().get(i);
			}
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this,"fallo_capas")+" : \n"+
					layersError);
		}
	}
	
	/**
	 * Reports to the user a bundle of driver exceptions produced in the
	 * same atomic MapContext transaction
	 */
	public void reportDriverExceptions(String introductoryText, List driverExceptions) {
		HtmlWindow htmlPanel = new HtmlWindow(570, 450, "Excepciones de driver");
		String htmlText = null;
		if(introductoryText == null){
			htmlText += "<p>Se han producido los siguientes errores durante la carga de capas.</p>";
		}else{
			htmlText += introductoryText;
		}
		int numErrors = driverExceptions.size();
		for(int i = 0; i < numErrors; i++){
			htmlText += "<br>\n";
			DriverException exception = (DriverException) driverExceptions.get(i);
			htmlText += exception.getExceptionType().getHtmlErrorMessage();
		}
		System.out.println(htmlText);
		htmlPanel.show(htmlText);
		
		PluginServices.getMDIManager().addCentredWindow(htmlPanel);
		
	}
	
	/**
	 * HtmlInfoToolPanel that implements IWindow
	 * @author azabala
	 *
	 */
	class HtmlWindow extends HTMLInfoToolPanel implements IWindow {
		WindowInfo viewInfo = null;
		public HtmlWindow(int width, int height, String windowTitle){
			viewInfo = new WindowInfo(WindowInfo.MODELESSDIALOG);
			viewInfo.setTitle(windowTitle);
			viewInfo.setWidth(width);
			viewInfo.setHeight(height);
		}
		
		public WindowInfo getWindowInfo() {
			return viewInfo;
		}
		
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getProjection() {
		return mapContext.getProjection();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getOverViewProjection() {
		return mapOverViewContext.getProjection();
	}

	public void setProjection (IProjection proj) {
		mapContext.setProjection(proj);
		mapOverViewContext.setProjection(proj);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getExtLink() {
		return m_extLink;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public int getTypeLink() {
		return m_typeLink;
	}

	/**
     * Se selecciona la extensin para realizar cuando se quiera el link.
     *
     * @param s nombre del campo.
     */
	public void setExtLink(String s) {
		m_extLink = s;
	}

	/**
     * Se selecciona el tipo de fichero para realizar cuando se quiera el link.
     *
     * @param i tipo de fichero.
     */
	public void setTypeLink(int i) {
		m_typeLink = i;
	}

	public void afterRemove() {
		FLayers layers=this.getMapContext().getLayers();

		for (int i = layers.getLayersCount()-1; i>=0; i--){
        	try {
        		if (layers.getLayer(i) instanceof AlphanumericData){
                    Project project = ((ProjectExtension)PluginServices.getExtension(ProjectExtension.class)).getProject();
                    ProjectTable pt = project.getTable((AlphanumericData) layers.getLayer(i));

                    ArrayList tables = project.getDocumentsByType(ProjectTableFactory.registerName);
                    for (int j = 0; j < tables.size(); j++) {
                        if (tables.get(j) == pt){
                            project.delDocument((ProjectDocument)tables.get(j));
                            break;
                        }
                    }

                    PluginServices.getMDIManager().closeSingletonWindow(pt);
                }
                layers.getLayer(i).getParentLayer().removeLayer(layers.getLayer(i));
			} catch (CancelationException e1) {
    			e1.printStackTrace();
    		}
    	}

	}

	public void afterAdd() {
		// TODO Auto-generated method stub

	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param c DOCUMENT ME!
	 */
	public void setBackColor(Color c) {
//		getMapContext().getViewPort().addViewPortListener(getMapContext()
	//													   .getEventBuffer());
		getMapContext().getViewPort().setBackColor(c);
		//getMapContext().getViewPort().removeViewPortListener(getMapContext()
			//												  .getEventBuffer());
	}

	/**
	 * Se selecciona el nombre del campo para realizar cuando se quiera el
	 * link.
	 *
	 * @param s nombre del campo.
	 */
	public void setSelectedField(String s) {
		m_selectedField = s;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSelectedField() {
		return m_selectedField;
	}

	public void errorThrown(ErrorEvent e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this,"fallo_capas")+" : \n"+
					e.getMessage());

	}

}
