/*
 * Created on 23-mar-2006
 * 
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 */
package com.iver.cit.gvsig.project.documents.view.toc.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.JMenuItem;

import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.InfoPanel;
import org.cresques.ui.raster.RasterTransparencyPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.RasterAdapter;
import com.iver.cit.gvsig.gui.panels.IRasterPropertiesRegistrable;
import com.iver.cit.gvsig.gui.panels.PropertiesRasterDialog;
import com.iver.cit.gvsig.project.documents.view.toc.TocMenuEntry;


/**
 * Entrada en del men contextual del TOC correspondiente al cuadro de 
 * propiedades del raster
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class FLyrRasterAdjustPropertiesTocMenuEntry extends TocMenuEntry {
	private JMenuItem 						properties;
	private FLayer							lyr = null;
	private PropertiesRasterDialog			propsDialog = null;
	private BandSetupPanel 					bandSetup = null;
	private ArrayList						listeners = new ArrayList();
	private RasterFilterStackManager		stackManager = null;
	private BrightnessContrastTocMenuEntry	bcMenuEntry = null;
	
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas raster
    		if ((lyr instanceof FLyrRaster)) {
    			properties = new JMenuItem(PluginServices.getText(this, "propiedades_raster"));
    			getMenu().add( properties );
    			properties.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    			properties.addActionListener(this);
    			stackManager = new RasterFilterStackManager(((FLyrRaster)lyr).getSource().getFilterStack());
    			if(bcMenuEntry == null)
    				bcMenuEntry = new BrightnessContrastTocMenuEntry();
     		}
		}
	}
	
	/**
	 * Obtiene la instancia del cuadro de propiedades
	 * @return PropertiesRasterDialog
	 */
	public PropertiesRasterDialog getPropertiesDialog(){
		return propsDialog;
	}
	
	/**
	 * Los objetos que quieren que se ejecute su listerner deben ser
	 * registrados a traves de este mtodo. Cuando se ejecute el actionPerformed
	 * se buscar todos los objetos registrados y ejecutar el actionPerformed de
	 * cada uno,
	 * @param obj Objeto a registrar
	 */
	public void register(Object obj){
		if(obj instanceof IRasterPropertiesRegistrable)
			listeners.add(obj);
	}

	/**
	 * Obtiene el gestor de la pila de filtros
	 * @return RasterFilterStackManager
	 */
	public RasterFilterStackManager getStackManager() {
		return stackManager;
	}
	
	/**
	 * Gestiona la apertura del dialogo de propiedades de raster cuando se pulsa
	 * la opcin asignando a este las propiedades iniciales. 
	 */
	public void actionPerformed(ActionEvent e) {
       	lyr = getNodeLayer();
                   
       	if(lyr instanceof FLyrRaster){
       		stackManager = new RasterFilterStackManager(((FLyrRaster)lyr).getSource().getFilterStack());
       		propsDialog = new PropertiesRasterDialog(((FLyrRaster)lyr), stackManager.getTransparencyRGB());
       		
       		//Aadimos los paneles de las ventanas registradas
       		for(int i=0;i<listeners.size();i++){
				if(listeners.get(i) instanceof IRasterPropertiesRegistrable){
					((IRasterPropertiesRegistrable)listeners.get(i)).setStackManager(stackManager);
					((IRasterPropertiesRegistrable)listeners.get(i)).addTabPanel(this);
				}
			}
       					
       		FilterRasterDialogPanel filterPanel = ((FilterRasterDialogPanel)propsDialog.getContentPane());
       	       			
			int alpha = 255-((FLyrRaster)lyr).getSource().getTransparency();
			
			bandSetup = (BandSetupPanel)filterPanel.getPanelByClassName("BandSetupPanel");
			
			//Asignacin del alpha actual de la imagen al dialogo
			RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)filterPanel.getPanelByClassName("RasterTransparencyPanel");
			rasterTrans.setOpacity(alpha);
			
			propsDialog.setRasterFilterStackManager(stackManager);
			
			ArrayList stackList = stackManager.getStringsFromStack();
			String hideBands = null;
			for(int i=0;i<stackList.size();i++){
				if(((String)stackList.get(i)).startsWith("filter.removebands.bands"))
					hideBands = RasterFilterStackManager.getValue((String)stackList.get(i));
			}
			
			GeoRasterFile[] files = ((FLyrRaster)lyr).getSource().getFiles();
			propsDialog.addFiles(files);
			for(int i=0; i< files.length;i++)
				propsDialog.addNumBands(files[i].getBandCount());
			
			RasterAdapter rad = ((FLyrRaster)lyr).getSource();
			int posR = 0, posG = 0, posB = 0;
			
			for(int i=0;i<rad.getPosFile(GeoRasterFile.RED_BAND);i++)
				posR += files[i].getBandCount();
			posR += rad.getBand(GeoRasterFile.RED_BAND);
			
			for(int i=0;i<rad.getPosFile(GeoRasterFile.GREEN_BAND);i++)
				posG += files[i].getBandCount();
			posG += rad.getBand(GeoRasterFile.GREEN_BAND);
			
			for(int i=0;i<rad.getPosFile(GeoRasterFile.BLUE_BAND);i++)
				posB += files[i].getBandCount();
			posB += rad.getBand(GeoRasterFile.BLUE_BAND);
						
			bandSetup.assignBand(posR, GeoRasterFile.RED_BAND);
			bandSetup.assignBand(posG, GeoRasterFile.GREEN_BAND);
			bandSetup.assignBand(posB, GeoRasterFile.BLUE_BAND);	
			
			InfoPanel pInfo = (InfoPanel)this.propsDialog.getPanelByClassName("InfoPanel");
			pInfo.setBands(posR, posG, posB);
			pInfo.refresh();
			
			//Seleccin de 1, 2 o 3 bandas para visualizar
			if(hideBands != null){
				int pos = 2;
				if(hideBands.length() == 1)
					pos = 1;
				else if(hideBands.length() == 2)
					pos = 0;
				bandSetup.getFileList().getJComboBox().setSelectedIndex(pos);
				
				//Reseteamos los controles de la tabla
				
				for(int i=0;i<bandSetup.getRGBTable().getModel().getRowCount();i++)
					for(int j=0;j<3;j++)
						bandSetup.getRGBTable().getModel().setValueAt(new Boolean(false), i, j);
				
				if(hideBands.equals("GB") || hideBands.equals("G") || hideBands.equals("B") )
					bandSetup.getRGBTable().getModel().setValueAt(new Boolean(true), posR, 0);
				
				if(hideBands.equals("RB") || hideBands.equals("R") || hideBands.equals("B") )
					bandSetup.getRGBTable().getModel().setValueAt(new Boolean(true), posG, 1);
				
				if(hideBands.equals("RG") || hideBands.equals("R") || hideBands.equals("G") )
					bandSetup.getRGBTable().getModel().setValueAt(new Boolean(true), posB, 2);
			}
			
			propsDialog.readStat();
						
			//Ejecuta el listener de todos los que se han registrado
			for(int i=0;i<listeners.size();i++){
				if(listeners.get(i) instanceof IRasterPropertiesRegistrable)
					((IRasterPropertiesRegistrable)listeners.get(i)).actionPerformed(stackManager, propsDialog, null);
			}
			
			bcMenuEntry.actionPerformed(stackManager, propsDialog, null);
			
       		PluginServices.getMDIManager().addWindow(propsDialog);
        }
	}

}
