/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.listeners;

import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;

import org.cresques.px.Extent;
import org.gvsig.georeferencing.toc.GeoRasterTocMenuEntry;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.project.documents.view.gui.View;

/**
* Clase que maneja los eventos del frame que contiene los paneles de georreferenciacin.
* @author Nacho Brodin (brodin_ign@gva.es)
*/
public class GeoRasterFrameListener implements InternalFrameListener {
		
	/**
	 * This is the default constructor
	 */
	public GeoRasterFrameListener() {
		super();
	}
	
	/**
	 * Cuando cerramos la ventana de georreferenciacin se elimina la barra de herramientas
	 * y se destruye la capa de puntos.
	 */
	public void internalFrameClosing(InternalFrameEvent arg0) {
		//Desbloqueamos la entrada de men para que se pueda abrir otro cuadro
		GeoRasterTocMenuEntry.isOpen = true;
		
		//Anulamos todos los cambios cargando el extent de origen a la capa 
		//Cambiamos el nombre a la capa de georreferenciacin y eliminamos la capa de puntos.

		View theView = null;
		try{
			theView = (View) PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return;
		}
		
		theView.getMapControl().setTool("zoomIn");
		
		//((GeoRasterTocMenuEntry)FPopupMenu.getEntry("GeoRasterTocMenuEntry")).getMenu().setEnabled(true);
		
		for(int i=0;i<theView.getMapControl().getMapContext().getLayers().getLayersCount();i++){
			FLayer lyr = theView.getMapControl().getMapContext().getLayers().getLayer(i);
			
			if(	lyr instanceof FLyrGeoRaster && 
				lyr.getName().startsWith("*")){
				
				//Recuperamos el estado inicial de la capa de puntos
				
				((FLyrGeoRaster)lyr).getFLyrPoints().recoveryState();
				
				((FLyrGeoRaster)lyr).getFLyrPoints().setVisible(false);
				
				lyr.setName(lyr.getName().substring(1, lyr.getName().length()));
				lyr.setActive(false);
				
				//Reponemos el extent inicial
				Extent initExtent = ((FLyrGeoRaster)lyr).getStackZoom().getInitExtent();
				if(initExtent != null){
					((FLyrGeoRaster)theView.getMapControl().getMapContext().getLayers().getLayer(lyr.getName())).setAssignExtent(initExtent);
					theView.getMapControl().getMapContext().invalidate();
				}		
				
				//Dejamos de pintar la capa de puntos
				((FLyrGeoRaster)lyr).getFLyrPoints().setVisible(false);
			}			
		}
		
		PluginServices.getMainFrame().enableControls();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameActivated(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameActivated(InternalFrameEvent arg0) {

	}
	
	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameClosed(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameClosed(InternalFrameEvent arg0) {

	}

	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameDeactivated(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameDeactivated(InternalFrameEvent arg0) {

	}
	
	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameDeiconified(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameDeiconified(InternalFrameEvent arg0) {

	}
	
	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameIconified(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameIconified(InternalFrameEvent arg0) {

	}
	
	/* (non-Javadoc)
	 * @see javax.swing.event.InternalFrameListener#internalFrameOpened(javax.swing.event.InternalFrameEvent)
	 */
	public void internalFrameOpened(InternalFrameEvent arg0) {

	}
}

