/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.options;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

/**
 * Panel que contiene opciones de georreferenciacin a elegir por
 * el usuario. 
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class TableOptions extends JScrollPane{
	
	private JTable 					table = null;
	private TableModelPoint 		tabModel = null;
	final static String[] 			columnNames = {"", ""};
	
	public TableModelPoint getTableModel(){
		return tabModel;
	}
	
    public TableOptions() {
        tabModel = new TableModelPoint();
        table = new JTable(tabModel);
		setSize(380, 160);
		table.setSize(380, 160);
        table.setPreferredScrollableViewportSize(new Dimension(380, 160));
                      
        TableColumn column = null;
        for (int i = 0; i < table.getModel().getColumnCount(); i++) {
        	column = table.getColumnModel().getColumn(i);
        	if(i == 0){
                column.setCellRenderer(new CheckColumnRenderer());
                column.setCellEditor(new CheckColumnEditor());
                column.setMaxWidth(19);
                column.setMinWidth(19);
        	}
        }
        
        add(table);
    }
    
    /**
     * Introduce una entrada de la tabla
     * @param active Valor para el checkBox
     * @param text Texto 
     */
    public void setEntry(boolean active, String text){
    	tabModel.setEntry(active, text);
    }
    
    /**
     * TableModel del JTable que contiene los puntos con sus errores
     * @author Nacho Brodin (brodin_ign@gva.es)
     */
    public class TableModelPoint extends DefaultTableModel {
    	 
        public TableModelPoint(){
        	super(new Object[0][2], columnNames);
        }
        
        public int getColumnCount() {
            return columnNames.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Class getColumnClass(int c) {
        	if (c == 0)
        		return Boolean.class;
        	return String.class;
        }

        public void addNew() {
            super.addRow(new Object[] {new Boolean(true), ""});
        }
        
        public void setValueAt(Object value, int row, int col) { 
        	super.setValueAt(value, row, col);
        }
        
        /**
         * Introduce una entrada de la tabla
         * @param active Valor para el checkBox
         * @param text Texto 
         */
        public void setEntry(boolean active, String text){
        	super.addRow(new Object[] {new Boolean(active), text});
        }
        
        /**
         * Devuelve true si la celda es editable. Solo son editables
         * las celdas de los puntos.
         */
        public boolean isCellEditable(int row, int col){
        	return false;
        }
    }
    
    /**
     * 
     * @author Nacho Brodin (brodin_ign@gva.es)
     */
    class CheckColumnRenderer extends JCheckBox implements TableCellRenderer {
        final private static long serialVersionUID = -3370601314380922368L;

        public Component getTableCellRendererComponent(JTable table,
                                                       Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row, int column) {
            if (value == null) {
                this.setSelected(false);
            }

            Boolean ValueAsBoolean = (Boolean) value;
            this.setSelected(ValueAsBoolean.booleanValue());
            this.setHorizontalAlignment(SwingConstants.CENTER);

            return this;
        }
    }
    
    /**
     * 
     * @author Nacho Brodin (brodin_ign@gva.es)
     */
    class CheckColumnEditor extends AbstractCellEditor implements TableCellEditor {
    	final private static long serialVersionUID = -3370601314380922368L;
    	public JCheckBox checkBox;

        public CheckColumnEditor() {
            super();
            checkBox = new JCheckBox();
            checkBox.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                    	 //checkBox.setSelected(!checkBox.isSelected());
                        fireEditingStopped();
                    }
                });
        }

        public Component getTableCellEditorComponent(JTable table, Object obj,
                                                     boolean isSelected,
                                                     int row, int col) {
        	checkBox.setHorizontalAlignment(SwingUtilities.CENTER);

            Boolean lValueAsBoolean = (Boolean) obj;
            checkBox.setSelected(lValueAsBoolean.booleanValue());

            return checkBox;
        }

        public Object getCellEditorValue() {
            return new Boolean(checkBox.isSelected());
        }
    }
}
