/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.gvsig.rasterTools.pansharpening.ui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.DataBuffer;
import java.io.File;
import java.util.Vector;

import javax.swing.AbstractCellEditor;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.cresques.filter.IRasterOperationsRegistrablePanels;
import org.cresques.filter.IStackManager;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;
import org.cresques.ui.raster.FilterRasterDialogPanel;
import org.cresques.ui.raster.IResize;
import org.cresques.ui.raster.RasterTransparencyPanel;
import org.gvsig.rasterTools.pansharpening.filter.PanSharpeningStackManager;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.gui.panels.IRasterPropertiesRegistrable;
import com.iver.cit.gvsig.gui.panels.PropertiesRasterDialog;
import com.iver.cit.gvsig.project.documents.view.toc.gui.FLyrRasterAdjustPropertiesTocMenuEntry;

/**
 * Selecciona las bandas visibles en un raster. Contiene una tabla con una fila por cada
 * banda de la imagen. Por medio de checkbox se selecciona para cada RGB que banda de la 
 * imagen ser visualizada. 
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class PanSharpeningPanel extends JPanel implements TableModelListener,
                                                      KeyListener, ChangeListener,
                                                      ActionListener,
                                                      IRasterPropertiesRegistrable,
                                                      IRasterOperationsRegistrablePanels,
                                                      IResize{
    final private static long 		serialVersionUID = -3370601314380922368L;

    private final static String[] 	columnNames = { " ", "Band" };

    
    private int						wComp = 445, hComp = 239;
    private int						wNorth = wComp - 5, hNorth = hComp -14;
    private int						wCheck = wNorth - 100, hCheck = 30;
    private int						wSouth = wNorth - 30, hSouth = 62;
    private int						wPSliders = wSouth - 10, hPSliders = 30;
    private int						wTable = wNorth - 40, hTable = hNorth - 165;
    private int						wSlider = wPSliders - 220, hSlider = 16;
    
    /**
     * Nombre del panel
     */
    private String 						nom = "Pansharp";
    private JTable 						rgbTable = null;
    private JScrollPane 				rgbBandAssignPane = null;
    RGBBandAsignTableModel 				tableModel = null;
    private int 						sizeX = 445;
    private int 						sizeY = 174;
	private JPanel						activationPanel = null;
	private JCheckBox 					cbActiveSharpening = null;
	private JPanel 						pSharpNorth = null;
	private JPanel 						pSharpSouth = null;
	private JRadioButton 				rbBrovey = null;
	private JRadioButton 				rbHSL = null;
	private JPanel 						pHSL = null;
	private JSlider 					jSlider = null;
	private JTextField 					jTextField = null;
	private JPanel 						pBrovey = null;
	private JSlider 					slBrovey = null;
	private JTextField 					jTextField1 = null;
	private PanSharpeningStackManager	pStackManager = null;
	
    /**
     * This method initializes
     *
     */
    public PanSharpeningPanel() {
        super();
        initialize();
        this.setTranslation();
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    void initialize() {
        this.setPreferredSize(new Dimension(sizeX, sizeY));
        this.setLayout(new FlowLayout());
        this.setLocation(0, 0);
        this.setSize(wComp, hComp);
        this.add(getPSharpNorth(), null);
        this.setTableEnabled(false);
    }

    /**
     * Obtiene el nombre del panel
     * @return Cadena con el nombre del panel
     */
    public String getName(){
    	return this.nom;
    }
    
    /**
     * Activa o desactiva la tabla de seleccin de banda de refinado
     * @param enable
     */
    public void setTableEnabled(boolean enabled){
    	this.getRGBBandAssignPane().setEnabled(enabled);
        this.getRGBTable().setEnabled(enabled);
        this.getJSlider().setEnabled(enabled);
        this.getJSlider1().setEnabled(enabled);
        this.getJTextField().setEnabled(enabled);
        this.getJTextField1().setEnabled(enabled);
        this.getRbBrovey().setEnabled(enabled);
        this.getRbHSL().setEnabled(enabled);
    }
    
    /**
     * Traducciones del panel
     */
    public void setTranslation(){
    	this.getCbActiveSharpening().setText(PluginServices.getText(this,"onSharpening"));
		this.getPSharpNorth().setBorder(javax.swing.BorderFactory.createTitledBorder(null, PluginServices.getText(this,"selectBandaRefinado"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
    }
    
    /**
     * This method initializes jTable
     *
     * @return javax.swing.JTable
     */
    private JScrollPane getRGBBandAssignPane() {
        if (rgbBandAssignPane == null) {
            rgbBandAssignPane = new JScrollPane(getRGBTable());
            rgbBandAssignPane.setPreferredSize(new java.awt.Dimension(wTable,hTable));
            rgbBandAssignPane.setEnabled(true);
            TableColumn column = null;

            for (int i = 0; i < 1; i++) {
                column = rgbTable.getColumnModel().getColumn(i);
                column.setCellRenderer(new RadioColumnRenderer());
                column.setCellEditor(new RadioColumnEditor());
                column.setMaxWidth(22);
                column.setMinWidth(22);
            }
        }
        return rgbBandAssignPane;
    }

    /**
     * Obtiene la Tabla
     * @return Tabla de bandas de la imagen
     */
    public JTable getRGBTable() {
        if (rgbTable == null) {
            tableModel = new RGBBandAsignTableModel();
            tableModel.addTableModelListener(this);
            rgbTable = new JTable(tableModel);
            rgbTable.setPreferredScrollableViewportSize(new Dimension(300, 60));
        }
        return rgbTable;
    }


    /**
     * Aade una banda a la tabla  bandas de la imagen asignandole un
     * nombre y valor a los checkbox
     * @param bandName        Nombre de la banda
     */
    private void addBand(String bandName) {
        Vector v = new Vector();
        v.add(new Boolean(false));
        v.add(bandName);
        ((DefaultTableModel) rgbTable.getModel()).addRow(v);
    }

    /**
     * Obtiene el nmero de bandas de la lista
     * @return
     */
    public int getNBands() {
        return ((DefaultTableModel) rgbTable.getModel()).getRowCount();
    }

    /**
     * Obtiene el nombre de la banda de la posicin i de la tabla
     * @param i
     * @return
     */
    public String getBandName(int i) {
        String s = (String) ((DefaultTableModel) rgbTable.getModel()).getValueAt(i, 1);
        return s.substring(s.lastIndexOf("[8U]") + 5, s.length());
    }

    /**
     * Selecciona una banda de la tabla.
     * @param i
     */
    public void selectRow(int i){
    	((DefaultTableModel) rgbTable.getModel()).setValueAt(new Boolean(true),i,0);
    }
    
    /**
     * Elimina todas las filas seleccionadas
     */
    public void removeSelections(){
    	for(int i=0;i<((DefaultTableModel) rgbTable.getModel()).getRowCount();i++){
    		((DefaultTableModel) rgbTable.getModel()).setValueAt(new Boolean(false),i,0);
    	}
    }
    
    /* (non-Javadoc)
     * @see javax.swing.event.TableModelListener#tableChanged(javax.swing.event.TableModelEvent)
     */
    public void tableChanged(TableModelEvent e) {
        rgbTable.revalidate();
        rgbBandAssignPane.revalidate();
        revalidate();
    }

    /* (non-Javadoc)
     * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
     */
    public void keyPressed(KeyEvent arg0) {
        // TODO Auto-generated method stub
    }

    /* (non-Javadoc)
     * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
     */
    public void keyReleased(KeyEvent arg0) {
        // TODO Auto-generated method stub
    }

    /* (non-Javadoc)
     * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
     */
    public void keyTyped(KeyEvent arg0) {
        // TODO Auto-generated method stub
    }

    /**
     * 
     */
    public void actionPerformed(ActionEvent e) {
        if(e.getSource() == this.cbActiveSharpening){
        	if(this.cbActiveSharpening.isSelected()){
        		this.setTableEnabled(true);
        	}else{
        		this.setTableEnabled(false);
        	}
        }
    }

    class RadioColumnEditor extends AbstractCellEditor
        implements TableCellEditor {
        final private static long serialVersionUID = -3370601314380922368L;
        public JRadioButton theRadioButton;

        public RadioColumnEditor() {
            super();
            theRadioButton = new JRadioButton();
            theRadioButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        fireEditingStopped();
                    }
                });
        }

        public Component getTableCellEditorComponent(JTable table, Object obj,
                                                     boolean isSelected,
                                                     int row, int col) {
            theRadioButton.setHorizontalAlignment(SwingUtilities.CENTER);

            Boolean lValueAsBoolean = (Boolean) obj;
            theRadioButton.setSelected(lValueAsBoolean.booleanValue());

            return theRadioButton;
        }

        public Object getCellEditorValue() {
            return new Boolean(theRadioButton.isSelected());
        }
    }

    class RadioColumnRenderer extends JRadioButton implements TableCellRenderer {
        final private static long serialVersionUID = -3370601314380922368L;

        public Component getTableCellRendererComponent(JTable table,
                                                       Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row, int column) {
            if (value == null) {
                this.setSelected(false);
            }

            Boolean ValueAsBoolean = (Boolean) value;
            this.setSelected(ValueAsBoolean.booleanValue());
            this.setHorizontalAlignment(SwingConstants.CENTER);

            return this;
        }
    }

    class RGBBandAsignTableModel extends DefaultTableModel {
        final private static long serialVersionUID = -3370601314380922368L;

        public RGBBandAsignTableModel() {
            super(new Object[0][2], columnNames);
        }

        public Class getColumnClass(int c) {
            if (c < 1) {
                return Boolean.class;
            }

            return String.class;
        }

        public void setValueAt(Object value, int row, int col) {
            if ((col < 1) && ((Boolean) value).booleanValue()) {
                for (int i = 0; i < getRowCount(); i++) {
                    if (i != row) {
                        setValueAt(new Boolean(false), i, col);
                    }
                }
            }

            super.setValueAt(value, row, col);
        }

        public void addNew() {
            super.addRow(new Object[] {
                             new Boolean(false), ""
                         });
        }
    }
    
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getActivationPanel() {
		if (activationPanel == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout.setVgap(0);
			flowLayout.setHgap(0);
			activationPanel = new JPanel();
			activationPanel.setLayout(flowLayout);
			activationPanel.setPreferredSize(new java.awt.Dimension(wCheck,hCheck));
			activationPanel.add(getCbActiveSharpening(), null);
		}
		return activationPanel;
	}
	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */    
	public JCheckBox getCbActiveSharpening() {
		if (cbActiveSharpening == null) {
			cbActiveSharpening = new JCheckBox();
			cbActiveSharpening.setText("Activar Pansharpening");
			cbActiveSharpening.setPreferredSize(new java.awt.Dimension(150,23));
			cbActiveSharpening.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
			cbActiveSharpening.addActionListener(this);
		}
		return cbActiveSharpening;
	}
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	public JPanel getPSharpNorth() {
		if (pSharpNorth == null) {
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.gridx = 0;
			gridBagConstraints10.gridy = 1;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			pSharpNorth = new JPanel();
			pSharpNorth.setLayout(new GridBagLayout());
			pSharpNorth.setPreferredSize(new java.awt.Dimension(wNorth,hNorth));
			pSharpNorth.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Selector de banda de refinado", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
			gridBagConstraints2.gridx = 0;
			gridBagConstraints2.gridy = 0;
			gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
			gridBagConstraints2.gridwidth = 0;
			gridBagConstraints2.gridheight = 1;
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.gridy = 2;
			gridBagConstraints3.weightx = 0.0D;
			gridBagConstraints3.weighty = 0.0D;
			gridBagConstraints3.fill = java.awt.GridBagConstraints.NONE;
			gridBagConstraints3.ipadx = 0;
			gridBagConstraints3.ipady = 0;
			gridBagConstraints3.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints3.gridheight = 0;
			gridBagConstraints3.gridwidth = 0;
			pSharpNorth.add(getActivationPanel(), gridBagConstraints2);
			pSharpNorth.add(getRGBBandAssignPane(), gridBagConstraints3);
			pSharpNorth.add(getPSharpSouth(), gridBagConstraints10);
		}
		return pSharpNorth;
	}
   
	private JPanel getPSharpSouth() {
		if (pSharpSouth == null) {
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			pSharpSouth = new JPanel();
			pSharpSouth.setLayout(new GridBagLayout());
			pSharpSouth.setPreferredSize(new java.awt.Dimension(wSouth,hSouth));
			gridBagConstraints9.gridy = 1;
			gridBagConstraints8.gridy = 0;
			pSharpSouth.add(getJPanel(), gridBagConstraints8);
			ButtonGroup group = new ButtonGroup();
			group.add(getRbBrovey());
			group.add(getRbHSL());
			gridBagConstraints9.ipadx = 0;
			pSharpSouth.add(getPKernel(), gridBagConstraints9);
		}
		return pSharpSouth;
	}
	/**
	 * This method initializes jRadioButton	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	public JRadioButton getRbBrovey() {
		if (rbBrovey == null) {
			rbBrovey = new JRadioButton();
			rbBrovey.setText("Brovey");
			rbBrovey.setPreferredSize(new java.awt.Dimension(70,23));
			rbBrovey.setSelected(true);
		}
		return rbBrovey;
	}
	/**
	 * This method initializes jRadioButton	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */    
	public JRadioButton getRbHSL() {
		if (rbHSL == null) {
			rbHSL = new JRadioButton();
			rbHSL.setText("HSL");
			rbHSL.setPreferredSize(new java.awt.Dimension(70,23));
		}
		return rbHSL;
	}
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getPKernel() {
		if (pHSL == null) {
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			pHSL = new JPanel();
			pHSL.setLayout(new GridBagLayout());
			pHSL.setPreferredSize(new java.awt.Dimension(wPSliders,hPSliders));
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.gridy = 0;
			gridBagConstraints5.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints5.gridheight = 0;
			gridBagConstraints6.gridx = 1;
			gridBagConstraints6.gridwidth = 1;
			gridBagConstraints6.gridy = 0;
			gridBagConstraints6.weightx = 1.0;
			gridBagConstraints6.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints6.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints6.gridheight = 0;
			gridBagConstraints7.weightx = 1.0D;
			gridBagConstraints7.weighty = 1.0D;
			gridBagConstraints7.gridwidth = 1;
			gridBagConstraints7.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridheight = 1;
			gridBagConstraints7.gridy = 0;
			gridBagConstraints7.gridx = 2;
			pHSL.add(getJTextField(), gridBagConstraints7);
			pHSL.add(getRbHSL(), gridBagConstraints5);
			pHSL.add(getJSlider(), gridBagConstraints6);
			this.getJSlider().setValue(0);
		}
		return pHSL;
	}
	/**
	 * This method initializes jSlider	
	 * 	
	 * @return javax.swing.JSlider	
	 */    
	public JSlider getJSlider() {
		if (jSlider == null) {
			jSlider = new JSlider();
			jSlider.setMinimum(0);
			jSlider.setPreferredSize(new java.awt.Dimension(wSlider,hSlider));
			jSlider.setMaximum(100);
			jSlider.addChangeListener(this);
		}
		return jSlider;
	}
	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */    
	public JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setPreferredSize(new java.awt.Dimension(20,19));
			jTextField.setText("0.0");
		}
		return jTextField;
	}

	public void stateChanged(ChangeEvent e) {
		//Ponemos el valor del texto del coeficiente 
        if (e.getSource().equals(this.getJSlider())) {
            this.getJTextField().setText(String.valueOf((getJSlider().getValue()/200.0)));
        }
        
        if (e.getSource().equals(this.getJSlider1())) {
            this.getJTextField1().setText(String.valueOf((getJSlider1().getValue()/2)));
        }
		
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (pBrovey == null) {
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints11.gridx = 2;
			gridBagConstraints11.gridy = 0;
			gridBagConstraints11.weighty = 1.0D;
			gridBagConstraints11.weightx = 1.0;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
			gridBagConstraints1.gridheight = 0;
			gridBagConstraints1.gridx = 1;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.weightx = 1.0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints.gridy = 0;
			gridBagConstraints.gridheight = 0;
			gridBagConstraints.gridx = 0;
			pBrovey = new JPanel();
			pBrovey.setLayout(new GridBagLayout());
			pBrovey.setPreferredSize(new java.awt.Dimension(wPSliders,hPSliders));
			pBrovey.add(getRbBrovey(), gridBagConstraints);
			pBrovey.add(getJSlider1(), gridBagConstraints1);
			this.getJSlider1().setValue(0);
			pBrovey.add(getJTextField1(), gridBagConstraints11);
		}
		return pBrovey;
	}

	/**
	 * This method initializes jSlider1	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	public JSlider getJSlider1() {
		if (slBrovey == null) {
			slBrovey = new JSlider();
			slBrovey.setMaximum(100);
			slBrovey.setPreferredSize(new java.awt.Dimension(wSlider,hSlider));
			slBrovey.addChangeListener(this);
		}
		return slBrovey;
	}

	/**
	 * This method initializes jTextField1	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getJTextField1() {
		if (jTextField1 == null) {
			jTextField1 = new JTextField();
			jTextField1.setPreferredSize(new java.awt.Dimension(20,19));
			jTextField1.setText("0");
		}
		return jTextField1;
	}
	
	/**
	 * Asigna el gestor del filtro de pansharpening
	 * @param p PanSharpeningStackManager
	 */
	public void setPanSharpeningStackManager(PanSharpeningStackManager p){
		pStackManager = p;
	}
	
	//--------------------------------------------------------------
	//IRasterPropertiesRegistrable

	/**
	 * Mtodo que es ejecutado desde PropertiesRasterDialog cuando se pulsa "Aceptar"
	 * en todas los paneles que tiene aadidos.
	 * @param fLayer FLyrRaster
	 * @param propsDialog PropertiesRasterDialog 
	 */
	public void actionPerformed(RasterFilterStackManager stackManager,  PropertiesRasterDialog propsDialog, FLyrRaster fLayer){
		PanSharpeningPanel pSharp = (PanSharpeningPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("PanSharpeningPanel");
		if(pSharp.getCbActiveSharpening().isSelected()){
			String pancr = null;
				
			//Obtenemos el nombre de la banda seleccionada
			for(int i=0;i<(pSharp.getRGBTable()).getRowCount();i++){
				if(((Boolean) (pSharp.getRGBTable()).getValueAt(i, 0)).booleanValue())
					pancr = ((String) (pSharp.getRGBTable()).getValueAt(i, 1));
			}
				
			//Buscamos el GeoRasterFile que corresponde a la capa y aadimos el filtro con el
			if(pancr!=null){
				pancr = pancr.substring(pancr.lastIndexOf("] ") + 2, pancr.length());
				for(int i=0; i<fLayer.getSource().getFiles().length;i++){
				 if(fLayer.getSource().getFiles()[i].getName().endsWith(File.separator+pancr)){
				 	int[] order = {	propsDialog.getAssignedBand(GeoRasterFile.RED_BAND),
				 					propsDialog.getAssignedBand(GeoRasterFile.GREEN_BAND),
				 					propsDialog.getAssignedBand(GeoRasterFile.BLUE_BAND)};
					 	
				 	RasterTransparencyPanel tpan = (RasterTransparencyPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("RasterTransparencyPanel");
				 	int opac = Integer.parseInt(tpan.getOpacityText().getText());
				 	opac = (int)((opac*255)/100);
					 	
				 	String method = "brovey";
				 	if(pSharp.getRbHSL().isSelected())
				 		method = "hsl";
					 	
				 	double coef = 0D;
				 	if(pSharp.getRbHSL().isSelected())
				 		coef = Double.parseDouble(pSharp.getJTextField().getText());
					 					 	
				 	int coefBrovey = 0;
				 	if(pSharp.getRbBrovey().isSelected())
				 		coefBrovey = Integer.parseInt(pSharp.getJTextField1().getText());
					 					 	
				 	pStackManager.addSharpeningFilter(fLayer.getSource().getFiles(), i, order, opac, method, coef, coefBrovey);
				 }
				}
			}
		}else{
			stackManager.removeFilter(stackManager.getTypeFilter("sharpening"));
		}
		fLayer.getMapContext().invalidate();
	}

	public void addTabPanel(FLyrRasterAdjustPropertiesTocMenuEntry menu) {
		//Este mtodo no tiene funcionalidad para el panel		
	}

	public void setStackManager(RasterFilterStackManager stackManager) {
		//Este mtodo no tiene funcionalidad para el panel
	}
	
	public IStackManager getStackManager() {
		return pStackManager;
	}
	
	//--------------------------------------------------------------
	//IRasterOperationsRegistrablePanels
	
	
	public void setComponentSize(int w, int h){
		wComp = w; hComp = h;
	    wNorth = wComp - 5; hNorth = hComp -14;
	    wCheck = wNorth - 100; hCheck = 30;
	    wSouth = wNorth - 30; hSouth = 62;
	    wPSliders = wSouth - 10; hPSliders = 30;
	    wTable = wNorth - 40; hTable = hNorth - 165;
	    wSlider = wPSliders - 220; hSlider = 16;
	 
	    this.setSize(wComp, hComp);
	    pSharpNorth.setPreferredSize(new java.awt.Dimension(wNorth,hNorth));
	    activationPanel.setPreferredSize(new java.awt.Dimension(wCheck,hCheck));
	    rgbBandAssignPane.setPreferredSize(new java.awt.Dimension(wTable,hTable));
	    pSharpSouth.setPreferredSize(new java.awt.Dimension(wSouth,hSouth));
	    pBrovey.setPreferredSize(new java.awt.Dimension(wPSliders,hPSliders));
	    slBrovey.setPreferredSize(new java.awt.Dimension(wSlider,hSlider));
	    pHSL.setPreferredSize(new java.awt.Dimension(wPSliders,hPSliders));
	    jSlider.setPreferredSize(new java.awt.Dimension(wSlider,hSlider));
		
	    
	}
	
	
	/**
     * Mtodo del interfaz IRasterOperationsRegistrablePanels
     * Aade la lista de georasterfiles a la tabla
     * @param files lista de ficheros a aadir
     */
    public void addFiles(GeoRasterFile[] files) {
        for (int i = 0; i < files.length; i++) {
            String fName = files[i].getName();
      
            String bandName = new File(fName).getName();
            String bandType = "";

            switch (files[i].getDataType()) {
            case DataBuffer.TYPE_BYTE:
                bandType = "8U";
                break;
            case DataBuffer.TYPE_INT:
                bandType = "32";
                break;
            case DataBuffer.TYPE_DOUBLE:
                bandType = "64";
                break;
            case DataBuffer.TYPE_FLOAT:
                bandType = "32";
                break;
            case DataBuffer.TYPE_SHORT:
                bandType = "16";
                break;
            case DataBuffer.TYPE_USHORT:
                bandType = "16U";
                break;
            case DataBuffer.TYPE_UNDEFINED:
                bandType = "??";
                break;
            }

            if (files[i].getBandCount() > 1) {
                for (int b = 0; b < files[i].getBandCount(); b++)
                    addBand((b + 1) + " [" + bandType + "] " + bandName);
            } else {
                addBand("1 [" + bandType + "] " + bandName);
            }
        }
    }
    
    /**
     * Mtodo del interfaz IRasterOperationsRegistrablePanels
     * Elimina un fichero de la lista
     * @param file Nombre del fichero a eliminar
     */
    public void removeFile(String file) {
        for (int i = 0; i < ((DefaultTableModel) rgbTable.getModel()).getRowCount(); i++) {
            if (((String) ((DefaultTableModel) rgbTable.getModel()).getValueAt(i, 1)).endsWith(file)) {
                ((DefaultTableModel) rgbTable.getModel()).removeRow(i);
                i--; //Ojo! que hemos eliminado una fila
            }
        }
    }
		
} //  @jve:decl-index=0:visual-constraint="10,10"
