package org.gvsig.rasterTools.saveRaster.operations;

import java.io.File;
import java.io.IOException;

import org.cresques.io.GdalWriter;
import org.gvsig.rasterTools.saveRaster.ui.properties.ProgressSaveRasterDialog;

import com.iver.cit.gvsig.fmap.ViewPort;

import es.gva.cit.jgdal.Gdal;
import es.gva.cit.jgdal.GdalDriver;
import es.gva.cit.jgdal.GdalException;

/**
 * @author Nacho Brodin <brodin_ign@gva.es>
 * 
 * Esta clase copia un dataset desde disco en un formato soportado por Gdal
 * a otro especificado.
 */
class CopyDatasetThread extends Thread{
	
	private GdalDriver 					driver = null;
	private String 						fDstName = null;
	private String 						fSrcName = null;
	private ViewPort 					vp = null;
	private String[]					driverProps = null;
	private String 						extension = null;
	private double 						maxPercentBar = 0D;
	private boolean 					runCopy = false;
	private ProgressSaveRasterDialog	window = null;
	
	/**
	 * Constructor
	 * @param src	Nombre del fichero fuente
	 * @param dst	Nombre del fichero destino
	 * @param viewPort	Viewport
	 * @param dp	DriverProperties
	 * @param drvName	Nombre del driver de Gdal
	 * @param extens	Extensin del fichero
	 */
	public CopyDatasetThread(String src, String dst, ViewPort viewPort, String[] dp,
				String drvName, String extens, ProgressSaveRasterDialog window){
		fSrcName = src;			
		fDstName = dst;
		vp = viewPort;
		this.window = window;
		this.driverProps = dp;
		this.extension = extens;
		try{
			driver = Gdal.getDriverByName(drvName);
		}catch(GdalException exc){
		    System.err.println("No se ha podido obtener el driver.");
		}
		runCopy = true;
	}
	
	/**
	 * Funcin que realiza la copia del dataset
	 */
	public void copy(){
		try{	
			GdalWriter.createCopy(	driver,
									fDstName, 
									fSrcName, 
									false, 
									driverProps,
									vp.getProjection());
		}catch(IOException ev){
			ev.printStackTrace();
		}catch(GdalException ev){
			ev.printStackTrace();
		}			
		runCopy = false;
	}
	
	/**
	 * El thread maneja el incremento de la barra
	 */
	public void run(){
		File f = new File(fSrcName);
		long sizeSrc = f.length();
		long sizeDst = 0;
		if(extension.equals("jpg"))
			sizeDst = (long)(sizeSrc * 0.085);
		double incrProgressBar = 0;
		int last = 0;
		
		String osname = System.getProperty("os.name").toLowerCase();
		if(osname.indexOf("windows")>=0){
			//Temporalmente hacemos esto en windows 
			long l1 = System.currentTimeMillis();
			long l2 = 0;
			while(runCopy){
				if((l2 - l1) > 1000){
					l1 = System.currentTimeMillis();
					window.getJProgressBar().setValue(window.getJProgressBar().getValue() + 1);
				}
				l2 = System.currentTimeMillis();
				try{
					Thread.sleep(10);
				}catch(InterruptedException exc){}
			}
		}else{
			while(runCopy){
				File fdst = new File(fDstName);
				incrProgressBar = (fdst.length() * maxPercentBar) / sizeDst;
				if(fdst.exists() && incrProgressBar < maxPercentBar){
					if(last != ((int)incrProgressBar)){
						window.getJProgressBar().setValue(window.getJProgressBar().getValue() + 1);
						//System.out.println(fdst.length()+" "+maxPercentBar+" "+sizeDst+"**>"+incrProgressBar);
					}
					last=(int)incrProgressBar;
				}
				try{
					Thread.sleep(10);
				}catch(InterruptedException exc){}
			}
		}
		window.getJProgressBar().setValue(100);
	}
	
	/**
	 * Asigna el porcentaje mximo a la barra
	 * @param max	Porcentaje mximo de la barra
	 */
	public void setMaxPercentBar(double max){
		this.maxPercentBar = max;
	}
}