package org.gvsig.rasterTools.saveRaster.util;

public class MathUtils {
	public static double INCHESMTR = 39.73007874D;
	public static double INCHESCM = 0.3973007874D;
	public static double INCHESMM = 0.03973007874D;
	public static double MTRSINCH = 0.0254D;
	public static double MMSINCH = 25.4D;
	public static double CMSINCH = 2.54D;
	
	
	/**
     * Recorta el nmero de decimales a n del nmero pasado por parmetro
     * @return Nmero recortado
     */
    public static double tailDecimals(double num, int n){
    	int m = (int)Math.pow(10, n);
        num *= m;
        int aux = ((int)num);
        num = (double)((double)aux / (double)m);
        return num;
    }
    
	/**
	 * Convierte pixels en milimetros dependiendo del los puntos por pulgada
	 * @param pixels Nmero de pixels
	 * @param ppp Puntos por pulgada
	 * @return Nmero de milimetros
	 */
	public static double convertPixelsToMms(int pixels, int ppp){
		return convertPixelsToInches(pixels, ppp) * MMSINCH;
	}
	
	/**
	 * Convierte pixels en centimetros dependiendo del los metros por pixel
	 * @param pixels Nmero de pixels
	 * @param mtsPixel Metros por pixel
	 * @return Nmero de centimetros
	 */
	public static double convertPixelsToCms(int pixels, int ppp){
		return convertPixelsToInches(pixels, ppp) * CMSINCH;
	}
	
	/**
	 * Convierte pixels en metros dependiendo del los metros por pixel
	 * @param pixels Nmero de pixels
	 * @param mtsPixel Metros por pixel
	 * @return Nmero de metros
	 */
	public static double convertPixelsToMts(int pixels, int ppp){
		return convertPixelsToInches(pixels, ppp) * MTRSINCH;
	}
	
	/**
	 * Convierte pixels en pulgadas dependiendo del los metros por pixel
	 * @param pixels Nmero de pixels
	 * @param mtsPixel Metros por pixel
	 * @return Nmero de pulgadas
	 */
	public static double convertPixelsToInches(int pixels, int ppp){
		return (double)((double)pixels / (double)ppp);
	}
	
	/**
	 * Convierte metros en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de metros
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertMtsToPixels(double mts, int ppp){
		return (int)(mts * MathUtils.INCHESMTR * ppp);
	}
	
	/**
	 * Convierte pulgadas en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de pulgadas
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertInchesToPixels(double inches, int ppp){
		return (int)(inches * ppp);
	}
	
	/**
	 * Convierte centimetros en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de centmetros
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertCmsToPixels(double cms, int ppp){
		return (int)(cms * MathUtils.INCHESCM * ppp);
	}
	
	/**
	 * Convierte milimetros en pixels dependiendo de los puntos por pulgada
	 * @param mts Nmero de milimetros
	 * @param ppp Puntos por pulgada
	 * @return Nmero de pixeles
	 */
	public static int convertMmsToPixels(double mms, int ppp){
		return (int)(mms * MathUtils.INCHESMM * ppp);
	}
}
