/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.gui.toolListeners;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;

import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWCS;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PointListener;


/**
*
* @author Nacho Brodin (brodin_ign@gva.es)
*/
public class WCSZoomPixelCursorListener implements PointListener {
	private static Logger 	logger = Logger.getLogger(WCSZoomPixelCursorListener.class.getName());
	private final Image 	img = new ImageIcon(MapControl.class.getResource(
   									"images/ZoomPixelCursor.gif")).getImage();
	private Cursor 			cur = Toolkit.getDefaultToolkit().createCustomCursor(	img,
																					new Point(16, 16), "");
	private MapControl 		mapCtrl;

	/**
	 * Crea un nuevo AreaListenerImpl.
	 *
	 * @param mc DOCUMENT ME!
	 */
	public WCSZoomPixelCursorListener(MapControl mc) {
		this.mapCtrl = mc;
	}
	
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PointListener#point(com.iver.cit.gvsig.fmap.tools.Events.PointEvent)
	 */
	public void point(PointEvent event) throws BehaviorException {
		Point2D pReal = mapCtrl.getMapContext().getViewPort().toMapPoint(event.getPoint());
		Point imagePoint = new Point((int) event.getPoint().getX(), (int) event.getPoint().getY());
		ViewPort v = mapCtrl.getMapContext().getViewPort();
		
		FLayer[] actives = mapCtrl.getMapContext().getLayers().getActives();
		Rectangle2D ext = null;
		try{
			ext = actives[0].getFullExtent();
		} catch (com.iver.cit.gvsig.fmap.DriverException e1) {
			e1.printStackTrace();
		}
  		       
  		if(	ext != null ){

  	    	double w2 = v.getImageWidth()/2D;
  	    	double h2 = v.getImageHeight()/2D;
  	    	double wcOriginX = pReal.getX();
  	    	double wcOriginY = pReal.getY();

  	    	Point2D maxRes = ((FLyrWCS)actives[0]).getMaxResolution();
  	    	double wcDstMinX = wcOriginX-w2*maxRes.getX();
  	    	double wcDstMinY = wcOriginY-h2*maxRes.getY();
  	    	double wcDstWidth = w2*maxRes.getX()*2D;
  	    	double wcDstHeight = h2*maxRes.getY()*2D;
  	    
  	    	ext = new Rectangle2D.Double(wcDstMinX, wcDstMinY, wcDstWidth, wcDstHeight);
  	       	mapCtrl.getMapContext().getViewPort().setExtent(ext);
  		}

	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return cur;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
		return false;
	}

	public void pointDoubleClick(PointEvent event) throws BehaviorException {
		// TODO Auto-generated method stub
		
	}

}