package com.iver.cit.gvsig.gui.panels;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.DefaultListModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;
import org.gvsig.gui.beans.filterPanel.filterQueryPanel.FilterQueryJPanel;
import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLElement;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.ComplexValue;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.IWindowListener;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.gui.filter.ExpressionDataSource;
import com.iver.cit.gvsig.gui.filter.ExpressionListener;
import com.iver.cit.gvsig.gui.filter.FilterException;
import com.iver.cit.gvsig.gui.panels.attributesTree.AttributesTreeTableModel;
import com.iver.cit.gvsig.project.documents.table.gui.Table;
import com.iver.utiles.DefaultCharSet;
import com.iver.utiles.StringUtilities;
import com.iver.utiles.exceptionHandling.ExceptionHandlingSupport;
import com.iver.utiles.exceptionHandling.ExceptionListener;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This will be the tab for add a filter to a WFS query.
 * This class gets the graphical interface from FilterQueryJPanel and add logic.
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class WFSFilterPanel extends FilterQueryJPanel implements IWindow, IWindowListener {
	private static Logger logger = Logger.getLogger(Table.class.getName());
	private WFSParamsPanel parent = null;
	private ArrayList expressionListeners = new ArrayList();
	private ExpressionDataSource model = null;
	private NumberFormat nf = NumberFormat.getNumberInstance();
	private ExceptionHandlingSupport exceptionHandlingSupport = new ExceptionHandlingSupport();
	private AttributesTreeTableModel attributesTreeTableModel;
	private boolean panelAsATabForWFSLayersLoad;
	
	/**
	 * This method initializes
	 *
	 */
	public WFSFilterPanel(WFSParamsPanel parent) {
		super();
		this.parent = parent;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.filterPanel.AbstractFilterQueryJPanel#initialize()
	 */
	protected void initialize() {
		super.initialize();
		this.resizeHeight(380);

		defaultTreeModel = (DefaultTreeModel)fieldsJTree.getModel();
		
		this.addNewListeners();
		panelAsATabForWFSLayersLoad = true;
	}
	
	/**
	 * Adds some more listener to the components of the panel
	 */
	private void addNewListeners() {
		txtExpression.getDocument().addDocumentListener(new DocumentListener() {
			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
			 */
			public void changedUpdate(DocumentEvent e) {
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
			 */
			public void insertUpdate(DocumentEvent e) {
				if (!panelAsATabForWFSLayersLoad)
					parent.isApplicable(true);
			}

			/*
			 *  (non-Javadoc)
			 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
			 */
			public void removeUpdate(DocumentEvent e) {
				if (!panelAsATabForWFSLayersLoad)
					parent.isApplicable(true);
			}
			
		});
		
		// Listener for "fieldsJTree" 
		getFieldsJTree().addMouseListener(new MouseAdapter() {
			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				int row = fieldsJTree.getRowForLocation(e.getX(), e.getY());
				TreePath treePath = fieldsJTree.getPathForLocation(e.getX(), e.getY());

				if (row > -1) {
					switch (e.getClickCount()) {
						case 1:
							if (!panelAsATabForWFSLayersLoad) {
								DataReturnedOfDataLoadingFromActiveView data = DataLoadingFromActiveView.getDefaultExpressionDataSource();
					
								if ((data != null) && (data.getData() != null)) {
									setModel(data.getData());
									fillValuesByPath(treePath);									
								}
							}
							break;
						case 2:						
							putSymbolOfSelectedByMouseBranch(treePath);
							break;
					}
				}
			}
		});
		
		// Listener for "valuesJList"
		getValuesJList().addMouseListener(new MouseAdapter() {
			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					Value valor = (Value) valuesListModel.getElementAt(getValuesJList().getSelectedIndex());

					putSymbol(valor.toString());	
				}
			}
		});
	}
	
	/**
	 * Sets the value of the inner attribute: 'panelAsATabForWFSLayersLoad'
	 * 
	 * @param b A boolean value
	 */
	public void setWFSFilterPanelIsAsTabForWFSLayersLoad (boolean b) {
		this.panelAsATabForWFSLayersLoad = b;
	}
	
	/**
	 * Rellena la lista con los valores del campo seleccionado
	 */
	private void fillValuesByPath(TreePath treePath) {
		// Duplicates are removed
		TreeSet conjunto = new TreeSet(new Comparator() {
			public int compare(Object o1, Object o2) {
				if ((o1 != null) && (o2 != null)) {
					Value v2 = (Value) o2;
					Value v1 = (Value) o1;
					BooleanValue boolVal;
					
					try {
						boolVal = (BooleanValue) (v1.greater(v2));
						
						if (boolVal.getValue()) {
							return 1;
						}
						
						boolVal = (BooleanValue) (v1.less(v2));
						
						if (boolVal.getValue()) {
							return -1;
						}
					} catch (IncompatibleTypesException e) {
						throw new RuntimeException(e);
					}
				}
				
				return 0;
			}
		}); // For ordernation
		
		// Remove the previous items
		valuesListModel.clear();
		
		try {
			//Object root = treePath.getPath()[0];
			XMLElement element = ((XMLElement)treePath.getLastPathComponent());
			
			// Gets the values associated to the selected branch 
			switch (element.getEntityType().getType()) {
				case IXMLType.SIMPLE:
					
					if(element.getParentElement().getParentElement() == null){
					
						// Find the selected field and try to obtein values related
						for (int i = 0; i < model.getFieldCount(); i++) {					
							String name = model.getFieldName(i);
							
							// If we find the field (this means that are loaded its values and we can obtein them)
							if (name.equals(element.getName())) {						
								for (int j = 0; j < model.getRowCount(); j++) {					
									Value value = model.getFieldValue(j, i);
						
									if (value instanceof NullValue)
									    continue;
									
									if (!conjunto.contains(value)) {
										conjunto.add(value);
									}
								}
								
								break;
							}
						}
					}else{
						//create a vector with the parent names from the leaf until the root
						XMLElement parent = element.getParentElement();
						Vector parentNames = new Vector();
						parentNames.add(element.getName());
						while (parent != null){
							parentNames.add(parent.getName());
							parent = parent.getParentElement();							
						}
						
						//The field name (in the gvSIG table) is the second field name
						String fieldName = (String)parentNames.get(parentNames.size()-2);
						
						for (int i = 0; i < model.getFieldCount(); i++) {					
							String name = model.getFieldName(i);
							
							// If we find the field (this means that are loaded its values and we can obtein them)
							if (name.equals(fieldName)) {						
								for (int j = 0; j < model.getRowCount(); j++) {					
									Value value = model.getFieldValue(j, i);
															
									if (value instanceof NullValue)
									    continue;
									
									if (value instanceof ComplexValue){
										for (int k=parentNames.size()-3 ; k>=0 ; k--){
											ComplexValue complex = (ComplexValue)value;
											Value childValue = (Value)complex.get(parentNames.get(k));
											if (k==0){
												if (!conjunto.contains(childValue)) {
													conjunto.add(childValue);
												}
											}else{
												value = childValue;
											}
										}
									}
								}
								
								break;
							}
						}
					}
					break;
				case IXMLType.COMPLEX:
					break;
				default:
					// Do Nothing
			}
			
			// Add the values to the model of the graphic list
			Iterator it = conjunto.iterator();

			while (it.hasNext())
				valuesListModel.addElement(it.next());
		} catch (Exception e) {
			throwException(e);
		}
	}
	
	/**
	 * Puts the symbol of selected brach
	 * 
	 * @param mouseEvent A MouseEvent with information  of the selected branch
	 */
	public void putSymbolOfSelectedByMouseBranch(TreePath treePath) {
		// Sets the node selected
		if (treePath != null) {
			Object node = treePath.getLastPathComponent();
			if ((node != null) && (node instanceof XMLElement)) {
				XMLElement element = (XMLElement) node;
				putSymbol(element.getName());
			}
		}
	}
	
	/**
	 * Gets the query that will be send to the server
	 * @return
	 * SQL query (just the where part)
	 */
	public String getQuery(){
		try {
			return this.validateExpression();
		} catch (ParseException e) {			
			e.printStackTrace();
			return null;
		}		
	}
	
	/**
	 * Writes the query in the user interface
	 * @param query
	 * SQL query (just the where part)
	 */
	public void setQuery(String query){
		this.txtExpression.setText(query);
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean addExpressionListener(ExpressionListener arg0) {
		return expressionListeners.add(arg0);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param arg0
	 *
	 * @return
	 */
	public boolean removeExpressionListener(ExpressionListener arg0) {
		return expressionListeners.remove(arg0);
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param o DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean removeExceptionListener(ExceptionListener o) {
		return exceptionHandlingSupport.removeExceptionListener(o);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	private void throwException(Throwable t) {
		exceptionHandlingSupport.throwException(t);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param t DOCUMENT ME!
	 */
	public void setModel(ExpressionDataSource t) {
		try {
			model = t;
            model.start();
        } catch (DriverException e1) {
            NotificationManager.addError(e1.getMessage(), e1);
        }
                
        try {
        	int numberOfFields = model.getFieldCount();
        	
        	if (numberOfFields > 0) {
        		Vector fields = new Vector(0, 1);
        		int j = 0;
        	
				for (int i = 0; i < numberOfFields; i++) {
					Object field = model.getFieldName(i);
					
					if (field != null) {
						fields.add(field);
						j++;
					}
				}
			
				attributesTreeTableModel = new AttributesTreeTableModel(fields.toArray());			
        	}
		} catch (FilterException e) {
			throwException(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param substring DOCUMENT ME!
	 * @param startingPos DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private int getIndex(String expresion, String substring, int startingPos) {
		int index = startingPos;

		do {
			index = expresion.indexOf(substring, index);
		} while ((StringUtilities.isBetweenSymbols(expresion, index, "\"")) &&
				(index != -1));

		return index;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 * @param word DOCUMENT ME!
	 * @param translation DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateWord(String expresion, String word,
		String translation) throws ParseException {
		int booleanIndex = 0;
		int endIndex = 0;
		StringBuffer res = new StringBuffer();

		while ((booleanIndex = getIndex(expresion, word, booleanIndex)) != -1) {
			res.append(expresion.substring(endIndex, booleanIndex));
			endIndex = booleanIndex + word.length();
			booleanIndex++;
			res.append(translation);
		}

		if (endIndex < expresion.length()) {
			res.append(expresion.substring(endIndex));
		}

		return res.toString();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String translateDates(String expresion) throws ParseException {
		//Se obtiene el valor de la fecha
		String date = StringUtilities.substringDelimited(expresion, "Date(",
				")", 0);

		if (date == null) {
			return expresion;
		}

		//Se comprueba que no est entre comillas 
		int startIndex = expresion.indexOf(date);

		while (startIndex != -1) {
			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex - 5) +
					expresion.substring(startIndex).replaceFirst(date + "\\)",
						new Long((filterButtonsJPanel.getDateFormat().parse(date)).getTime()).toString());
				;
			} else {
				startIndex += date.length();
			}
			
			if (date == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(date, startIndex);
		}

		return expresion;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expresion DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	public String translateNumber(String expresion) throws ParseException {
		DefaultCharSet ss = new DefaultCharSet();
		ss.addInterval('0', '9');
		ss.addCharacter(',');
		ss.addCharacter('.');

		String number = StringUtilities.substringWithSymbols(expresion, ss, 0);

		if (number == null) {
			return expresion;
		}

		int startIndex = expresion.indexOf(number);

		while (startIndex != -1) {
			Number n = nf.parse(number);

			if (!StringUtilities.isBetweenSymbols(expresion, startIndex, "\"")) {
				
				//Se sustituye por el valor ordinal de la fecha
				expresion = expresion.substring(0, startIndex) +
					expresion.substring(startIndex).replaceFirst(number,
						n.toString());
			} else {
				startIndex += n.toString().length();
			}

			number = StringUtilities.substringWithSymbols(expresion, ss,
					startIndex);

			if (number == null) {
				return expresion;
			}

			startIndex = expresion.indexOf(number, startIndex);
		}

		return expresion;
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String validateExpression() throws ParseException {
		String expression = txtExpression.getText();
//		HashSet variablesIndexes = new HashSet();
//
//		StringBuffer traducida = new StringBuffer();

		//Se transforman los nombres de los campos en las variables xix que analizarn
		//Se quitan los Date(fecha) y se mete la fecha correspondiente
		expression = translateDates(expression);
		expression = translateNumber(expression);
		expression = translateWord(expression, "true", "1");
		expression = translateWord(expression, "false", "0");

//		String replacement;
//		Pattern patron = Pattern.compile("[^<>!]=");
//		Matcher m = patron.matcher(expression);
//		int index = 0;
//
//		while (m.find(index)) {
//			index = m.start();
//			replacement = expression.charAt(index) + "==";
//			m.replaceFirst(replacement);
//			index++;
//		}
//
//		expression = expression.replaceAll("[^<>!]=", "==");

		logger.debug(expression);

		return expression;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws ParseException DOCUMENT ME!
	 */
	private String oldValidateExpressionMethod() throws ParseException {
		String expression = txtExpression.getText();
//		HashSet variablesIndexes = new HashSet();
//
//		StringBuffer traducida = new StringBuffer();

		//Se transforman los nombres de los campos en las variables xix que analizarn
		//Se quitan los Date(fecha) y se mete la fecha correspondiente
		expression = translateDates(expression);
		expression = translateNumber(expression);
		expression = translateWord(expression, "true", "1");
		expression = translateWord(expression, "false", "0");

		String replacement;
		Pattern patron = Pattern.compile("[^<>!]=");
		Matcher m = patron.matcher(expression);
		int index = 0;

		while (m.find(index)) {
			index = m.start();
			replacement = expression.charAt(index) + "==";
			m.replaceFirst(replacement);
			index++;
		}

		expression = expression.replaceAll("[^<>!]=", "==");

		logger.debug(expression);

		return expression;
	}

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewActivated()
     */
    public void windowActivated() {
    }

    /* (non-Javadoc)
     * @see com.iver.andami.ui.mdiManager.ViewListener#viewClosed()
     */
    public void windowClosed() {
        try {
            model.stop();
        } catch (DriverException e) {
            NotificationManager.addError(e.getMessage(), e);
        }        
    }

	public WindowInfo getWindowInfo() {
		// TODO Auto-generated method stub
		return null;
	}

	public void refresh(WFSLayerNode feature) {
		setFields(feature);
	}
	
	/**
	 * Sets Fields
	 *
	 * @param feature
	 */
	private void setFields(WFSLayerNode feature) {
		Vector fields = feature.getFields();
				
		this.resetFieldsAndValuesData();
		
		int numberOfFields = fields.size();
		
		if (numberOfFields > 0) {
			int j = 0;
			Vector fieldBranches = new Vector(0, 1);
			
			for (int i=0 ; i<fields.size() ;i++) {
				XMLElement field = (XMLElement)fields.get(i);				

				IXMLType type = field.getEntityType();
				
				if (type != null) {
					switch (type.getType()) {
						case IXMLType.GML_GEOMETRY:
							break;
						case IXMLType.COMPLEX:
							fieldBranches.add(field);
							j++;
							break;
						case IXMLType.SIMPLE:
							fieldBranches.add(field);
							j++;
							break;
					}
				}
			}
			
			attributesTreeTableModel = new AttributesTreeTableModel(fieldBranches.get(0));
			fieldsJTree.setModel(attributesTreeTableModel);
		}
	}

	/**
	 * Resets the data of fields and their values of the current layer feature, and removes the branches of JTree
	 */
	private void resetFieldsAndValuesData() {
		fieldsJTree.setModel(new DefaultTreeModel(new DefaultMutableTreeNode()));
		txtExpression.setText("");
		((DefaultListModel)valuesJList.getModel()).removeAllElements();
	}
	
	
	/**
	 * @see WFSParamsPanel#isApplicable(boolean)
	 * 
	 * @param b A boolean value
	 */
	private void setApplicate(boolean b) {
		parent.isApplicable(b);
	}
}
