package com.iver.cit.gvsig.gui.panels;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.ObjectInputStream.GetField;
import java.util.prefs.Preferences;

import javax.swing.JTabbedPane;

import org.gvsig.remoteClient.wfs.WFSStatus;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriver;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrWFS;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.gui.WizardPanel;
import com.iver.cit.gvsig.gui.wizards.WFSWizardData;
import com.iver.cit.gvsig.gui.wizards.WizardListener;
import com.iver.cit.gvsig.gui.wizards.WizardListenerSupport;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSParamsPanel.java,v 1.19 2006/10/27 12:10:02 ppiqueras Exp $
 * $Log: WFSParamsPanel.java,v $
 * Revision 1.19  2006/10/27 12:10:02  ppiqueras
 * Nueva funcionalidad
 *
 * Revision 1.16  2006/10/23 07:37:04  jorpiell
 * Ya funciona el filterEncoding
 *
 * Revision 1.14  2006/10/13 13:05:32  ppiqueras
 * Permite el refrescado de datos del panel de filtrado sobre capa WFS.
 *
 * Revision 1.13  2006/10/10 12:55:06  jorpiell
 * Se ha aadido el soporte de features complejas
 *
 * Revision 1.12  2006/10/02 09:17:48  jorpiell
 * Aadido el setCRS a la capa
 *
 * Revision 1.11  2006/09/29 13:02:38  ppiqueras
 * Filtro para WFS. De momento sólo interfaz gráfica.
 *
 * Revision 1.10  2006/07/21 11:50:31  jaume
 * improved appearance
 *
 * Revision 1.9  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.8  2006/06/15 07:50:58  jorpiell
 * Aadida la funcionalidad de reproyectar y hechos algunos cambios en la interfaz
 *
 * Revision 1.7  2006/05/25 16:22:47  jorpiell
 * Se limpia el panel cada vez que se conecta con un servidor distinto
 *
 * Revision 1.6  2006/05/25 16:01:43  jorpiell
 * Se ha aadido la funcionalidad para eliminar el namespace de los tipos de atributos
 *
 * Revision 1.5  2006/05/25 10:31:06  jorpiell
 * Como ha cambiado la forma de mostrar las capas (una tabla, en lugar de una lista), los paneles han tenido que ser modificados
 *
 * Revision 1.4  2006/05/23 08:09:39  jorpiell
 * Se ha cambiado la forma en la que se leian los valores seleccionados en los paneles y se ha cambiado el comportamiento de los botones
 *
 * Revision 1.3  2006/05/19 12:57:08  jorpiell
 * Modificados algunos paneles
 *
 * Revision 1.2  2006/04/20 16:38:24  jorpiell
 * Ahora mismo ya se puede hacer un getCapabilities y un getDescribeType de la capa seleccionada para ver los atributos a dibujar. Queda implementar el panel de opciones y hacer el getFeature().
 *
 * Revision 1.1  2006/04/19 12:50:16  jorpiell
 * Primer commit de la aplicacin. Se puede hacer un getCapabilities y ver el mensaje de vienvenida del servidor
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSParamsPanel extends WizardPanel{
	private JTabbedPane jTabbedPane = null;  //  @jve:decl-index=0:visual-constraint="37,30"
	private WFSSelectFeaturePanel featurePanel = null;
	private WFSSelectAttributesPanel fieldsPanel = null;
	private WFSOptionsPanel optionsPanel = null;
	private WFSFormatPanel formatPanel = null;
	private WFSFilterPanel filterPanel = null;
	private WFSAreaPanel areaPanel = null;
	private WFSWizardData data;
	private WizardListenerSupport listenerSupport;
	public	static Preferences fPrefs = Preferences.userRoot().node( "gvsig.wcs-wizard" );
	private WFSInfoPanel infoPanel;
	private int indFields;
	private int indFormat;
	private int indFilter;
	private int indArea;
	

	/**
	 * This method initializes jTabbedPane
	 *
	 * @return javax.swing.JTabbedPane
	 */

	public WFSParamsPanel(){
		super();
		initialize();
	}
	/**
	 * Returns the tab amount that the WFSParamsPanel currently have
	 *
	 * Devuelve el nmero de solapas que tiene actualmente el WFSParamsPanel
	 *
	 * @return int
	 */
	public int getNumTabs(){
	 	return getJTabbedPane().getTabCount();
	}

	private void initialize() {
		this.setLayout(null);
		this.setVisible(false);
		this.setBounds(0, 0, 510, 427);
		this.add(getJTabbedPane(), null);
	}

	/**
	 * Sets the focus to the tab next to the current one.
	 *
	 * Enfoca a la solapa siguiente a la actualmente enfocada del TabbedPane
	 *
	 */
	public void avanzaTab(){
	    int currentPage = currentPage();
	    int nPages = getNumTabs();
		if (nPages -1 > currentPage){
			getJTabbedPane().setSelectedIndex(nextPageEnabled());
		}
	}

	/**
	 * Sets the focus to the tab previous to the current one.
	 *
	 * Enfoca a la solapa anterior a la actualmente enfocada del TabbedPane
	 *
	 */
	public void retrocedeTab(){
		this.getJTabbedPane().setSelectedIndex(previousEnabledPage());
	}

	/**
	 * Returns the current tab index.
	 *
	 * Devuelve el nmero de solapa en que se encuentra
	 *
	 * @return
	 */
	public int getIndiceSolapaActual(){
		return this.getJTabbedPane().getSelectedIndex();
	}

	public JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.setBounds(4, 4, 502, 415);
			jTabbedPane.addTab(PluginServices.getText(this, "info"), null, getInfoPanel());
			jTabbedPane.addTab(PluginServices.getText(this, "feature"), null, getFeaturesPanel(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "attributes"),null, getAttributesPanel(), null);
			indFields = jTabbedPane.getTabCount() - 1;
			jTabbedPane.setEnabledAt(indFields, false);
			//jTabbedPane.addTab(PluginServices.getText(this,"formatos"),null,getFormatPanel(),null);
//			indFormat = jTabbedPane.getTabCount() - 1;
//			jTabbedPane.setEnabledAt(indFormat, false);
			jTabbedPane.addTab(PluginServices.getText(this, "options"),null, getOptionsPanel(), null);
			jTabbedPane.addTab(PluginServices.getText(this, "filter"),null, getFilterPanel(), null);
			indFilter = jTabbedPane.getTabCount() - 1;
			jTabbedPane.setEnabledAt(indFilter, false);
			//jTabbedPane.addTab(PluginServices.getText(this, "area"),null, getAreaPanel(), null);
			indArea = jTabbedPane.getTabCount() - 1;
			jTabbedPane.setEnabledAt(indArea, false);
		}
		return jTabbedPane;
	}

	private WFSInfoPanel getInfoPanel() {
		if (infoPanel==null) {
			infoPanel = new WFSInfoPanel();
			infoPanel.addFocusListener(new FocusListener() {
				public void focusGained(java.awt.event.FocusEvent e) {
					refreshInfo();
				}

				private void refreshInfo() {
					WFSLayerNode selectedNode = featurePanel.getSelectedFeature();
					if (selectedNode != null){
						selectedNode.setSelectedFields(fieldsPanel.getSelectedFields());
					}
					data.setUserName(optionsPanel.getUserName());
					data.setBuffer(optionsPanel.getBuffer());
					data.setTimeOut(optionsPanel.getTimeout());
					infoPanel.refresh(data,selectedNode);
				}

				public void focusLost(FocusEvent e) {

				}
			});
		}
		return infoPanel;
	}

	/**
	 *
	 * @return the selected list
	 */
	private Object getSelectedFeature() {
		return featurePanel.getSelectedFeature();
	}


	/**
	 * This method initializes panelFeatures
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSSelectFeaturePanel getFeaturesPanel() {
		if (featurePanel == null) {
			featurePanel = new WFSSelectFeaturePanel(this);
			featurePanel.setLayout(null);
		}
		return featurePanel;
	}

	/**
	 * This method initializes fieldsPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSSelectAttributesPanel getAttributesPanel(){
		if (fieldsPanel == null) {
			fieldsPanel = new WFSSelectAttributesPanel(this);
			fieldsPanel.setLayout(null);
		}
		return fieldsPanel;
	}



	/**
	 * This method initializes fieldsPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSOptionsPanel getOptionsPanel(){
		if (optionsPanel == null) {
			optionsPanel = new WFSOptionsPanel(this);
			optionsPanel.setLayout(null);
		}
		return optionsPanel;
	}

	
	/**
	 * This method initializes filterPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private WFSFilterPanel getFilterPanel(){
		if (filterPanel == null) {
			filterPanel = new WFSFilterPanel(this);
			filterPanel.setVisible(true);
			//filterPanel.setLayout(null);
		}
		return filterPanel;
	}
	
	/**
	 * This method initializes areaPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private WFSAreaPanel getAreaPanel(){
		if (areaPanel == null) {
			areaPanel = new WFSAreaPanel(this);
			areaPanel.setVisible(true);			
		}
		return areaPanel;
	}

	/**
	 * This method initializes formatPanel
	 *
	 * @return javax.swing.JPanel
	 */
	private WFSFormatPanel getFormatPanel(){
		if (formatPanel == null) {
			formatPanel = new WFSFormatPanel();
			formatPanel.setLayout(null);
		}
		return formatPanel;
	}

	/**
	 * Verifies that the selected parameters are enough to request the coverage
	 * to the server.
	 *
	 * Comprueba que los parmetros seleccionados son suficientes para pedir la
	 * cobertura al servidor.
	 *
	 * @return boolean
	 */
	public boolean isCorretlyConfigured() {
		if (featurePanel.getSelectedFeature()!= null){
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Cleans up the wizard's components but the server's layers list.
	 *
	 * Limpia todos los componentes del wizard excepto la lista de capas del
	 * servidor.
	 */
	public void cleanupWizard() {
		jTabbedPane.setEnabledAt(indFields,false);
//		jTabbedPane.setEnabledAt(indFormat,false);
	}

	/**
	 * Refreshes the wizard components data each time a feature
	 * is selected.
	 *
	 * Actualiza los datos de los componentes del wizard cada vez
	 * que se selecciona una feature diferente.
	 *
	 * @param feature
	 * Selected feature
	 */
	public void refreshData(WFSLayerNode feature){
		if (feature != null){
			feature = (WFSLayerNode)data.getFeatureInfo(feature.getName());
			jTabbedPane.setEnabledAt(indFields,true);
			jTabbedPane.setEnabledAt(indFilter,true);
			jTabbedPane.setEnabledAt(indArea,true);
			fieldsPanel.refresh(feature);
			optionsPanel.refresh(feature);
			filterPanel.refresh(feature);
		}
		listenerSupport.callStateChanged(isCorretlyConfigured());

	}

	public void refreshInfo(WFSLayerNode feature){
		infoPanel.refresh(data,feature);
	}

	/**
	 * Sets the object that holds the wizard data.
	 *
	 * Establece el objeto que contiene los datos del wizard.
	 *
	 * @param data
	 */
	public void setWizardData(WFSWizardData data) {
		this.data = data;
		//WFSLayerNode selectedNode = (WFSLayerNode)featurePanel.getSelectedFeature();
		featurePanel.setWizardData(data);
		data.setBuffer(optionsPanel.getBuffer());
		data.setTimeOut(optionsPanel.getTimeout());
		infoPanel.refresh(data,null);
	}

	/**
	 * Sets the wizard's data source
	 *
	 * Establece el origen de los datos del wizard
	 * @param dataSource
	 */
	public void setDataSource(WFSWizardData dataSource) {
		this.data = dataSource;

	}


    /**
     * Returns the next enabled tab's index.
     *
     * Devuelve el ndicie de la siguiente pestaa habilitada del wizard o -1 si no hay
     * ninguna.
     *
     * @return The index or -1 if there is no one.
     */
    public int nextPageEnabled() {
        int currentPage = currentPage();
        int nPages = getNumTabs();
        if (currentPage == nPages)
    	    return -1;
        for (int i=currentPage+1; i<nPages; i++){
            if (getJTabbedPane().isEnabledAt(i)){
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns the index of the current tab.
     *
     * Devuelve el ndice de la pgina actual del wizard.
     *
     * @return
     */
    public int currentPage() {
        return getJTabbedPane().getSelectedIndex();
    }

    /**
     * Returns the index of the previous enabled tab.
     *
     * Devuelve el ndice de la anterior pgina habilitada del wizard o -1 si no hay
     * ninguna.
     *
     * @return The index, or -1 if there is no one.
     */
    public int previousEnabledPage() {
        int currentPage = currentPage();
        int j=0;
        if (currentPage == 0)
            j= -1;
        for (int i = currentPage-1; i>-1; i--){
            if (getJTabbedPane().isEnabledAt(i)){
                j= i;
            	break;
            }
        }
        return j;
    }

	public FMapWFSDriver getDriver() {
		return data.getDriver();
	}

	public void setListenerSupport(WizardListenerSupport support) {
		listenerSupport = support;
	}


	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#initWizard()
	 */
	public void initWizard() {

	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.WizardPanel#execute()
	 */
	public void execute() {

	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.ILayerPanel#getLayer()
	 */
	public FLayer getLayer() {
		FLyrWFS layer = new FLyrWFS();
		layer.setName(getFeaturesPanel().getLayerText());
		layer.setLayerName(getFeaturesPanel().getLayerName());
		layer.setWfsLayerNode(getLayerNode());
		layer.setFields(getAttributesPanel().getSelectedFields());
		layer.setUserName(getOptionsPanel().getUserName());
		layer.setPassword(getOptionsPanel().getPassword());
		layer.setNumfeatures(getOptionsPanel().getBuffer());
		layer.setTimeout(getOptionsPanel().getTimeout());
		layer.setSrs(getOptionsPanel().getSRS());
		layer.setAttributesQuery(getFilterPanel().getQuery());
		layer.setWfsDriver(data.getDriver());
		return layer;
	}

	private WFSLayerNode getLayerNode(){
		WFSLayerNode layerNode = getFeaturesPanel().getWFSLayerNode();
		layerNode.setSelectedFields(getAttributesPanel().getSelectedFields());
		layerNode.setFilter(getFilterPanel().getQuery());
		return layerNode;
	}


	public void setSelectedFeature(WFSLayerNode layerNode){
		getFeaturesPanel().setSelectedFeature(layerNode);
		getAttributesPanel().setSelectedFields(layerNode);
		getFilterPanel().setQuery(layerNode.getFilter());
	}	
	
	
	/**
	 * @return Returns the data.
	 */
	public WFSWizardData getData() {
		return data;
	}
	public void setLayerName(String name) {
		getFeaturesPanel().setLayerName(name);

	}

	public void addWizardListener(WizardListener listener) {
		listenerSupport.addWizardListener(listener);
	}

	public void isApplicable(boolean applicable){
		listenerSupport.callStateChanged(applicable);
	}


	public void setStatus(WFSStatus status) {
		getOptionsPanel().setStatus(status);
	}

	/**
	 * @see WFSFilterPanel#setWFSFilterPanelIsAsTabForWFSLayersLoad(boolean)
	 */
	public void setWFSFilterPanelIsAsTabForWFSLayersLoad(boolean b) {
		this.filterPanel.setWFSFilterPanelIsAsTabForWFSLayersLoad(b);
	}	
}
