package com.iver.cit.gvsig.gui.panels.attributesTree;

import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

import org.gvsig.gui.beans.swing.treeTable.TreeTableModelAdapter;
import org.gvsig.remoteClient.gml.schemas.XMLElement;

import com.iver.cit.gvsig.gui.panels.WFSSelectAttributesPanel;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: AttributesTreeTable.java,v 1.3 2006/11/01 17:29:07 jorpiell Exp $
 * $Log: AttributesTreeTable.java,v $
 * Revision 1.3  2006/11/01 17:29:07  jorpiell
 * Se ha elimiado el nodo virtual de la raiz. Adems ya se cargan los valores de un campo complejo en la pestaa del filtro
 *
 * Revision 1.2  2006/10/31 09:39:44  jorpiell
 * Se devuelve el tipo de la entidad completo, y no sus hijos
 *
 * Revision 1.1  2006/10/27 11:33:19  jorpiell
 * Aadida la treetable con los check box para seleccionar los atributos
 *
 * Revision 1.3  2006/10/24 10:13:19  jorpiell
 * Se ha eliminado la pestaa de filter
 *
 * Revision 1.2  2006/10/24 07:58:14  jorpiell
 * Eliminado el parametro booleano que haca que apareciesen mas de una columna en el tree table
 *
 * Revision 1.1  2006/10/24 07:27:56  jorpiell
 * Algunos cambios en el modelo que usa la tabla
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class AttributesTreeTable extends org.gvsig.gui.beans.swing.treeTable.TreeTable{
	Object[] attributes = null;
	WFSSelectAttributesPanel parent = null;
	
	public AttributesTreeTable(AttributesTreeTableModel treeTableModel) {
		super(treeTableModel);
		JTree tree = (JTree)getTree();
		tree.setCellRenderer(new CheckBoxTreeCellRenderer(this));
		tree.setCellEditor(new CheckBoxTreeCellEditor(this));
		tree.setEditable(true);		
	}
	
	public AttributesTreeTable(AttributesTreeTableModel treeTableModel,WFSSelectAttributesPanel parent) {
		super(treeTableModel);
		this.parent = parent;
		JTree tree = (JTree)getTree();
		tree.setCellRenderer(new CheckBoxTreeCellRenderer(this));
		tree.setCellEditor(new CheckBoxTreeCellEditor(this));
		tree.setEditable(true);
	}
	
	/**
	 * Sets the attributes
	 * @param attributes
	 */
	public void setAttributes(Vector vAttributes){
		attributes = new Object[vAttributes.size()];
		for (int i=0 ; i<vAttributes.size() ; i++){
			attributes[i] = vAttributes.get(i);
		}
		setModel(new AttributesTreeTableModel(attributes));
		deleteIcons();
		setRootVisible(false);
	}
	
	/**
	 * Gets the selected layer
	 * @return
	 */
	public Object[] getSelectedValues(){
		int[] selectedRows = getSelectedRows();
		TreeTableModelAdapter obj = (TreeTableModelAdapter)getModel();
		Object[] objects = new Object[selectedRows.length];
		for (int i=0 ; i<selectedRows.length ; i++){
			objects[i] = obj.nodeForRow(selectedRows[i]);
		}
		return objects;
	}
	
	/**
	 * return the selected elements
	 * @return
	 */
	public XMLElement[] getSelectedElements(){
		TreeTableModelAdapter obj = (TreeTableModelAdapter)getModel();
		ArrayList elements = new ArrayList();
		for (int i=0 ; i< obj.getRowCount() ; i++){
			if (obj.nodeForRow(i) instanceof CheckBoxNode){
				CheckBoxNode node = (CheckBoxNode)obj.nodeForRow(i);
				if (node.isSelected()){
						elements.add(node.getElement());
				}					
			}
		}
	
	XMLElement[] selected = new XMLElement[elements.size()];
	for (int i=0 ; i<elements.size() ; i++){
		selected[i] = (XMLElement)elements.get(i);
	}
	return selected;
}

/**
 * Return all the attributes
 * @return
 */
public Object[] getAllValues() {
	return attributes;
}

/**
 * Return true if the node is queryable
 * @param node
 * @return
 */
public boolean isQueryable(Object node){
	for (int i=0 ; i<attributes.length ; i++){
		if (node.equals(attributes[i])){
			return true;
		}
	}
	return false;
}

public void setModel(AttributesTreeTableModel treeTableModel){
	super.setModel(treeTableModel);
	JTree tree = (JTree)getTree();
	tree.setCellRenderer(new CheckBoxTreeCellRenderer(this));
	tree.setCellEditor(new CheckBoxTreeCellEditor(this));
	tree.setEditable(true);		
}

/**
 * Set the selected fields
 * @param selectedFields
 */
public void setSelectedFields(Vector selectedFields) {
	TreeTableModelAdapter obj = (TreeTableModelAdapter)getModel();
	JTree tree = (JTree)getTree();
	for (int i = 0; i < tree.getRowCount(); i++) {
		tree.expandRow(i);
	}	
	for (int i=0 ; i<obj.getRowCount() ; i++){
		if (obj.nodeForRow(i) instanceof CheckBoxNode){
			XMLElement element = ((CheckBoxNode)obj.nodeForRow(i)).getElement();
			for (int j=0 ; j<selectedFields.size() ; j++){
				//If the name is equals
				if (((XMLElement)selectedFields.get(j)).getName().equals(element.getName())){
					//If the type is equals
					if (((XMLElement)selectedFields.get(j)).getEntityType().getName().equals(element.getEntityType().getName())){
						((CheckBoxNode)obj.nodeForRow(i)).setSelected(true);
					}
				}
			}
		}			
	}
	
}

/**
 * @param updated The updated to set.
 */
public void setUpdated(boolean updated) {
	if (parent != null){
		parent.setUpdated(updated);
	}
}


}
