package com.iver.cit.gvsig.gui.panels.attributesTree;

import java.util.Vector;

import org.gvsig.gui.beans.swing.treeTable.AbstractTreeTableModel;
import org.gvsig.gui.beans.swing.treeTable.TreeTableModel;
import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLComplexType;
import org.gvsig.remoteClient.gml.schemas.XMLElement;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.wfs.WFSUtils;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: AttributesTreeTableModel.java,v 1.4 2006/11/01 17:29:07 jorpiell Exp $
 * $Log: AttributesTreeTableModel.java,v $
 * Revision 1.4  2006/11/01 17:29:07  jorpiell
 * Se ha elimiado el nodo virtual de la raiz. Adems ya se cargan los valores de un campo complejo en la pestaa del filtro
 *
 * Revision 1.3  2006/10/31 13:51:15  ppiqueras
 * Soporte parent Element
 *
 * Revision 1.2  2006/10/31 12:24:04  jorpiell
 * Comprobado el caso en el que los atributos no tienen tipo
 *
 * Revision 1.1  2006/10/27 11:33:19  jorpiell
 * Aadida la treetable con los check box para seleccionar los atributos
 *
 * Revision 1.2  2006/10/24 07:58:14  jorpiell
 * Eliminado el parametro booleano que haca que apareciesen mas de una columna en el tree table
 *
 * Revision 1.1  2006/10/24 07:27:56  jorpiell
 * Algunos cambios en el modelo que usa la tabla
 *
 * Revision 1.2  2006/10/23 08:17:18  jorpiell
 * Algunos cambios realizados para tener un rbol com ms de una raz
 *
 * Revision 1.1  2006/10/23 07:37:04  jorpiell
 * Ya funciona el filterEncoding
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class AttributesTreeTableModel extends AbstractTreeTableModel {
	private String[]  cNames = {PluginServices.getText(this,"attributeName"),
			PluginServices.getText(this,"attributeType")};
	private Class[]  cTypes = {TreeTableModel.class,String.class};
			
	public AttributesTreeTableModel(Object root) { 
		super(root);			
	}		
	
	public AttributesTreeTableModel() { 
		super(null); 		
	}
	
	//
	// The TreeModel interface
	//
	
	public int getChildCount(Object node) { 
		XMLElement element = (XMLElement)node;			
		if (element.getEntityType().getType() == IXMLType.COMPLEX){
			return ((XMLComplexType)element.getEntityType()).getAttributes().size();
		}else{
			return 0;
		}
	}
	
	public Object getChild(Object node, int i) { 
		XMLElement element = (XMLElement)node;			
		if (element.getEntityType().getType() == IXMLType.COMPLEX){
			Vector vector = ((XMLComplexType)element.getEntityType()).getAttributes();
			XMLElement child = (XMLElement) vector.get(i);
			child.setParentElement(element);
			return child;
		}
		return null;
	}
	
	// The superclass's implementation would work, but this is more efficient. 
	public boolean isLeaf(Object node) { 
		if (node == null){
			return true;
		}
		XMLElement element = (XMLElement)node;	
		if (element.getEntityType() != null){
			if (element.getEntityType().getType() == IXMLType.COMPLEX){
				return false;
			}
		}
		return true;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.swing.treeTable.TreeTableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return cNames.length;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.swing.treeTable.TreeTableModel#getColumnName(int)
	 */
	public String getColumnName(int column) {
		return cNames[column];
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.swing.treeTable.TreeTableModel#getColumnClass(int)
	 */
	public Class getColumnClass(int column) {
		return cTypes[column];
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.gui.beans.swing.treeTable.TreeTableModel#getValueAt(java.lang.Object, int)
	 */
	public Object getValueAt(Object node, int column) {
		XMLElement element = (XMLElement)node;	
		try {
			switch(column) {
			case 0:
				return element.getName();
			case 1:
				return PluginServices.getText(this,WFSUtils.getAttributeType(element.getEntityType()));
			}
		}
		catch  (SecurityException se) { }
		
		return null; 
	}
}


