/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.BrightnessContrast;

import java.util.ArrayList;

import org.cresques.filter.IStackManager;
import org.cresques.filter.RasterBuf;
import org.cresques.filter.RasterFilter;
import org.cresques.filter.RasterFilterStack;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;

/**
 * Gestor de la pila de filtros para el filtro de brillo.
 * @author Miguel �ngel Querol Carratal�  <querol_mig@gva.es>
 *
 */

public class BrightnessContrastStackManager implements IStackManager{
	
	public final static int				brightness = 7;
	public final static int				histogram = 6;
	public final static int				contrast = 8;
	protected RasterFilterStack 		filterStack = null;
	private RasterFilterStackManager	filterStackManager = null;
	private GeoRasterFile[] 			grfList = null;
	
	public BrightnessContrastStackManager(RasterFilterStackManager filterStackManager){
		this.filterStackManager = filterStackManager;
		this.filterStack = filterStackManager.getFilterStack();
		filterStackManager.addTypeFilter("histogram", BrightnessContrastStackManager.histogram, 5);
		filterStackManager.addTypeFilter("brightness", BrightnessContrastStackManager.brightness, 6);
		filterStackManager.addTypeFilter("contrast", BrightnessContrastStackManager.contrast, 7);
		
	}
	
	
	/**
	 * Obtiene el valor entero que corresponde al tipo de filtro.
	 * @param rasterFilter
	 * @return
	 */
	public int getType(RasterFilter rasterFilter){
		if(rasterFilter instanceof BrightnessFilter)
			return BrightnessContrastStackManager.brightness;
		/*if(rasterFilter instanceof HistogramFilter)
			return BrightnessContrastStackManager.histogram;*/
		if(rasterFilter instanceof ContrastFilter)
			return BrightnessContrastStackManager.contrast;
					
		return filterStackManager.getType(rasterFilter);
	}
	
	/**
	 * A�ade un filtro de brillo a la pila de filtros.
	 * @param incrBrillo
	 */
	public void addBrightnessFilter(int incrBrillo){
		//Si ya hay un filtro de brillo en la pila lo sobreescribimos
		
		if(filterStack.isActive(((Integer) filterStackManager.getTypeFilters().get("brightness")).intValue()))
			filterStack.removeFilter(((Integer) filterStackManager.getTypeFilters().get("brightness")).intValue());
	
		RasterFilter filtro = null;
		
		switch(filterStack.getDataTypeInFilter(((Integer)this.filterStackManager.getTypeFilters().get("brightness")).intValue())){
		case RasterBuf.TYPE_IMAGE:
			filtro = new BrightnessImageFilter();
			break;
		case RasterBuf.TYPE_SHORT:
			filtro = new BrightnessShortFilter();
			break;
		case RasterBuf.TYPE_USHORT:
		case RasterBuf.TYPE_INT:
			break;
		}
		
		//Cuando el filtro esta creado, tomamos los valores y lo a�adimos a la pila
		
		if(filtro != null){
			filtro.addParam("incrBrillo", new Integer(incrBrillo));
			filterStack.addFilter(((Integer) filterStackManager.getTypeFilters().get("brightness")).intValue(), filtro);
			filterStackManager.controlTypes();
		}
	}
	
	/**
	 * A�ade un filtro de histograma a la pila de filtros
	 *
	 */
	/*public void addHistogramFilter(){
		
		RasterFilter filtro =  null;
		
		//Si ya hay un filtro de histrograma en la pila lo sobreescribimos
		
		if(filterStack.isActive(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue()))
			filterStack.removeFilter(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue());
		
		//Crea el filtro a partir del dato de entrada que tenemos
		switch(filterStack.getDataTypeInFilter(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue())){
		case RasterBuf.TYPE_IMAGE:
			filtro = new HistogramImageFilter();
		case RasterBuf.TYPE_SHORT:
		case RasterBuf.TYPE_USHORT:
		case RasterBuf.TYPE_INT:
			break;
		}
		
		if(filtro != null){
			filterStack.addFilter(((Integer) filterStackManager.getTypeFilters().get("histogram")).intValue(),filtro);
	        
	        filterStackManager.controlTypes();
		}
	}*/
	
	
	/**
	 * A�ade un filtro de contraste a la pila de filtros.
	 * @param incrBrillo
	 */
	public void addContrastFilter(int incrContraste){
		//Si ya hay un filtro de contraste en la pila lo sobreescribimos
		
		if(filterStack.isActive(((Integer) filterStackManager.getTypeFilters().get("contrast")).intValue()))
			filterStack.removeFilter(((Integer) filterStackManager.getTypeFilters().get("contrast")).intValue());
	
		RasterFilter filtro = null;
		
		switch(filterStack.getDataTypeInFilter(((Integer)this.filterStackManager.getTypeFilters().get("contrast")).intValue())){
		case RasterBuf.TYPE_IMAGE:
			filtro = new ContrastImageFilter();
			break;
		case RasterBuf.TYPE_SHORT:
			filtro = new ContrastShortFilter();
			break;
		case RasterBuf.TYPE_USHORT:
		case RasterBuf.TYPE_INT:
			
			break;
		}
		
		//Cuando el filtro esta creado, tomamos los valores y lo a�adimos a la pila
		
		if(filtro != null){
			filtro.addParam("incrContraste", new Integer(incrContraste));
			filterStack.addFilter(((Integer) filterStackManager.getTypeFilters().get("contrast")).intValue(), filtro);
			filterStackManager.controlTypes();
		}
	}
	
	
	public void createStackFromStrings(ArrayList f, Integer pos) {
		// TODO Auto-generated method stub
		
	}

	
	
	/**
     * Crea una pila de filtros a partir de un Array de Strings. Cada elemento del array debe
     * tener la forma elemento=valor.
     * @param filters
     */
	public int createStackFromStrings(ArrayList filters, String fil, GeoRasterFile[] grfList, int filteri) {
		this.grfList = grfList;

			if((fil.startsWith("filter.brightCont.active"))&&
					(RasterFilterStackManager.getValue(fil).equals("true"))){
				
				int incrBrillo = 0;
				int incrContraste = 0;
				filters.remove(0);
				
				for(int prop = 0; prop < filters.size() ; prop++){	
					String elem = (String) filters.get(prop);
					if(elem.startsWith("filter.brightness.incrBrillo")){
						incrBrillo = Integer.parseInt(RasterFilterStackManager.getValue(elem));
						filters.remove(prop);
						prop--;
					}
				
					if(elem.startsWith("filter.contrast.incrContraste")){
						incrContraste = Integer.parseInt(RasterFilterStackManager.getValue(elem));
						filters.remove(prop);
						prop--;
					}
				}
					addBrightnessFilter(incrBrillo);
					addContrastFilter(incrContraste);	
			}
		return filteri;
	}

	public ArrayList getStringsFromStack() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * Obtiene un Array de Strings a partir de una pila de filtros. Cada elemento del array
	 * tendr� la forma de elemento=valor.
	 */
	
	public ArrayList getStringsFromStack(ArrayList filterList, RasterFilter rf) {
		
		
		if((rf instanceof BrightnessFilter) || 
			(rf instanceof ContrastFilter)){
			filterList.add("filter.brightCont.active=true");
			
		}
		
		if(rf instanceof BrightnessFilter){
			BrightnessFilter bright = (BrightnessFilter) rf;
			filterList.add("filter.brightness.incrBrillo="+bright.incrBrillo);
        }else if(rf instanceof ContrastFilter){
        	ContrastFilter contrast = (ContrastFilter) rf;
        	filterList.add("filter.contrast.incrContraste="+contrast.incrContraste);
        }
			
		return filterList;
	}

}
