/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter.ComputeMinMax;

import org.cresques.filter.RasterFilter;
import org.cresques.filter.RasterStats;


/**
 * Calcula el mximo y el mnimo de un raster, As como el segundo menor valor y
 * el segundo mayor valor.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public abstract class ComputeMinMaxFilter extends RasterFilter {
    protected int[] min = {
                              Integer.MAX_VALUE, Integer.MAX_VALUE,
                              Integer.MAX_VALUE
                          };
    protected int[] max = {
                              Integer.MIN_VALUE, Integer.MIN_VALUE,
                              Integer.MIN_VALUE
                          };
    protected int[] secondMin = {
                                    Integer.MAX_VALUE, Integer.MAX_VALUE,
                                    Integer.MAX_VALUE
                                };
    protected int[] secondMax = {
                                    Integer.MIN_VALUE, Integer.MIN_VALUE,
                                    Integer.MIN_VALUE
                                };
    protected int[] tmpMin = {
                                 Integer.MAX_VALUE, Integer.MAX_VALUE,
                                 Integer.MAX_VALUE
                             };
    protected int[] tmpMax = {
                                 Integer.MIN_VALUE, Integer.MIN_VALUE,
                                 Integer.MIN_VALUE
                             };

    /**
     * Constructor
     *
     */
    public ComputeMinMaxFilter() {
        super();
    }

    /**
     * Operaciones que se realizan antes de la ejecucin del filtro
     */
    public void pre() {
        //Obtencin de parmetros comunes a todos
        this.stats = (RasterStats) params.get("stats");
        this.stats.tailPercent = 0D;

        //Operaciones del filtro concreto
        this.min = new int[stats.minBandValue.length];
        this.max = new int[stats.maxBandValue.length];
        this.secondMin = new int[stats.minBandValue.length];
        this.secondMax = new int[stats.maxBandValue.length];
        this.tmpMin = new int[stats.minBandValue.length];
        this.tmpMax = new int[stats.maxBandValue.length];

        for (int i = 0; i < min.length; i++) {
            secondMin[i] = tmpMin[i] = min[i] = Integer.MAX_VALUE;
            secondMax[i] = tmpMax[i] = max[i] = Integer.MIN_VALUE;
        }
    }

    /**
     * Operaciones que se realizan despus de la ejecucin del filtro
     */
    public void post() {
        for (int i = 0; i < 3; i++) {
            stats.minBandValue[i] = min[i];
            stats.maxBandValue[i] = max[i];
            stats.secondMinBandValue[i] = secondMin[i];
            stats.secondMaxBandValue[i] = secondMax[i];
        }
    }

    /**
     * Obtiene le mnimo para una banda dada
     * @param bandNr        banda
     * @return        valor mnimo
     */
    public int getMin(int bandNr) {
        return min[bandNr];
    }

    /**
     * Obtiene el mximo para una banda dada
     * @param bandNr        banda
     * @return        valor mximo
     */
    public int getMax(int bandNr) {
        return max[bandNr];
    }

    /**
     * Obtiene el objeto stats con el calculo de mximos y mnimos
     */
    public Object getResult(String name) {
        if (name.equals("stats")) {
            return stats;
        } else {
            return null;
        }
    }
}
