/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter;


/**
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class PixelFilter {
    public boolean debug = false;
    int transparente = 0x10ffff00;
    int orColor = 0xff000000;
    int umbral = 0xf0f0f0;
    private int alpha = 0xff;

    /**
     * Constructor para el Oleico
     */
    public PixelFilter(int alpha) {
        transparente = 0x10ffff00;
        setAlpha(alpha);

        //orColor = 0xff000000 | (alpha * 0x1000000);
        umbral = 0xf0f0f0;
    }

    /**
     * Constructor.
     *
     * @param trans Color que sustituye a los pixeles filtrados
     * @param or        Mscara que se aplica a los pixeles restantes
     * @param pxMax        Umbral que determina los pixeles.
     */
    public PixelFilter(int trans, int or, int pxMax) {
        transparente = trans;
        setAlpha((orColor >> 24) & 0xff);
        orColor = or;
        umbral = pxMax;
    }

    public void setAlpha(int alpha) {
        this.alpha = alpha;
        orColor = (alpha << 24) & 0xff000000;
    }

    public int getAlpha() {
        return alpha;
    }

    /**
     * Filtra una lnea de pixeles.
     *
     * @param pRGBArray
     */
    public void filterLine(int[] pRGBArray) {
        for (int i = 0; i < pRGBArray.length; i++) {
            /*if (debug)
                    System.out.print(""+i+":"+Integer.toHexString(pRGBArray[i])+",");*/
            if (pRGBArray[i] >= umbral) {
                pRGBArray[i] = transparente;
            } else {
                pRGBArray[i] |= orColor;
            }
        }

        /*if (debug)
                System.out.println("");*/
    }
}
