/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter;

import java.awt.image.BufferedImage;


/**
 * Renderiza el raster.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterToImageFilter extends RasterFilter {
    private BufferedImage image;
    private int alpha;
    private int rgb;

    /**
     * Constructor
     *
     */
    public RasterToImageFilter() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtenemos parmetros
        this.raster = (RasterBuf) params.get("raster");
        this.alpha = ((Integer) params.get("alpha")).intValue();
        this.height = raster.getHeight();
        this.width = raster.getWidth();
        this.alpha = (alpha & 0xff) << 24;
        image = new BufferedImage(raster.getWidth(), raster.getHeight(),
                                  BufferedImage.TYPE_INT_ARGB);
    }

    /* (non-Javadoc)
     * @see org.cresques.px.PxRaster.RasterFilter#process(int, int)
     */
    public void process(int x, int y) {
        raster.getElemInt(x, y, px);
        rgb = (alpha & ((px[3] & 0xff) << 24)) | ((px[0] & 0xff) << 16) |
              ((px[1] & 0xff) << 8) | (px[2] & 0xff);

        image.setRGB(x, y, rgb);
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) image;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.px.PxRaster.RasterFilter#processLines(int)
     */
    public void processLine(int y) {
        for (int x = 0; x < width; x++) {
            int[][] line = raster.getLineInt(y);
            rgb = (alpha & ((px[3] & 0xff) << 24)) |
                  ((line[x][0] & 0xff) << 16) | ((line[x][1] & 0xff) << 8) |
                  (line[x][2] & 0xff);

            image.setRGB(x, y, rgb);
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return raster.getDataType();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }
}
