/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.geo.cover;

import org.cresques.geo.Ellipsoid;
import org.cresques.geo.UtmZone;

import org.cresques.px.Extent;
import org.cresques.px.IObjList;
import org.cresques.px.Minuteo;
import org.cresques.px.PxContour;
import org.cresques.px.PxObj;
import org.cresques.px.PxObjList;
import org.cresques.px.dxf.DxfSolid;
import org.cresques.px.dxf.DxfText;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import java.lang.Character;

import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;


/**
 * Calculos de hojas del Instituto Geogrfico Nacional de Espaa.
 *
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
class Ign {
    /* MTN50: En coordenadas geogrficas: ancho =
     *
     */
}


/**
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class Mtn {
    public static Mtn25 mtn25 = new Mtn25();
    static Mtn50 mtn50 = Mtn.mtn25.getMtn50();
    public static TreeMap hoja25 = Mtn.mtn25.hoja;
    public static TreeMap hoja50 = Mtn.mtn50.hoja;
    UtmZone zone = null;
    String name = null;
    TreeMap hoja = null;
    private Minuteo min = null;

    public Minuteo getMinuteo() {
        return min;
    }

    protected void calcMinuteo() {
        System.out.println(name + ": genero Minuteo");
        min = new Minuteo(zone);

        String hName;
        PxContour contour;
        Iterator keys;
        Hoja h = null;

        keys = hoja.keySet().iterator();

        while (keys.hasNext()) {
            hName = (String) keys.next();
            h = (Hoja) hoja.get(hName);
            contour = new PxContour(h.getVertex(), hName);
            contour.setColor(null);
            min.add(contour);
        }
    }

    public static void parseLayer(PxObjList layer) {
        Mtn25.mtn25.parseLyr(layer);
    }

    public static Minuteo getMinuteo25() {
        return mtn25.getMinuteo();
    }

    public static Minuteo getMinuteo50() {
        return mtn50.getMinuteo();
    }
}


class Mtn50 extends Mtn {
    public Mtn50() {
        name = "MTN50";
        zone = UtmZone.getProjection(Ellipsoid.hayford, 30, UtmZone.NORTH);
        hoja = new TreeMap();
    }
}


class Mtn25 extends Mtn {
    String buf;

    //private TreeMap hojas25 = new TreeMap();
    BufferedReader fi;
    long l = 0;

    public Mtn25() {
        name = "MTN25";
        zone = UtmZone.getProjection(Ellipsoid.hayford, 30, UtmZone.NORTH);
        hoja = new TreeMap();

        /*String fname = "data/mtn25.txt";
        load(fname);
        calcMinuteo();*/
    }

    public Mtn50 getMtn50() {
        Mtn50 mtn50 = new Mtn50();

        Iterator keys = hoja.keySet().iterator();
        Hoja h = null;
        Hoja h50 = null;
        String code50;

        while (keys.hasNext()) {
            h = (Hoja) hoja.get(keys.next());

            //System.err.println("Hoja 25"+h.code);
            code50 = h.code.split("-")[0];

            if (mtn50.hoja.containsKey(code50)) {
                h50 = (Hoja) mtn50.hoja.get(code50);

                switch (Integer.parseInt(h.code.split("-")[1])) {
                case 1:
                    h50.setTL(h.tl);

                    break;

                case 2:
                    h50.setTR(h.tr);

                    break;

                case 3:
                    h50.setBL(h.bl);

                    break;

                case 4:
                    h50.setBR(h.br);

                    break;
                }
            } else {
                h50 = new Hoja(code50, h, null);
            }

            mtn50.hoja.put(code50, h50);
        }

        mtn50.calcMinuteo();

        return mtn50;
    }

    public void load(String name) {
        try {
            System.out.println("Cargando '" + name + "' ...");
            fi = new BufferedReader(new FileReader(name));

            while ((buf = fi.readLine()) != null) {
                //System.out.println(buf);
                l++;
                parseLine(buf);
            }

            fi.close();
            System.out.println("'" + name + "' cargado. (" + l + " lneas).");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException ie) {
            System.err.println("ERROR." + l + "lineas leidas");
            ie.printStackTrace();
        }
    }

    void parseLine(String l) {
        //756-1,[(311114.0, 4337768.0), (325515.0, 4337434.0), (325309.0, 4328185.0), (310891.0, 4328519.0)]
        String[] field = l.split(",");
        field[1] = field[1].substring(1);
        field[8] = field[8].substring(0, field[8].length() - 1);

        String name = field[0];
        String aux;
        double x;
        double y;
        Vector pt = new Vector();

        for (int i = 1; i < 8; i += 2) {
            x = Double.parseDouble(field[i].trim().substring(1));
            aux = field[i + 1].trim();
            y = Double.parseDouble(aux.substring(0, aux.length() - 1));
            pt.add(zone.createPoint(x, y));
        }

        hoja.put(name, new Hoja(name, pt, ""));
        System.out.println(name);
    }

    /*
     * Genera las hojas del MTN25 a partir de una capa dxf.
     * Llena un TreeMap con las hojas, de manera que se pueden
     * obtener por su cdigo.
     */
    private void fillSolids(IObjList marcos, IObjList oList) {
        Iterator iter = oList.iterator();
        ;

        while (iter.hasNext()) {
            Object o = iter.next();

            if (o instanceof IObjList) {
                fillSolids(marcos, (IObjList) o);
            } else if (o instanceof DxfSolid) {
                marcos.add((PxObj) o);
            }
        }
    }

    private void fillTexts(IObjList textos, IObjList oList) {
        Iterator iter = oList.iterator();
        ;

        while (iter.hasNext()) {
            Object o = iter.next();

            if (o instanceof IObjList) {
                fillTexts(textos, (IObjList) o);
            } else if (o instanceof DxfText) {
                //if (Character.isDigit(((DxfText) o).getText().charAt(1)))
                textos.add((Extent.Has) o);
            }
        }
    }

    public void parseLyr(IObjList layer) {
        hoja = new TreeMap();

        PxObjList marcos = new PxObjList();
        PxObjList textos = new PxObjList();
        Iterator iter = null;

        // Lleno una lista con solids, otra con texts.
        System.out.println("layer: " + layer.size() + " objetos (" +
                           layer.iterator().next() + ")");
        fillSolids(marcos, layer);
        System.out.println("fillSolids: " + marcos.size() + " marcos");
        fillTexts(textos, layer);
        System.out.println("fillTexts: " + textos.size() + " textos");

        // Para cada text busco un solid. Si lo encuentro creo
        // una hoja con el solid y el text como name, la mento en
        // el TreeMap y saco el solid de marcos
        Hoja h = null;
        DxfText text = null;
        DxfSolid marco = null;
        IObjList select = null;
        iter = textos.iterator();

        while (iter.hasNext()) {
            text = (DxfText) iter.next();

            if (hoja.containsKey(text.getText())) {
                continue;
            }

            if (!Character.isDigit(text.getText().split("-")[1].charAt(0))) {
                continue;
            }

            select = marcos.getAt(text.getPt1());

            if (select.size() >= 1) {
                marco = (DxfSolid) select.iterator().next();
                h = new Hoja(text.getText(), marco.getPts(), null);
                h.setProjection(layer.getProjection());
                hoja.put(text.getText(), h);
                marcos.remove(marco);
            } else {
                // ERROR. No puede haber ms de un marco por texto 
            }
        }

        calcMinuteo();
        Mtn.mtn50 = Mtn.mtn25.getMtn50();
        Mtn.hoja25 = Mtn.mtn25.hoja;
        Mtn.hoja50 = Mtn.mtn50.hoja;
    }
}
