/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import org.cresques.px.Extent;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.net.MalformedURLException;
import java.net.URL;


/**
 * Consulta para un servidor de mapas.
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public abstract class MapServerClient implements Extent.Has {
    public static final int MAXBUFSIZE = 1024 * 1024;
    BufferedInputStream bufIn = null;
    byte[] buffer;
    int bufPos;
    String serverName;
    protected String urlBase = null;
    protected long xMin;
    protected long xMax;
    protected long yMin;
    protected long yMax;
    protected int ancho;
    protected int alto;
    public boolean echo = false;
    private Extent extent = null;
    private int maxViewWidth = 1500;
    private int maxViewHeight = 1500;
    private float minTimeBetweenQuerys = 0;

    public MapServerClient(String serverName) {
        this.serverName = serverName;
    }

    public String getName() {
        return serverName;
    }

    public byte[] getBuffer() {
        return buffer;
    }

    public void setView(long xMin, long yMin, long xMax, long yMax) {
        this.xMin = xMin;
        this.xMax = xMax;
        this.yMax = yMax;
        this.yMin = yMin;
    }

    public void setViewSize(int w, int h) {
        ancho = w;
        alto = h;
    }

    /**
     * Pone el tamao mximo de la vista. Solicitando una vista ms
     * grande que esta al servidor dar error (y no imgenes)  :-(
     * @param w        ancho
     * @param h alto
     */
    public void setMaxViewSize(int w, int h) {
        maxViewWidth = w;
        maxViewHeight = h;
    }

    public Extent getViewExtent() {
        return new Extent(xMin, yMax, xMax, yMin);
    }

    public void setExtent(Extent e) {
        extent = e;
    }

    public Extent getExtent() {
        return extent;
    }

    abstract public String getUrl();

    public void get() throws Exception {
        get(getUrl());
    }

    public void get(String address) throws Exception {
        bufPos = 0;
        buffer = new byte[MAXBUFSIZE];

        try {
            // Check to see that a command parameter was entered
            // Create an URL instance
            URL url = new URL(address);

            // Get an input stream for reading
            InputStream in = url.openStream();

            // Create a buffered input stream for efficency
            bufIn = new BufferedInputStream(in);

            // Repeat until end of file
            int data;

            for (bufPos = 0; (data = bufIn.read()) != -1; bufPos++) {
                buffer[bufPos] = (byte) data;

                if (echo) {
                    System.out.println(data);
                }
            }

            byte[] buf = new byte[bufPos];

            for (int i = 0; i < bufPos; i++)
                buf[i] = buffer[i];

            buffer = buf;
            System.out.println(bufPos + " bytes leidos.");

            bufIn.close();
            in.close();
        } catch (MalformedURLException mue) {
            System.err.println("Invalid URL");
        } catch (IOException ioe) {
            System.err.println("I/O Error - " + ioe);
        }
    }
}
