/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.cmd;

import org.cresques.ui.CQMapCanvas;

import java.awt.Cursor;
import java.awt.geom.Point2D;

import java.util.TreeMap;


/**
 * Clase Cmd, ancestro de todos los comandos del canvas.
 * Permite aadir al canvas funcionalidades que responden a eventos de
 * ratn y producen canvios en el estado del canvas.
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public abstract class Cmd {
    public static final int LEFT = 0x0001;
    public static final int RIGHT = 0x0002;
    public static final int MIDDLE = 0x0004;
    public static final int PRESS = 0x0008;
    public static final int RELEASE = 0x0010;
    public static final int DRAG = 0x0020;
    public static final int type = 0;
    private static TreeMap cmdPool = null;

    /**
     * Inicializacin de la pool de comandos.
     */
    static {
        cmdPool = new TreeMap();
    }

    CQMapCanvas canvas = null;
    int eventsWanted = 0;
    protected Cursor cursor = null;

    /**
     * Construye un nuevo Cmd para el Canvas
     * @param canvas
     */
    public Cmd(CQMapCanvas canvas) {
        this.canvas = canvas;
    }

    public static void register(String cmdStr, Cmd command) {
        Cmd.cmdPool.put(cmdStr, command);
    }

    public static Cmd get(String cmdStr) {
        return (Cmd) Cmd.cmdPool.get(cmdStr);
    }

    /**
     * Recibe los eventos del ratn.
     */
    abstract public void cmd(Point2D pt, int bt, int mouseEvent);

    /**
     * Devuelve un Bitset con los eventos de raton que requiere.
     * @return eventsWanted (BitSet)
     */
    public int getEventsWanted() {
        return eventsWanted;
    }

    public Cursor getCursor() {
        return cursor;
    }
}
