/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.JComboBox;
import javax.swing.JPanel;

import org.cresques.io.CXMLParser;
import org.cresques.io.GeoRasterWriter;


/*
 * Created on 04-mar-2005
 *
 */

/**
 * Panel que contiene las propiedades de un driver de escritura.
 * Este panel se monta a partir de un XML que es solicitado al driver
 * porque dependiendo del driver tendr unas propiedades u otras.
 * Los elementos del panel son XMLObjects que se almacenan en
 * vectores. El XML se procesa con la clase ProcessXML que ser la
 * encargada a partir del texto en XML de crear los XMLObjects que
 * compondran el panel.
 * @author Nacho Brodin <brodin_ign@gva.es>
 */
public class DataInputProps extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;
    private GeoRasterWriter writer = null;
    private String formato = null;
    private JPanel jPanel = null;
    private JPanel panelSup = null;
    private JComboBox jComboBox = null;
    private XMLPanel[] panels = null;
    private XMLCombo[] combos = null;
    private XMLLabel[] labels = null;
    private XMLSlider[] sliders = null;
    private XMLCheck[] checks = null;
    private XMLText[] texts = null;

    public DataInputProps(){};
    
    public DataInputProps(GeoRasterWriter writer) {
        super();
        this.writer = writer;
        this.formato = writer.getIdent();

        ProcessXML pXML = new ProcessXML(writer.getXMLPropertiesDialog());
        panels = pXML.getPanels();
        labels = pXML.getLabels();
        combos = pXML.getCombos();
        sliders = pXML.getSliders();
        checks = pXML.getChecks();
        texts = pXML.getTexts();
        initialize();
        addComponents();

        //showElements();
    }

    /**
     * Inicializa el panel central con los datos del primer panel de la lista
     */
    private void initialize() {
        for (int i = 0; i < this.panels.length; i++) {
            if (i == 0) {
                if (panels[i].layout.equals("FlowLayout")) {
                    this.setLayout(new FlowLayout());
                }

                if (panels[i].layout.equals("BorderLayout")) {
                    this.setLayout(new BorderLayout());
                }
                this.setPreferredSize(new Dimension(panels[i].sizex, panels[i].sizey));
                this.setSize(panels[i].sizex, panels[i].sizey);
            } else {
                if ((panels[i].position != null) &&
                        !panels[i].position.equals("")) {
                    this.add(panels[i], panels[i].position);
                } else {
                    this.add(panels[i]);
                }
            }
        }
    }

    /**
     * Aade los componentes de los vectores al panel
     *
     */
    public void addComponents() {
        for (int iPanel = 1; iPanel < panels.length; iPanel++) {
            int nElemInsert = 0;

            while (nElemInsert < panels[iPanel].nElements) {
                for (int i = 0; i < labels.length; i++) {
                    if ((labels[i].numPanel == iPanel) &&
                            (labels[i].posIntoPanel == nElemInsert)) {
                        if ((labels[i].position != null) &&
                                !labels[i].position.equals("")) {
                            panels[iPanel].add(labels[i], labels[i].position);
                        } else {
                            panels[iPanel].add(labels[i]);
                        }

                        nElemInsert++;
                    }
                } //for(int i=0;i<labels.length;i++)

                for (int i = 0; i < combos.length; i++) {
                    if ((combos[i].numPanel == iPanel) &&
                            (combos[i].posIntoPanel == nElemInsert)) {
                        if ((combos[i].position != null) &&
                                !combos[i].position.equals("")) {
                            panels[iPanel].add(combos[i], combos[i].position);
                        } else {
                            panels[iPanel].add(combos[i]);
                        }

                        nElemInsert++;
                    }
                } //for(int i=0;i<combos.length;i++)

                for (int i = 0; i < sliders.length; i++) {
                    if ((sliders[i].numPanel == iPanel) &&
                            (sliders[i].posIntoPanel == nElemInsert)) {
                        if ((sliders[i].position != null) &&
                                !sliders[i].position.equals("")) {
                            panels[iPanel].add(sliders[i], sliders[i].position);
                        } else {
                            panels[iPanel].add(sliders[i]);
                        }

                        nElemInsert++;
                    }
                } //for(int i=0;i<sliders.length;i++)

                for (int i = 0; i < checks.length; i++) {
                    if ((checks[i].numPanel == iPanel) &&
                            (checks[i].posIntoPanel == nElemInsert)) {
                        if ((checks[i].position != null) &&
                                !checks[i].position.equals("")) {
                            panels[iPanel].add(checks[i], checks[i].position);
                        } else {
                            panels[iPanel].add(checks[i]);
                        }

                        nElemInsert++;
                    }
                } //for(int i=0;i<checks.length;i++)

                for (int i = 0; i < texts.length; i++) {
                    if ((texts[i].numPanel == iPanel) &&
                            (texts[i].posIntoPanel == nElemInsert)) {
                        if ((texts[i].position != null) &&
                                !texts[i].position.equals("")) {
                            panels[iPanel].add(texts[i], texts[i].position);
                        } else {
                            panels[iPanel].add(texts[i]);
                        }

                        nElemInsert++;
                    }
                } //for(int i=0;i<checks.length;i++)
            } //while(nElemInsert<panels[iPanel].nElements)
        } //for(int iPanel=1;iPanel<panels.length;iPanel++)
    }

    /**
     * Obtiene el nmero de componentes que recogen propiedades
     * para el driver cargado
     * @return        Nmero de componentes
     */
    public int getNElements() {
        return (combos.length + sliders.length + checks.length + texts.length);
    }

    /**
     * Obtiene los identificadores de las propiedades del cuadro de dialogo
     * @return        Lista con los identificadores de propiedades
     */
    public String[] getElements() {
        String[] lista = new String[this.getNElements()];

        for (int i = 0; i < combos.length; i++)
            lista[i] = combos[i].id + "=" +
                       combos[i].getSelectedItem().toString();

        for (int i = 0; i < sliders.length; i++)
            lista[combos.length + i] = sliders[i].id + "=" +
                                       sliders[i].getSlider().getValue();

        for (int i = 0; i < checks.length; i++) {
            String sel = null;

            if (checks[i].isSelected()) {
                sel = new String("yes");
            } else {
                sel = new String("no");
            }

            lista[combos.length + sliders.length + i] = checks[i].id + "=" +
                                                        sel;
        }

        for (int i = 0; i < texts.length; i++)
            lista[i] = texts[i].id + "=" + texts[i].getText();

        return lista;
    }

    /**
     * Obtiene la lista de Combos
     * @return XMLCombo[]
     */
    public XMLCombo[] getCombos() {
        return combos;
    }

    /**
     * Obtiene la lista de Sliders
     * @return XMLSlider[]
     */
    public XMLSlider[] getSliders() {
        return sliders;
    }

    /**
     * Obtiene la lista de CheckBoxs
     * @return XMLCheck[]
     */
    public XMLCheck[] getChecks() {
        return checks;
    }

    /**
     * Muestra informacin sobre los elementos aadidos al panel
     *
     */
    public void showElements() {
        for (int i = 0; i < panels.length; i++)
            System.out.println("PANEL " + i + " NElem=" + panels[i].nElements);

        for (int i = 0; i < labels.length; i++)
            System.out.println("LABEL " + i + " Panel=" + labels[i].numPanel +
                               " PosIntoPanel=" + labels[i].posIntoPanel);

        for (int i = 0; i < combos.length; i++)
            System.out.println("COMBO " + i + " Panel=" + combos[i].numPanel +
                               " PosIntoPanel=" + combos[i].posIntoPanel);

        for (int i = 0; i < checks.length; i++)
            System.out.println("CHECK " + i + " Panel=" + checks[i].numPanel +
                               " PosIntoPanel=" + checks[i].posIntoPanel);

        for (int i = 0; i < sliders.length; i++)
            System.out.println("SLIDER " + i + " Panel=" + sliders[i].numPanel +
                               " PosIntoPanel=" + sliders[i].posIntoPanel);
    }
}


/**
 * Clase para procesar el XML que manda un driver de escritura y
 * que corresponde a la ventana de dialogo para las propiedades del mismo.
 *
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 */
class ProcessXML {
    private XMLPanel[] panels = null;
    private XMLCombo[] combos = null;
    private XMLLabel[] labels = null;
    private XMLSlider[] sliders = null;
    private XMLCheck[] checks = null;
    private XMLText[] texts = null;
    private int contPanels = 0;
    private int contCombos = 0;
    private int contLabels = 0;
    private int contSliders = 0;
    private int contChecks = 0;
    private int contTexts = 0;

    /**
     * Constructor
     * @param XML        XML correspondiente a la ventana de dialogo
     */
    public ProcessXML(String XML) {
        CXMLParser parser = new CXMLParser(XML);

        //Reservamos memoria para el nmero de paneles
        panels = new XMLPanel[parser.getNumTags("panel")];
        combos = new XMLCombo[parser.getNumTags("combo")];
        labels = new XMLLabel[parser.getNumTags("label")];
        sliders = new XMLSlider[parser.getNumTags("slider")];
        checks = new XMLCheck[parser.getNumTags("check")];
        texts = new XMLText[parser.getNumTags("text")];

        String valor = parser.getValue("window", true);

        while (!valor.equals("")) {
            String tag = parser.getTag();

            if (tag.equals("panel")) {
                processPanel(parser);
            } else if (tag.equals("label")) {
                processLabel(parser);
            } else if (tag.equals("combo")) {
                processCombo(parser);
            } else if (tag.equals("check")) {
                processCheck(parser);
            } else if (tag.equals("slider")) {
                processSlider(parser);
            } else if (tag.equals("text")) {
                processText(parser);
            } else if (tag.equals("")) {
                valor = new String("");
            } else {
                valor = parser.getXML();
            }
        }

        if (!valor.equals("")) {
            System.err.println("Se ha interrumpido el proceso!!!");
        }
    }

    /**
     * Obtiene los paneles leidos del XML como objetos XMLPanel
     * @return        Listado de XMLPanel
     */
    public XMLPanel[] getPanels() {
        return panels;
    }

    /**
     * Obtiene los combos leidos del XML como objetos XMLCombo
     * @return        Listado de XMLCombo
     */
    public XMLCombo[] getCombos() {
        return combos;
    }

    /**
     * Obtiene los label leidos del XML como objetos XMLLabel
     * @return        Listado de XMLLabel
     */
    public XMLLabel[] getLabels() {
        return labels;
    }

    /**
     * Obtiene los slider leidos del XML como objetos XMLSlider
     * @return        Listado de XMLSlider
     */
    public XMLSlider[] getSliders() {
        return sliders;
    }

    /**
     * Obtiene los checks leidos del XML como objetos XMLCheks
     * @return        Listado de XMLCheck
     */
    public XMLCheck[] getChecks() {
        return checks;
    }

    /**
     * Obtiene los texts leidos del XML como objetos XMLText
     * @return        Listado de XMLText
     */
    public XMLText[] getTexts() {
        return texts;
    }

    /**
     * Procesa un tag panel
     * @param parser        Objeto parser con el XML
     */
    public void processPanel(CXMLParser parser) {
        panels[contPanels] = new XMLPanel();

        String sizex = parser.getAttr("panel", "sizex");
        String sizey = parser.getAttr("panel", "sizey");

        if ((sizex != null) && !sizex.equals("")) {
            panels[contPanels].sizex = Integer.parseInt(sizex);
        }

        if ((sizey != null) && !sizey.equals("")) {
            panels[contPanels].sizey = Integer.parseInt(sizey);
        }

        panels[contPanels].layout = parser.getAttr("panel", "layout");
        panels[contPanels].border = parser.getAttr("panel", "border");
        panels[contPanels].position = parser.getAttr("panel", "position");
        panels[contPanels].align = parser.getAttr("panel", "align");
        panels[contPanels].init();
        contPanels++;

        //System.out.println("PANEL="+sizex+"X"+sizey+" Layout="+layout+" Border="+border+" Pos="+position+" Align="+align);
        parser.getValue("panel", true);
    }

    /**
         * Procesa un tag label
         * @param parser        Objeto parser con el XML
         */
    public void processLabel(CXMLParser parser) {
        labels[contLabels] = new XMLLabel();

        //Posicionamiento. En q panel y su posicin dentro de este
        labels[contLabels].numPanel = contPanels - 1;
        panels[contPanels - 1].nElements++;
        labels[contLabels].posIntoPanel = panels[contPanels - 1].nElements - 1;

        labels[contLabels].position = parser.getAttr("label", "position");
        labels[contLabels].text = parser.getValue("label", true);
        labels[contLabels].init();
        contLabels++;
    }

    /**
     * Procesa un tag combo
     * @param parser        Objeto parser con el XML
     */
    public void processCombo(CXMLParser parser) {
        combos[contCombos] = new XMLCombo();

        //Posicionamiento. En q panel y su posicin dentro de este
        combos[contCombos].numPanel = contPanels - 1;
        panels[contPanels - 1].nElements++;
        combos[contCombos].posIntoPanel = panels[contPanels - 1].nElements - 1;

        combos[contCombos].selected = parser.getAttr("combo", "selected");
        combos[contCombos].position = parser.getAttr("combo", "position");
        combos[contCombos].id = parser.getAttr("combo", "ident");

        String sizex = parser.getAttr("combo", "sizex");
        String sizey = parser.getAttr("combo", "sizey");

        if ((sizex != null) && !sizex.equals("")) {
            combos[contCombos].sizex = Integer.parseInt(sizex);
        }

        if ((sizey != null) && !sizey.equals("")) {
            combos[contCombos].sizey = Integer.parseInt(sizey);
        }

        String valoresCombo = parser.getValue("combo", true);

        processComboValues(parser, combos[contCombos]);
        combos[contCombos].init();
        contCombos++;
    }

    /**
     * Procesa los elementos de un combo
     * @param parser        Objeto parser con el XML
     * @param combo        Objeto XMLCombo del cual se procesan los elementos
     */
    public void processComboValues(CXMLParser parser, XMLCombo combo) {
        String elem = parser.getValue("elem", true);

        if (elem != null) {
            combo.addItem(elem);
        }

        if (parser.getTag().equals("elem")) {
            processComboValues(parser, combo);
        }
    }

    /**
     * Procesa un tag check
     * @param parser        Objeto parser con el XML
     */
    public void processCheck(CXMLParser parser) {
        String select = parser.getAttr("check", "selected");

        if (select.equals("yes")) {
            checks[contChecks] = new XMLCheck(parser.getAttr("check", "text"),
                                              true);
        } else {
            checks[contChecks] = new XMLCheck(parser.getAttr("check", "text"),
                                              false);
        }

        checks[contChecks].position = parser.getAttr("check", "position");
        checks[contChecks].id = parser.getAttr("check", "ident");

        String sizex = parser.getAttr("check", "sizex");
        String sizey = parser.getAttr("check", "sizey");

        if ((sizex != null) && !sizex.equals("")) {
            checks[contChecks].sizex = Integer.parseInt(sizex);
        }

        if ((sizey != null) && !sizey.equals("")) {
            checks[contChecks].sizey = Integer.parseInt(sizey);
        }

        parser.getValue("check", true);

        //Posicionamiento. En q panel y su posicin dentro de este
        checks[contChecks].numPanel = contPanels - 1;
        panels[contPanels - 1].nElements++;
        checks[contChecks].posIntoPanel = panels[contPanels - 1].nElements - 1;

        checks[contChecks].init();
        contChecks++;
    }

    /**
     * Procesa un tag slider
     * @param parser        Objeto parser con el XML
     */
    public void processSlider(CXMLParser parser) {
        String min = null;
        String max = null;
        String value = null;
        String minorspacing = null;
        String majorspacing = null;
        String name = parser.getAttr("slider", "name");
        String id = parser.getAttr("slider", "ident");
        String position = parser.getAttr("slider", "position");
        String sizex = parser.getAttr("slider", "sizex");
        String sizey = parser.getAttr("slider", "sizey");
        String elem = parser.getValue("slider", true);

        String tag = parser.getTag();

        while (tag.equals("min") || tag.equals("max") || tag.equals("value") ||
                   tag.equals("minorspacing") || tag.equals("majorspacing")) {
            if (tag.equals("min")) {
                min = parser.getValue("min", true);
            }

            if (tag.equals("max")) {
                max = parser.getValue("max", true);
            }

            if (tag.equals("value")) {
                value = parser.getValue("value", true);
            }

            if (tag.equals("minorspacing")) {
                minorspacing = parser.getValue("minorspacing", true);
            }

            if (tag.equals("majorspacing")) {
                majorspacing = parser.getValue("majorspacing", true);
            }

            tag = parser.getTag();
        }

        sliders[contSliders] = new XMLSlider(name, min, max, value,
                                             minorspacing, majorspacing);
        sliders[contSliders].position = position;
        sliders[contSliders].id = id;

        if ((sizex != null) && !sizex.equals("")) {
            sliders[contSliders].sizex = Integer.parseInt(sizex);
        }

        if ((sizey != null) && !sizey.equals("")) {
            sliders[contSliders].sizey = Integer.parseInt(sizey);
        }

        //Posicionamiento. En q panel y su posicin dentro de este
        sliders[contSliders].numPanel = contPanels - 1;
        panels[contPanels - 1].nElements++;
        sliders[contSliders].posIntoPanel = panels[contPanels - 1].nElements -
                                            1;

        contSliders++;
    }

    /**
    * Procesa un tag text
    * @param parser        Objeto parser con el XML
    */
    public void processText(CXMLParser parser) {
        texts[contTexts] = new XMLText(parser.getAttr("text", "text"));

        String sizex = parser.getAttr("text", "sizex");
        String sizey = parser.getAttr("text", "sizey");

        if ((sizex != null) && !sizex.equals("")) {
            texts[contTexts].sizex = Integer.parseInt(sizex);
        }

        if ((sizey != null) && !sizey.equals("")) {
            texts[contTexts].sizey = Integer.parseInt(sizey);
        }

        //Posicionamiento. En q panel y su posicin dentro de este
        texts[contTexts].numPanel = contPanels - 1;
        panels[contPanels - 1].nElements++;
        texts[contTexts].posIntoPanel = panels[contPanels - 1].nElements - 1;

        texts[contTexts].position = parser.getAttr("text", "position");
        texts[contTexts].text = parser.getValue("text", true);
        texts[contTexts].init();
        contTexts++;
    }
}


/**
 * Clase que representa a un panel obtenido a partir del XML
 * que manda un driver de escritura.
 *
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 */
class XMLPanel extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;
    public int sizex = 0;
    public int sizey = 0;
    public String layout = null;
    public String border = null;
    public String position = null;
    public String align = null;
    public int nElements = 0;

    /**
     * Inicializacin del XMLPanel
     *
     */
    public void init() {
        if (layout.equals("FlowLayout")) {
            FlowLayout fl = new FlowLayout();
            this.setLayout(fl);

            if (align.equals("left")) {
                fl.setAlignment(java.awt.FlowLayout.LEFT);
            } else if (align.equals("center")) {
                fl.setAlignment(java.awt.FlowLayout.CENTER);
            } else if (align.equals("right")) {
                fl.setAlignment(java.awt.FlowLayout.RIGHT);
            } else if (align.equals("leading")) {
                fl.setAlignment(java.awt.FlowLayout.LEADING);
            } else if (align.equals("trailing")) {
                fl.setAlignment(java.awt.FlowLayout.TRAILING);
            }
        } else if (layout.equals("BorderLayout")) {
            this.setLayout(new BorderLayout());
        }

        if ((sizex != 0) && (sizey != 0)) {
            this.setPreferredSize(new java.awt.Dimension(sizex, sizey));
        }

        if (border.equals("yes")) {
            this.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,
                                                                      1));
        }
    }
}








