package org.cresques.ui.raster;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;

/**
 * Clase que representa a un slider obtenido a partir del XML
 * que manda un driver de escritura.
 *
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 */
public class XMLSlider extends JPanel {
    final private static long serialVersionUID = -3370601314380922368L;
    public int numPanel = -1;
    public int posIntoPanel = -1;
    public int sizex = 0;
    public int sizey = 0;
    public String position = null;
    private JSlider slider = null;
    private JLabel label = null;
    public String id = null;

    /**
     * Constructor
     * @param text        Texto de la barra
     * @param smin        Valor mnimo de la barra
     * @param smax        Valor mximo de la barra
     * @param sdefaultValue        Valor por defecto de la barra.
     * @param sminor        Separacin menor de las secciones de la barra
     * @param smajor        Separacin mayor de las secciones de la barra
     */
    public XMLSlider(String text, String smin, String smax,
                     String sdefaultValue, String sminor, String smajor) {
        int min = 0;
        int max = 100;
        int defaultValue = 0;
        int minor = 5;
        int major = 10;

        //Obtenemos los valores de los parmetros
        if (smin != null) {
            min = Integer.parseInt(smin);
        }

        if (smax != null) {
            max = Integer.parseInt(smax);
        }

        if (sdefaultValue != null) {
            defaultValue = Integer.parseInt(sdefaultValue);
        }

        if (sminor != null) {
            minor = Integer.parseInt(sminor);
        }

        if (smajor != null) {
            major = Integer.parseInt(smajor);
        }

        //Colocacin de elementos en el panel
        label = new JLabel(text, JLabel.CENTER);
        slider = new JSlider(JSlider.HORIZONTAL, min, max, defaultValue);
        slider.setMajorTickSpacing(major);
        slider.setMinorTickSpacing(minor);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        this.add(label);
        this.add(slider);

        if ((sizex != 0) && (sizey != 0)) {
            this.setPreferredSize(new java.awt.Dimension(sizex, sizey));
        }
    }

    /**
     * Obtiene la barra
     * @return        Objeto JSlider
     */
    public JSlider getSlider() {
        return slider;
    }
}


