/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.drivers.shp;

import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.drivers.shp.write.SHPFileWrite;
import com.iver.cit.gvsig.fmap.drivers.shp.write.ShapefileException;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import java.net.MalformedURLException;

import java.nio.channels.FileChannel;
import java.nio.channels.WritableByteChannel;


/**
 * DOCUMENT ME!
 *
 * @author Vicente Caballero Navarro
 */
public class SHPSHXFromGeometries {
	private IGeometry[] geometries = null;
	private String shpPath;
	private String shxPath;
	private int temp = 0;
	private int type;

	/**
	 * Crea un nuevo SHPSHXFromGeometries.
	 *
	 * @param geometries DOCUMENT ME!
	 * @param file DOCUMENT ME!
	 */
	public SHPSHXFromGeometries(IGeometry[] geometries, File file) {
		this.geometries = geometries;
		setFile(file);
	}

	//private FBitSet bitset = null;
	//private int num = 0;

	/**
	 * DOCUMENT ME!
	 *
	 * @param f DOCUMENT ME!
	 */
	private void setFile(File f) {
		shpPath = f.getAbsolutePath();

		String strFichshx = f.getAbsolutePath().replaceAll("\\.shp", ".shx");
		shxPath = strFichshx.replaceAll("\\.SHP", ".SHX");
	}

	/**
	 * DOCUMENT ME!
	 */

	/*public void setType(int i) {
	   type = i;
	   }
	 */

	/**
	 * Finaliza el visitor.
	 */
	public void create() {
		SHPFileWrite filewrite;

		try {
			filewrite = new SHPFileWrite((FileChannel) getWriteChannel(shpPath),
					(FileChannel) getWriteChannel(shxPath));

			if (geometries.length > 0) {
				type = getTypeShape(geometries[0].getGeometryType());
				filewrite.write(geometries, type);
			}
		} catch (ShapefileException e1) {
			e1.printStackTrace();
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		System.out.println("Acabado SHP y SHX");
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param path DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 *
	 * @throws IOException DOCUMENT ME!
	 */
	private WritableByteChannel getWriteChannel(String path)
		throws IOException {
		WritableByteChannel channel;

		File f = new File(path);

		if (!f.exists()) {
			System.out.println("Creando fichero " + f.getAbsolutePath());

			if (!f.createNewFile()) {
				System.err.print("Error al crear el fichero " +
					f.getAbsolutePath());
				throw new IOException("Cannot create file " + f);
			}
		}

		RandomAccessFile raf = new RandomAccessFile(f, "rw");
		channel = raf.getChannel();

		return channel;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param geometryType DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	private int getTypeShape(int geometryType) {
	if (geometryType>=FShape.Z){
		switch (geometryType - FShape.Z) {
		case (FShape.POINT):
			return FConstant.SHAPE_TYPE_POINTZ;

		case (FShape.LINE):
			return FConstant.SHAPE_TYPE_POLYLINEZ;

		case FShape.POLYGON:
			return FConstant.SHAPE_TYPE_POLYGONZ;

		case FShape.MULTIPOINT:
			return FConstant.SHAPE_TYPE_MULTIPOINTZ; //TODO falta aclarar cosas aqu.
	}

	}else{
		switch (geometryType) {
			case FShape.POINT:
				return FConstant.SHAPE_TYPE_POINT;

			case FShape.LINE:
				return FConstant.SHAPE_TYPE_POLYLINE;

			case FShape.POLYGON:
				return FConstant.SHAPE_TYPE_POLYGON;

			case FShape.MULTIPOINT:
				return FConstant.SHAPE_TYPE_MULTIPOINT; //TODO falta aclarar cosas aqu.
		}
	}
		return FConstant.SHAPE_TYPE_NULL;
	}
}
