package com.iver.cit.gvsig.fmap.edition;

import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.hardcode.gdbms.engine.values.DoubleValue;
import com.hardcode.gdbms.engine.values.IntValue;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.hardcode.gdbms.engine.values.StringValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.core.v02.FLabel;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.BoundedShapes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLyrAnnotation;
import com.iver.cit.gvsig.fmap.layers.MappingAnnotation;
import com.iver.cit.gvsig.fmap.operations.strategies.AnnotationStrategy;
import com.iver.cit.gvsig.fmap.rendering.Legend;
import com.iver.cit.gvsig.fmap.rendering.VectorialUniqueValueLegend;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.index.quadtree.Quadtree;

public class AnnotationEditableAdapter extends VectorialEditableAdapter{
	private ArrayList labels;
	private MappingAnnotation mapping;
	private Legend legend;
	private boolean isInPixels=true;
	private FLyrAnnotation lyrAnnotation;
	private LabelExpansion labelExpansion=new LabelExpansion();

	public FLabel createLabel(IRow row){
		FSymbol symbol;
		// El mapping[0] es el text
		int fieldId = mapping.getColumnText();
		// El mapping[1] es el ngulo
		int idFieldRotationText = mapping.getColumnRotate();
		// El mapping[2] es el color
		int idFieldColorText = mapping.getColumnColor();
		// El mapping[3] es el alto
		int idFieldHeightText = mapping.getColumnHeight();
		// El mapping[4] es el tipo de fuente
		int idFieldTypeFontText = mapping.getColumnTypeFont();
		// El mapping[5] es el estilo de fuente
		int idFieldStyleFontText = mapping.getColumnStyleFont();

		IGeometry geom = ((IFeature) row).getGeometry();
		String t = new String();
		Value val = row.getAttribute(fieldId);
		t = val.toString();
		if (idFieldColorText != -1) {
			DoubleValue valColor = (DoubleValue) row
					.getAttribute(idFieldColorText);
			t = t.concat(valColor.toString());
		}
		if (idFieldTypeFontText != -1) {
			StringValue valTypeFont = (StringValue) row
					.getAttribute(idFieldTypeFontText);
			t = t.concat(valTypeFont.toString());
		}
		if (idFieldStyleFontText != -1) {
			IntValue valStyleFont = (IntValue) row
					.getAttribute(idFieldStyleFontText);
			t = t.concat(valStyleFont.toString());
		}

		Value total = ValueFactory.createValue(t);

		FLabel[] lbls = geom.createLabels(0, true);
		double rotat = 0;
		FSymbol sym = (FSymbol) legend.getDefaultSymbol();
		for (int j = 0; j < lbls.length; j++) {
			if (lbls[j] != null) {
				lbls[j].setString(val.toString());
				if (idFieldRotationText != -1) {
					DoubleValue rotation = (DoubleValue) row
							.getAttribute(idFieldRotationText);
					rotat = rotation.getValue();
				} else {
					rotat = sym.getRotation();
				}
				lbls[j].setRotation(rotat);

				float height;
				if (idFieldHeightText != -1) {
					NumericValue h = (NumericValue) row
							.getAttribute(idFieldHeightText);
					height = h.floatValue();
					lbls[j].setHeight(height);
				} else {
					height = sym.getFontSize();
					lbls[j].setHeight(height);
				}
				VectorialUniqueValueLegend vuvl = (VectorialUniqueValueLegend) legend;
				if (vuvl.getSymbolByValue(total) == null) {
					Color color;
					if (idFieldColorText != -1) {
						DoubleValue c = (DoubleValue) row
								.getAttribute(idFieldColorText);
						color = new Color((int) c.getValue());
					} else {
						color = sym.getFontColor();
					}
					String typeFont;
					if (idFieldTypeFontText != -1) {
						StringValue tf = (StringValue) row
								.getAttribute(idFieldTypeFontText);
						typeFont = tf.getValue();
					} else {
						typeFont = sym.getFont()
								.getFontName();
					}
					int style;
					if (idFieldStyleFontText != -1) {
						IntValue sf = (IntValue) row
								.getAttribute(idFieldStyleFontText);
						style = sf.getValue();
					} else {
						style = sym.getFont().getStyle();
					}
					symbol = new FSymbol(FConstant.SYMBOL_TYPE_TEXT);
					symbol.setFontSizeInPixels(isInPixels);
					symbol.setFont(new Font(typeFont, style, (int) height));
					symbol.setDescription(lbls[j].getString());
					symbol.setFontColor(color);
					vuvl.addSymbol(total, symbol);
				}

			}
			return lbls[0];
		}
		return null;
	}




	public int doAddRow(IRow feat, int sourceType) throws DriverIOException, IOException {
		boolean cancel = fireBeforeRowAdded(sourceType);
		if (cancel)
			return -1;
		Value[] values=feat.getAttributes();
		values[mapping.getColumnText()]=ValueFactory.createValue("New");
		FLabel label=createLabel(feat);
		//Rectangle2D r=((DefaultFeature)feat).getGeometry().getBounds2D();
		//r.setRect(r.getX()-r.getWidth()*4,r.getY()-r.getHeight()*4,r.getWidth()*10,r.getHeight()*4);
		//label.setBoundBox(r);
		AnnotationStrategy strategy = (AnnotationStrategy)lyrAnnotation.getStrategy();
		Rectangle2D r=strategy.getBoundBox(label.getOrig(),(float)label.getHeight(),label.getJustification(),label.getString());
        label.setBoundBox(r);
		IGeometry geom=getGeometry(label.getBoundBox());
		feat=new DefaultFeature(geom,values,feat.getID());

		///lyrAnnotation.deleteSpatialIndex();

		int calculatedIndex = -1;
		calculatedIndex = (int) getRowCount()+delRows.cardinality() ;

		int pos = expansionFile.addRow(feat, IRowEdited.STATUS_ADDED, actualIndexFields);
		labelExpansion.addLabel(label);
		relations.put(new Integer(calculatedIndex), new Integer(pos));
		numAdd++;

		index.insert(new Envelope(r.getX(), r.getX() + r.getWidth(), r.getY(),
                r.getY() + r.getHeight()), new Integer(calculatedIndex));
        //lyrAnnotation.getFMap().invalidate();
        fireAfterRowAdded(feat,calculatedIndex, sourceType);
        return calculatedIndex;
	}

	public int doModifyRow(int calculatedIndex, IRow feat,int sourceType) throws IOException, DriverIOException {
		boolean cancel = fireBeforeModifyRow(feat,calculatedIndex, sourceType);
		if (cancel)
			return -1;
		int posAnteriorInExpansionFile = -1;
		Integer integer = new Integer(calculatedIndex);

		System.err.println("Modifica una Row en la posicin: "
				+ calculatedIndex);
		// Si la geometra no ha sido modificada
		if (!relations.containsKey(integer)) {
			FLabel label=(FLabel)getLabel(calculatedIndex,true).clone();
			Value value=feat.getAttribute(mapping.getColumnText());
			Rectangle2D rLabelAnt=(Rectangle2D)label.getBoundBox().clone();
			label.setString(value.toString());

			if (mapping.getColumnRotate()==mapping.getColumnText()){
				label.setRotation(((NumericValue)value).doubleValue());
			} //Aqu hay que comprobar, si se quiere, el resto de columnas, si son iguales a la de Texto.


			int newPosition = expansionFile.addRow(feat, IRowEdited.STATUS_MODIFIED, actualIndexFields);
			relations.put(integer, new Integer(newPosition));

			// Se actualiza el ndice espacial
			IGeometry g=((IFeature) feat).getGeometry();
			double[] d=new double[4];
			//Rectangle2D r = g.getBounds2D();
			g.getPathIterator(null).currentSegment(d);
			Point2D p=new Point2D.Double(d[0],d[1]);
			label.setBoundBox(new Rectangle2D.Double(p.getX(),p.getY(),rLabelAnt.getWidth(),rLabelAnt.getHeight()));
			Rectangle2D rLabel=(Rectangle2D)label.getBoundBox();
			label.setOrig(p);
			index.remove(new Envelope(rLabelAnt.getX(), rLabelAnt.getX()
					+ rLabelAnt.getWidth(), rLabelAnt.getY(), rLabelAnt.getY()
					+ rLabelAnt.getHeight()), new Integer(calculatedIndex));
			index.insert(new Envelope(rLabel.getX(), rLabel.getX() + rLabel.getWidth(), rLabel
					.getY(), rLabel.getY() + rLabel.getHeight()), new Integer(
					calculatedIndex));
			labelExpansion.addLabel(label);
		} else {
			FLabel label=(FLabel)labelExpansion.getLabel(((Integer)relations.get(new Integer(calculatedIndex))).intValue()).clone();
			Value value=feat.getAttribute(mapping.getColumnText());
			Rectangle2D rLabelAnt=(Rectangle2D)label.getBoundBox().clone();
			label.setString(value.toString());

			if (mapping.getColumnRotate()==mapping.getColumnText()){
				label.setRotation(((NumericValue)value).doubleValue());
			} //Aqu hay que comprobar, si se quiere, el resto de columnas, si son iguales a la de Texto.


			// Obtenemos el ndice en el fichero de expansin
			int num = ((Integer) relations.get(integer)).intValue();
			posAnteriorInExpansionFile = num;

			/*
			 * Se modifica la geometra y nos guardamos el ndice dentro del
			 * fichero de expansin en el que se encuentra la geometra
			 * modificada
			 */
			num = expansionFile.modifyRow(num, feat, actualIndexFields);

			/*
			 * Actualiza la relacin del ndice de la geometra al ndice en el
			 * fichero de expansin.
			 */
			relations.put(integer, new Integer(num));

			Rectangle2D r = ((IFeature) feat).getGeometry().getBounds2D();

			Point2D p=new Point2D.Double(r.getX(),r.getY());

			label.setBoundBox(new Rectangle2D.Double(p.getX(),p.getY()+rLabelAnt.getHeight(),rLabelAnt.getWidth(),rLabelAnt.getHeight()));
			Rectangle2D rLabel=(Rectangle2D)label.getBoundBox();
			label.setOrig(p);
			index.remove(new Envelope(rLabelAnt.getX(), rLabelAnt.getX()
					+ rLabelAnt.getWidth(), rLabelAnt.getY(), rLabelAnt.getY()
					+ rLabelAnt.getHeight()), new Integer(calculatedIndex));
			index.insert(new Envelope(rLabel.getX(), rLabel.getX() + rLabel.getWidth(), rLabel
					.getY(), rLabel.getY() + rLabel.getHeight()), new Integer(
					calculatedIndex));
			labelExpansion.modifyLabel(num,label);
		}
		fireAfterModifyRow(calculatedIndex,sourceType);
		return posAnteriorInExpansionFile;
	}

	public IRow doRemoveRow(int index,int sourceType) throws DriverIOException, IOException {
		boolean cancel = fireBeforeRemoveRow(index, sourceType);
		if (cancel)
			return null;
		//Llega el calculatedIndex
    	Integer integer = new Integer(index);

        IFeature feat = null;

        FLabel label=getLabel(index,true);
        delRows.set(index, true);
        //Si la geometra no ha sido modificada
        if (!relations.containsKey(integer)) {

            try {
				feat = (DefaultFeature) (ova.getFeature(index));
			} catch (DriverException e) {
				throw new DriverIOException(e);
			}
       } else {
			int num = ((Integer) relations.get(integer)).intValue();
			feat = (IFeature) expansionFile.getRow(num).getLinkedRow();
		}
        System.err.println("Elimina una Row en la posicin: " + index);
        //Se actualiza el ndice
        if (feat != null) {
            Rectangle2D r = label.getBoundBox();
            this.index.remove(new Envelope(r.getX(), r.getX() + r.getWidth(),
                    r.getY(), r.getY() + r.getHeight()), new Integer(index));
        }
        setSelection(new FBitSet());
        fireAfterRemoveRow(index, sourceType);
        return feat;
	}

	public void undoAddRow(int calculatedIndex, int sourceType) throws DriverIOException, IOException {
		boolean cancel = fireBeforeRemoveRow(calculatedIndex, sourceType);
		if (cancel)
			return;
		Rectangle2D r = getLabel(calculatedIndex,true).getBoundBox();
        this.index.remove(new Envelope(r.getX(), r.getX() + r.getWidth(),
                r.getY(), r.getY() + r.getHeight()), new Integer(calculatedIndex));
        expansionFile.deleteLastRow();
		relations.remove(new Integer(calculatedIndex));
		numAdd--;
        setSelection(new FBitSet());
		labelExpansion.deleteLastLabel();
		fireAfterRemoveRow(calculatedIndex, sourceType);
	}

	public void undoModifyRow(int calculatedIndex, int previousExpansionFileIndex, int sourceType) throws IOException, DriverIOException {

		if (previousExpansionFileIndex == -1) {
	            //Se obtiene la geometra para actualizar el ndice
	        	int inverse = getInversedIndex(calculatedIndex);
	        	DefaultFeature df=(DefaultFeature)getRow(inverse).getLinkedRow();


	        	IGeometry g = df.getGeometry();
	            Rectangle2D r = g.getBounds2D();
	            //Se elimina de las relaciones y del fichero de expansin
	            relations.remove(new Integer(calculatedIndex));
	            expansionFile.deleteLastRow();
	            labelExpansion.deleteLastLabel();
	            //Se actualizan los ndices
	            DefaultFeature dfAnt=(DefaultFeature)getRow(inverse).getLinkedRow();
	            IGeometry gAnt = dfAnt.getGeometry();
	            boolean cancel = fireBeforeModifyRow(dfAnt,calculatedIndex, sourceType);
	    		if (cancel)
	    			return;
	            Rectangle2D rAnt = gAnt.getBounds2D();
	            this.index.remove(new Envelope(r.getX(), r.getX() + r.getWidth(),
	                    r.getY(), r.getY() + r.getHeight()),
	                new Integer(calculatedIndex));
	            this.index.insert(new Envelope(rAnt.getX(),
	                    rAnt.getX() + rAnt.getWidth(), rAnt.getY(),
	                    rAnt.getY() + rAnt.getHeight()), new Integer(calculatedIndex));


	        } else {
	            //Se obtiene la geometra para actualizar el ndice
	            IGeometry g = null;
	            int inverse = getInversedIndex(calculatedIndex);
	            DefaultFeature df=(DefaultFeature)getRow(inverse).getLinkedRow();
	            g = df.getGeometry();
	            Rectangle2D r = g.getBounds2D();
	            this.index.remove(new Envelope(r.getX(), r.getX() + r.getWidth(),
	                    r.getY(), r.getY() + r.getHeight()),
	                new Integer(calculatedIndex));

	            //Se actualiza la relacin de ndices
	            relations.put(new Integer(calculatedIndex),
	                new Integer(previousExpansionFileIndex));

	            //Se actualizan los ndices
	            df=(DefaultFeature)getRow(inverse).getLinkedRow();
	            boolean cancel = fireBeforeModifyRow(df,calculatedIndex, sourceType);
	    		if (cancel)
	    			return;
	            g = df.getGeometry();
	            r = g.getBounds2D();
	            this.index.insert(new Envelope(r.getX(), r.getX() + r.getWidth(),
	                    r.getY(), r.getY() + r.getHeight()),
	                new Integer(calculatedIndex));
	            Value value=df.getAttribute(mapping.getColumnText());
	    		FLabel label= getLabel(inverse,true);
	    		label.setString(value.toString());
	    		if (mapping.getColumnRotate()==mapping.getColumnText()){
	    			label.setRotation(((NumericValue)value).doubleValue());
	    		}
	        }
		fireAfterModifyRow(calculatedIndex, sourceType);
	}

	public void undoRemoveRow(int index, int sourceType) throws IOException, DriverIOException {
		boolean cancel = fireBeforeRowAdded(sourceType);
		if (cancel)
			return;
		delRows.set(index, false);
	    Rectangle2D r = getLabel(index,true).getBoundBox();
	    this.index.insert(new Envelope(r.getX(), r.getX() + r.getWidth(),
	                r.getY(), r.getY() + r.getHeight()), new Integer(index));
	    fireAfterRowAdded(null,index, sourceType);
	}

	public AnnotationEditableAdapter(FLyrAnnotation lyrAnnotation) {
		super();
		this.labels=lyrAnnotation.getLabels();
		this.mapping=lyrAnnotation.getMapping();
		this.legend=lyrAnnotation.getLegend();
		this.isInPixels=lyrAnnotation.isInPixels();
		this.lyrAnnotation=lyrAnnotation;
	}
	public IRowEdited[] getFeatures(Rectangle2D r, String strEPSG) throws DriverException {
		// En esta clase suponemos random access.
		// Luego tendremos otra clase que sea VectorialEditableDBAdapter
		// que reescribir este mtodo.
        Envelope e = FConverter.convertRectangle2DtoEnvelope(r);
        List l = index.query(e);
        IRowEdited[] feats = new IRowEdited[l.size()];
        try {
	        for (int index = 0; index < l.size(); index++) {
	            Integer i = (Integer) l.get(index);
	            int inverse = getInversedIndex(i.intValue());
				feats[index] = (IRowEdited) getRow(inverse);
	        }
		} catch (DriverIOException e1) {
			throw new DriverException(e1);
		} catch (IOException e1) {
			throw new DriverException(e1);
		}

		return feats;
	}
	 /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.edition.IEditableSource#getRow(int)
     */
    public IRowEdited getRow(int index) throws DriverIOException, IOException {
    	int calculatedIndex=getCalculatedIndex(index);
    	Integer integer = new Integer(calculatedIndex);
        //Si no est en el fichero de expansin
    	DefaultRowEdited edRow=null;

    	if (!relations.containsKey(integer)) {
			try {
				IFeature f=ova.getFeature(calculatedIndex);
				String s=f.getID();
				FLabel label=getLabel(index,false);
				if (label==null)
					return null;
				IGeometry geom=getGeometry(label.getBoundBox());
				f=new DefaultFeature(geom,f.getAttributes(),s);
				//f.setID(s);
				edRow = new DefaultRowEdited(f,
				        DefaultRowEdited.STATUS_ORIGINAL, index);
			} catch (DriverException e) {
				throw new DriverIOException(e);
			}

            return edRow;
        } else {
            int num = ((Integer) relations.get(integer)).intValue();
            IRowEdited aux = expansionFile.getRow(num);
            IFeature f=(IFeature)aux.getLinkedRow().cloneRow();
            IGeometry geom=getGeometry(labelExpansion.getLabel(num).getBoundBox());
            String s=f.getID();
            f=new DefaultFeature(geom,f.getAttributes(),s);
            //f.setID(s);
            edRow = new DefaultRowEdited(f, aux.getStatus(), index);
            return edRow;
        }
    }
    private IGeometry getGeometry(Rectangle2D r){
    	GeneralPathX resul = new GeneralPathX();
		Point2D[] vs=new Point2D[4];
		vs[0]=new Point2D.Double(r.getX(),r.getY());
    	vs[1]=new Point2D.Double(r.getMaxX(),r.getY());
    	vs[2]=new Point2D.Double(r.getMaxX(),r.getMaxY());
    	vs[3]=new Point2D.Double(r.getX(),r.getMaxY());
    	//vs[4]=new Point2D.Double(r.getX(),r.getY());
		for (int i = 0; i < vs.length; i++) {
			if (i == 0) {
				resul.moveTo(vs[i].getX(),vs[i].getY());
			} else {
				resul.lineTo(vs[i].getX(),vs[i].getY());
			}
		}
		resul.closePath();
		return ShapeFactory.createPolygon2D(resul);
    }
    public void startEdition(int sourceType) throws EditionException {
    	isEditing = true;
		try {
			expansionFile.open();
				index = new Quadtree();

				for (int i = 0; i < labels.size(); i++) {
					Rectangle2D r=((FLabel)labels.get(i)).getBoundBox();
					Envelope e = new Envelope(r.getX(),
							r.getX() + r.getWidth(), r.getY(), r.getY()
									+ r.getHeight());
					index.insert(e, new Integer(i));
					if (fullExtent == null) {
						fullExtent = r;
					} else {
						fullExtent.add(r);
					}
				}
		} catch (IOException e) {
			throw new EditionException(e);
		}

		System.err.println("Se han metido en el ndice "
				+ index.queryAll().size() + " labels");
	}

    public Rectangle2D getShapeBounds(int index) throws IOException {
		//Solo se utiliza cuando el driver es BoundedShapes
//		 Si no est en el fichero de expansin
		Integer integer = new Integer((int) index);
		if (!relations.containsKey(integer)) {
			if (ova.getDriver() instanceof BoundedShapes){
				BoundedShapes bs = (BoundedShapes) ova.getDriver();
				return bs.getShapeBounds(index);
			}else{
				return ova.getDriver().getShape(index).getBounds2D();
			}

		} else {
			int num = ((Integer) relations.get(integer)).intValue();
			DefaultRowEdited feat;
			feat = (DefaultRowEdited) expansionFile.getRow(num);
			if (feat.getStatus() == IRowEdited.STATUS_DELETED)
				return null;
			IGeometry geom = ((IFeature)feat.getLinkedRow()).getGeometry();
			return geom.getBounds2D();
		}

	}
    public FLabel getLabel(int index,boolean calculated) {
    	FLabel label=null;
    	int calculatedIndex=index;
    	Integer integer=new Integer(index);
    	if (!calculated){
    		calculatedIndex=getCalculatedIndex(index);
    		integer = new Integer(calculatedIndex);
    	}
    	//Si no est en el fichero de expansin
    	if (!relations.containsKey(integer)) {
    		if (calculatedIndex > labels.size()){
    			return null;
    		}
    		label=(FLabel)labels.get(calculatedIndex);
		    return label;
        } else {
            int num = ((Integer) relations.get(integer)).intValue();
            label=labelExpansion.getLabel(num);
            return (FLabel)label;
        }
    }




//	public IGeometry getShape(int rowIndex) throws DriverIOException {
//		IGeometry geom=null;
//		//		 Si no est en el fichero de expansin
//		int calculatedIndex = getCalculatedIndex(rowIndex);
//		Integer integer = new Integer(calculatedIndex);
//		if (!relations.containsKey(integer)) {
//			geom=ova.getShape(calculatedIndex);
//		} else {
//			int num = ((Integer) relations.get(integer)).intValue();
//			DefaultRowEdited feat;
//			try {
//				feat = (DefaultRowEdited) expansionFile.getRow(num);
//				geom= ((IFeature) feat.getLinkedRow()).getGeometry()
//						.cloneGeometry();
//			} catch (IOException e) {
//				e.printStackTrace();
//				throw new DriverIOException(e);
//			}
//		}
//		if (geom.getGeometryType()!=FShape.POINT) {
//			Point2D p=FLabel.createLabelPoint((FShape)geom.getInternalShape());
//			geom=ShapeFactory.createPoint2D(p.getX(),p.getY());
//		}
//		return geom;
//
//	}
}
