package com.iver.cit.gvsig.fmap.edition.writers;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.drivers.ITableDefinition;
import com.iver.cit.gvsig.fmap.drivers.XTypes;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;

public class JdbcWriter extends AbstractWriter {
	Connection conn;
	ResultSet rs;
	Value[] record;
	int numRecord;
	
	private boolean bWriteAll;
	private boolean bCreateTable;
	
	private ResultSetMetaData metaData = null;

	public JdbcWriter(){
	}
	public void initialize(Connection conn, ResultSet rs) throws SQLException{
		this.conn = conn;
		this.rs = rs;
		metaData = rs.getMetaData();
		System.out.println("INICIO CONEXIN DE ESCRITURA");
	}
	public void preProcess() throws EditionException {
			numRecord = 0;
			try {
				conn.setAutoCommit(false);
			} catch (SQLException e) {
				e.printStackTrace();
				throw new EditionException(e);
			}
			/* Statement st = conn.createStatement();

			if (bCreateTable) {
				try {
					st.execute("DROP TABLE " + lyrDef.getTableName() + ";");
				} catch (SQLException e1) {
					// Si no existe la tabla, no hay que borrarla.
				}

				String sqlCreate = PostGIS.getSqlCreateSpatialTable(lyrDef,
						lyrDef.getFieldsDesc(), true);
				System.out.println("sqlCreate =" + sqlCreate);
				st.execute(sqlCreate);
			} */
			
	}

	public void process(IRowEdited editedRow) throws EditionException {
		IRow row = (IRow) editedRow.getLinkedRow();

		try {
			System.out.println("Intento escribir el registro " +
					numRecord + " de la capa " + metaData.getTableName(1));
			switch (editedRow.getStatus())
			{
    		case IRowEdited.STATUS_ADDED:
    			record=row.getAttributes();
    			rs.moveToInsertRow();
    			for (int i=0; i < record.length; i++)
    				XTypes.updateValue(rs, i, record[i]); 
    			rs.insertRow();
    			break;
    		case IRowEdited.STATUS_MODIFIED:
    			record=row.getAttributes();
    			rs.absolute(editedRow.getIndex()+1);
    			for (int i=0; i < record.length; i++)
    				XTypes.updateValue(rs, i, record[i]); 
    			rs.updateRow();
    			break;
    		case IRowEdited.STATUS_ORIGINAL:
    			if (bWriteAll)
    			{
        			record=row.getAttributes();
        			rs.moveToInsertRow();
        			for (int i=0; i < record.length; i++)
        				XTypes.updateValue(rs, i, record[i]); 
        			rs.insertRow();    				
    			}
    			break;
    		case IRowEdited.STATUS_DELETED:
    			rs.absolute(editedRow.getIndex()+1);
        		rs.deleteRow();
    			break;
			}
			numRecord++;
			rs.next();

		} catch (SQLException e) {
			System.out.println(e.getSQLState() + " " + e.getMessage());
			try {
				conn.rollback();
			} catch (SQLException e1) {
				e1.printStackTrace();
				throw new EditionException(e);				
			}
			
			throw new EditionException(e.getMessage(),e);
		} 

	}

	public void postProcess() throws EditionException {
		try {			
			conn.commit();
		} catch (SQLException e) {
			try {
				conn.rollback();
			} catch (SQLException e1) {
				e1.printStackTrace();
				throw new EditionException(e);				
			}
			e.printStackTrace();
			throw new EditionException(e);
		}
	}

	public boolean canWriteAttribute(int sqlType) {
		return true;
	}

	public String getName() {
		return "JDBC Writer";
	}
	/**
	 * @return Returns the bCreateTable.
	 */
	public boolean isCreateTable() {
		return bCreateTable;
	}

	/**
	 * @param createTable
	 *            The bCreateTable to set.
	 */
	public void setCreateTable(boolean createTable) {
		bCreateTable = createTable;
	}

	/**
	 * @return Returns the bWriteAll.
	 */
	public boolean isWriteAll() {
		return bWriteAll;
	}

	/**
	 * @param writeAll
	 *            The bWriteAll to set.
	 */
	public void setWriteAll(boolean writeAll) {
		bWriteAll = writeAll;
	}

	public void close() throws SQLException
	{
		rs.close();
		// conn.close();
		System.out.println("CIERRO CONEXIN DE ESCRITURA");
	}
	public boolean canAlterTable() {
		// TODO Auto-generated method stub
		return false;
	}
	public boolean canSaveEdits() {
		try {
			return (rs.getConcurrency() == ResultSet.CONCUR_UPDATABLE);
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		} 
	}
}
