/*
 * Created on 19-sep-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import org.cresques.cts.ICoordTrans;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.core.v02.FGraphicUtilities;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.rendering.FGraphic;
import com.iver.utiles.swing.threads.Cancellable;
import com.vividsolutions.jts.index.ItemVisitor;
import com.vividsolutions.jts.index.SpatialIndex;
import com.vividsolutions.jts.index.quadtree.Quadtree;

public class GraphicLayer extends FLyrDefault {
    private ArrayList graphics = new ArrayList();
    private ArrayList symbols = new ArrayList();
    private Rectangle2D fullExtent;
    private SpatialIndex spatialIndex = new Quadtree();
    private FBitSet selection = new FBitSet();

    public GraphicLayer() {
        super();
        System.err.println("GraphicLayer: Constructor");
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#getFullExtent()
     */
    public Rectangle2D getFullExtent() throws DriverException {
        return fullExtent;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#draw(java.awt.image.BufferedImage, java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable, double)
     */
    public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale) throws DriverException {
        if (isVisible() && isWithinScale(scale)){
            drawGraphics(image, g, viewPort, cancel);
            }
    }

    /**
     * Generic visitor pattern. It may be used for queryByRect, byPoint,
     * by polygon, etc.
     * @param visitor
     * @param cancel
     */
    public void process(ItemVisitor visitor, Cancellable cancel)
    {
        int numReg;

        for (numReg = 0; numReg < graphics.size(); numReg++) {
            if (cancel.isCanceled()) {
                break;
            }

            FGraphic theGraphic = (FGraphic) graphics.get(numReg);
            visitor.visitItem(theGraphic);
        }

    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.layers.FLayer#print(java.awt.Graphics2D, com.iver.cit.gvsig.fmap.ViewPort, com.iver.cit.gvsig.fmap.operations.Cancellable, double)
     */
    public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel, double scale) throws DriverException {
        if (isVisible() && isWithinScale(scale)){
            drawGraphics(null, g, viewPort, cancel);
            }
    }

    private void drawGraphics(BufferedImage image, Graphics2D g,
            ViewPort viewPort, Cancellable cancel)
    {
        int numReg;
        Rectangle2D elExtent = viewPort.getAdjustedExtent();
        if (elExtent == null) return;

        //int anchoMapa;
        //int altoMapa;
        //double anchoReal;
        //double altoReal;
        //double escala;
        ISymbol theSymbol = null;
        // System.out.println("Dibujando grficos...(" + graphics.size() + ")");

        ICoordTrans ct = getCoordTrans();

        for (numReg = 0; numReg < graphics.size(); numReg++) {
            if (cancel.isCanceled()) {
                break;
            }

            FGraphic theGraphic = (FGraphic) graphics.get(numReg);
            IGeometry geom = theGraphic.getGeom();

            // Modificacin para Jorge, para que le reproyecte los grficos.
			if (ct != null) {
		        geom = geom.cloneGeometry();
				geom.reProject(ct);
			}

            if (geom.fastIntersects(elExtent.getMinX(),
                        elExtent.getMinY(), elExtent.getWidth(), elExtent.getHeight()))
             {
                theSymbol = (ISymbol) symbols.get(theGraphic.getIdSymbol());
                if (selection.get(numReg)) // Si est seleccinado
                {
                	FGraphicUtilities.DrawHandlers(g, viewPort.getAffineTransform(), geom.getHandlers(IGeometry.SELECTHANDLER));
                }
                else
                {
                	theGraphic.draw(g, viewPort, theSymbol);
                }
            }
        }
    }

    public int addSymbol(ISymbol newSymbol)
    {
        if (symbols.add(newSymbol))
            return symbols.size()-1;
        return -1;

    }
    /**
     * Add a graphic to the graphic layer.
     * @param g
     */
    public void addGraphic(FGraphic g)
    {
        if (graphics.add(g))
        {

//        	spatialIndex.insert(g.getGeom().getBounds2D())
            if (fullExtent == null) {
                fullExtent = g.getGeom().getBounds2D();
            } else {
                fullExtent.add(g.getGeom().getBounds2D());
            }

//            return graphics.size()-1;
        }
//        return -1;

    }
    
    public void insertGraphic(int position, FGraphic g) {
    	graphics.add(position, g);
        if (fullExtent == null) {
            fullExtent = g.getGeom().getBounds2D();
        } else {
            fullExtent.add(g.getGeom().getBounds2D());
        }
    }

    /**
     *
     */
    public void clearAllGraphics()
    {
        System.err.println("Clear all graphics. Antes haba " + graphics.size());
        graphics.clear();
        fullExtent = null;
    }
    /**
     *
     */
    public void clearSymbolsGraphics()
    {
        symbols.clear();
    }

	public FBitSet getSelection() {
		return selection;
	}

	public void setSelection(FBitSet selection) {
		this.selection = selection;
	}

	public FBitSet queryByRect(Rectangle2D r) {
		FBitSet newSel = new FBitSet();
        for (int numReg = 0; numReg < graphics.size(); numReg++) {

            FGraphic theGraphic = (FGraphic) graphics.get(numReg);
            IGeometry geom = theGraphic.getGeom();
            if (geom.intersects(r))
            {
            	newSel.set(numReg);
            }
        }
        return newSel;
	}

	public void removeGraphics(FGraphic graphic) {
		graphics.remove(graphic);
		reCalculateFullExtent();
		
	}

	private void reCalculateFullExtent(){
		if (graphics.size() > 0) {
			FGraphic g = (FGraphic) graphics.get(0);
			fullExtent = g.getGeom().getBounds2D();
			for (int i = 1; i < graphics.size(); i++) {
				g = (FGraphic) graphics.get(i);
				Rectangle2D rAux = g.getGeom().getBounds2D();

				fullExtent.add(rAux);
			}
		}
	}

}
