/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import java.util.ArrayList;

import org.cresques.filter.RasterFilterStack;

import com.iver.cit.gvsig.fmap.layers.layerOperations.InfoByPoint;


/**
 * Interfaz de operaciones sobre una capa raster.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public interface RasterOperations extends InfoByPoint {
	/**
	 * Asigna RGB en la posicin en la que se ha 
	 * hecho click al mostrar informacin del raster.
	 * @param r	valor de rojo
	 * @param g	valor de verde
	 * @param b	valor de azul
	 */
	public void setRGB(int r, int g, int b);
	
	/**
	 * Obtiene la pila de filtros aplicada al raster
	 * @return
	 */
	public RasterFilterStack getFilterStack();

	/**
	 * Asigna la pila de filtros aplicada al raster
	 * @return
	 */
	public void setFilterStack(RasterFilterStack stack);
	
	/**
	 * Devuelve el RasterAdapter de la capa.
	 *
	 * @return RasterAdapter.
	 */
	public RasterAdapter getSource() ;
		
	/**
	 * Inserta el RasterAdapter.
	 *
	 * @param ra RasterAdapter.
	 */
	public void setSource(RasterAdapter ra) ;
	
	/**
	 * Asigna la posicin en la que se ha hecho click al mostrar
	 * informacin del raster.
	 * @param x	Posicin en X
	 * @param y	Posicin en Y
	 */
	public void setPos(int x, int y);
	
	/**
	 * Asigna la posicin en coordenadas del mundo real en la que se ha 
	 * hecho click al mostrar informacin del raster.
	 * @param x	Posicin en X
	 * @param y	Posicin en Y
	 */
	public void setPosWC(double x, double y);
	
	/**
	 * Obtiene el valor del pixel del Image en la posicin x,y
	 * @param x Posicin x
	 * @param y Posicin y
	 * @return valor de pixel
	 */
	public int[] getPixel(double wcx, double wcy);
	
	public void setTransparency(int trans);
	
	public void setBand(int flag, int nBand);
		
	/**
	 * Obtiene atributos a partir de un georasterfile
	 * @return
	 */
	public ArrayList getAttributes();
		
	public double getMaxX();
	public double getMaxY();
	public double getMinX();
	public double getMinY();
	public double getWidth();
	public double getHeight();
	
	/**
	 * Consulta si una capa est siendo tileada o no
	 * @return true si est siendo tileada y false si no lo est
	 */
	public boolean isTiled();
	
	/**
	 * Obtiene el tamao del tile si la capa est siendo tileada. Esta funcin e fundamentalmente
	 * usada por salvar a raster donde se consulta el tamao del tile para hacer bandas de ese ancho.
	 * @return array con dos enteros que representan el ancho y el alto del tile
	 */
	public int[] getTileSize();
}
