/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.TreeMap;

import org.geotools.filter.ExpressionBuilder;
import org.geotools.filter.Filter;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.NamedLayer;
import org.geotools.styling.Rule;
import org.geotools.styling.SLDTransformer;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.StyleFactory;
import org.geotools.styling.StyledLayerDescriptor;
import org.geotools.styling.Symbolizer;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.instruction.IncompatibleTypesException;
import com.hardcode.gdbms.engine.instruction.SemanticException;
import com.hardcode.gdbms.engine.values.BooleanValue;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.StringValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.ISLDCompatible;
import com.iver.cit.gvsig.fmap.core.ISymbol;
import com.iver.cit.gvsig.fmap.core.SLDTags;
import com.iver.cit.gvsig.fmap.core.SLDUtils;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.utiles.XMLEntity;

/**
 * Leyenda vectorial por valores nicos.
 *
 * @author Vicente Caballero Navarro
 */
public class VectorialUniqueValueLegend implements UniqueValueLegend,
    VectorialLegend {
    private TreeMap symbols = new TreeMap(new Comparator() {
                public int compare(Object o1, Object o2) {
                    if ((o1 != null) && (o2 != null)) {
                        Value v2 = (Value) o2;
                        Value v1 = (Value) o1;
                        BooleanValue boolVal;

                        //						TODO estas dos comprobaciones son por evitar un bug en el gdbms, cuando se solucione se puede eliminar.
                        if (v1 instanceof NullValue && v2 instanceof NullValue) {
                            return 0;
                        }

                        if (v1 instanceof NullValue) {
                            return -1;
                        }

                        if (v2 instanceof NullValue) {
                            return 1;
                        }

                        try {
                            boolVal = (BooleanValue) (v1.greater(v2));

                            if (boolVal.getValue()) {
                                return 1;
                            }

                            boolVal = (BooleanValue) (v1.less(v2));

                            if (boolVal.getValue()) {
                                return -1;
                            }
                        } catch (IncompatibleTypesException e) {
                            // TODO Auto-generated catch block
                            //e.printStackTrace();
                        }

                        try {
                            if (((BooleanValue) v1.equals(v2)).getValue()) {
                                return 0;
                            }
                        } catch (IncompatibleTypesException e) {
                            // TODO Auto-generated catch block
                            //e.printStackTrace();
                        }

                        if (v1 instanceof StringValue) {
                            return -1;
                        }

                        if (v2 instanceof StringValue) {
                            return 1;
                        }
                    }

                    return 0;
                }
            }); // Para poder ordenar
    private ArrayList keys = new ArrayList(); // En lugar de un HashSet, para tener acceso por ndice
    private String fieldName;
    private int fieldId = -1;
    private String labelFieldName;
    private String labelFieldHeight;
    private String labelFieldRotation;
    private ISymbol defaultSymbol;
    private DataSource dataSource;
    private int shapeType;
    private String valueType = NullValue.class.getName();
    private boolean useDefaultSymbol = false;
    // private boolean bWithHeightText = false;

    /**
     * Crea un nuevo VectorialUniqueValueLegend.
     */
    public VectorialUniqueValueLegend() {
        //	defaultSymbol = LegendFactory.DEFAULT_POLYGON_SYMBOL;
    }

    /**
     * Crea un nuevo VectorialUniqueValueLegend.
     *
     * @param shapeType Tipo de shape.
     */
    public VectorialUniqueValueLegend(int shapeType) {
        setShapeType(shapeType);
    }

    /**
     * Inserta el tipo de shape.
     *
     * @param shapeType Tipo de shape.
     */
    public void setShapeType(int shapeType) {
        if (this.shapeType != shapeType) {
            switch (shapeType) {
                case FShape.POINT:
                    defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_POINT);

                    break;

                case FShape.LINE:
                    defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_LINE);

                    break;

                case FShape.POLYGON:
                    defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_FILL);

                    break;

                default:
                    defaultSymbol = new FSymbol(shapeType);
            }

            this.shapeType = shapeType;
        }
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.UniqueValueLegend#setValueSymbolByID(int,
     *      ISymbol)
     */
    public void setValueSymbolByID(int id, ISymbol symbol) {
        symbols.put(keys.get(id), symbol);
    }

    /**
     * Devuelve un smbolo a partir del ID. Mira en el m_ArrayKeys  el elemento
     * ID, y con esa clave recupera el FSymbol por valor
     *
     * @param id ID.
     * @param symbol DOCUMENT ME!
     */

    /*
       public FSymbol getSymbolByID(int ID) {
           return (FSymbol) symbols.get(keys.get(ID));
       }
     */

    /**
     * Se usa en la tabla que muestra una leyenda.
     *	@deprecated use setValueSymbolByID(int id, ISymbol symbol);
     * @param id
     * @param symbol
     */
    public void setValueSymbol(int id, ISymbol symbol) {
        symbols.put(keys.get(id), symbol);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.UniqueValueLegend#getValues()
     */
    public Object[] getValues() {
        return symbols.keySet().toArray(new Object[0]);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.UniqueValueLegend#addSymbol(java.lang.Object,
     *      ISymbol)
     */
    public void addSymbol(Object key, ISymbol symbol) {
        Object resul;
        resul = symbols.put(key, symbol);

        if (resul != null) {
            System.err.println("Error: la clave " + key +
                " ya exista. Resul = " + resul);
            System.err.println("symbol nuevo:" + symbol.getDescription() +
                " Sviejo= " + ((ISymbol) resul).getDescription());
        } else {
            keys.add(key);

            if (!key.getClass().equals(NullValue.class)) {
                valueType = key.getClass().getName();
            }
        }
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#clear()
     */
    public void clear() {
        keys.clear();
        symbols.clear();
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#getDescriptions()
     */
    public String[] getDescriptions() {
        String[] descriptions = new String[symbols.size()];
        ISymbol[] auxSym = getSymbols();

        for (int i = 0; i < descriptions.length; i++)
            descriptions[i] = auxSym[i].getDescription();

        return descriptions;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#getSymbols()
     */
    public ISymbol[] getSymbols() {
        return (ISymbol[]) symbols.values().toArray(new ISymbol[0]);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#getFieldName()
     */
    public String getFieldName() {
        return fieldName;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setDefaultSymbol(ISymbol)
     */
    public void setDefaultSymbol(ISymbol s) {
        defaultSymbol = s;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelField()
     */
    public String getLabelField() {
        return labelFieldName;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#setLabelField(int)
     */
    public void setLabelField(String fieldName) {
        labelFieldName = fieldName;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#setField()
     */
    public void setFieldName(String str) {
        fieldName = str;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getSymbol(int)
     */
    public ISymbol getSymbol(int recordIndex) throws DriverException {
        try {
            Value val = dataSource.getFieldValue(recordIndex, fieldId);
            ISymbol theSymbol = getSymbolByValue(val);

            //if (theSymbol != null) {
            return theSymbol;

            //} else {
            //	return getDefaultSymbol();
            //}
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Devuelve un smbolo a partir de una IFeature.
     * OJO!! Cuando usamos un feature iterator de base de datos
     * el nico campo que vendr rellenado es el de fieldID.
     * Los dems vendrn a nulos para ahorra tiempo de creacin.
     * @param feat IFeature
     *
     * @return Smbolo.
     */
    public ISymbol getSymbolByFeature(IFeature feat) {
        Value val = feat.getAttribute(fieldId);
        // Value val = feat.getAttribute(0);
        ISymbol theSymbol = getSymbolByValue(val);

        //if (theSymbol != null) {
        return theSymbol;

        //} else {
        //    return getDefaultSymbol();
        //}
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getDefaultSymbol()
     */
    public ISymbol getDefaultSymbol() {
        return defaultSymbol;
    }

	/**
	 * @deprecated
	 * Writes and SLD using GEOTOOLS objetcs.
	 * @see com.iver.cit.gvsig.fmap.rendering.Legend#getSLDString()
	 */
	public String getSLDString_()
	{
	    try{
			StyledLayerDescriptor sld = new StyledLayerDescriptor();
			StyleFactory styleFactory = StyleFactory.createStyleFactory();
			StyleBuilder sb = new StyleBuilder();
	        Style style = sb.createStyle();
	        style.setName("default");
	        Filter filter = null;
	        Rule rule = null;

			FeatureTypeStyle featStyle = styleFactory.createFeatureTypeStyle();
			featStyle.setFeatureTypeName(fieldName);

			ISymbol[] symbols = this.getSymbols();
			Symbolizer[] theSymbolizers = new Symbolizer[symbols.length];
			Object[] values = this.getValues();
			String valueStr = null;
			String condition = null;

			for(int i = 0; i < symbols.length; i++ )
			{
				valueStr = values[i].toString();
				//if(this.valueType == "")
					condition = fieldName +"='"+valueStr+"'";
				//else
				//	condition = fieldName +"="+values[i];
				filter = (Filter)ExpressionBuilder.parse(condition);
				theSymbolizers[0] = SLDUtils.toGeotoolsSymbol(symbols[i]);
				rule = styleFactory.createRule();
				rule.setName(valueStr);
				rule.setTitle(valueStr);
				rule.setFilter(filter);
				rule.setSymbolizers((Symbolizer[])theSymbolizers.clone());
				featStyle.addRule(rule);
			}
			style.addFeatureTypeStyle(featStyle);
			SLDTransformer st = new SLDTransformer();
			NamedLayer namedLayer = new NamedLayer();
			namedLayer.setName("comunidades");
			namedLayer.addStyle(style);
			sld.addStyledLayer(namedLayer);
			return st.transform(style);

	    }catch(Exception e)
	    {
	    	e.printStackTrace();
	    	return null;
	    }
	}
	/**
	 * creates the SLD String that defines this legend type.
	 */
	public String getSLDString(String name)
	{
	    try{

			XmlBuilder xmlBuilder = new XmlBuilder();
			xmlBuilder.writeHeader();
			xmlBuilder.openTag(SLDTags.SLD_ROOT, SLDTags.VERSION_ATTR, SLDTags.VERSION_1_0_0);
			xmlBuilder.openTag(SLDTags.NAMEDLAYER);
			xmlBuilder.writeTag(SLDTags.NAME,name);
			xmlBuilder.openTag(SLDTags.USERSTYLE);
			xmlBuilder.openTag(SLDTags.FEATURETYPESTYLE);
			//xmlBuilder.writeTag(SLDTags.FEATURETYPENAME,"fieldName");

			ISymbol[] symbols = this.getSymbols();
			Object[] values = this.getValues();
			String valueStr = null;

			for(int i = 0; i < symbols.length; i++ )
			{
				valueStr = values[i].toString();
				xmlBuilder.openTag(SLDTags.RULE);
				xmlBuilder.writeTag(SLDTags.NAME, valueStr);
				xmlBuilder.openTag(SLDTags.FILTER);
				xmlBuilder.openTag(SLDTags.PROPERTYISEQUALTO);
				xmlBuilder.writeTag(SLDTags.PROPERTYNAME,fieldName);
				xmlBuilder.writeTag(SLDTags.LITERAL, valueStr);
				xmlBuilder.closeTag();
				xmlBuilder.closeTag();
				if (symbols[i] instanceof ISLDCompatible)
				{
					ISLDCompatible symSLD = (ISLDCompatible) symbols[i];
					xmlBuilder.writeRaw(symSLD.toSLD());
				}
				else
					throw new RuntimeException("Cannot convert Symbol " + i + " " + symbols[i].getDescription() + " to SLD");

				xmlBuilder.closeTag();
			}

			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			return xmlBuilder.getXML();

	    }catch(Exception e)
	    {
	    	e.printStackTrace();
	    	return null;
	    }
	}
    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getXMLEntity()
     */
    public XMLEntity getXMLEntity() {
        XMLEntity xml = new XMLEntity();
        xml.putProperty("className", this.getClass().getName());
        xml.putProperty("fieldName", fieldName);
        xml.putProperty("labelfield", labelFieldName);
        xml.putProperty("labelFieldHeight", labelFieldHeight);
        xml.putProperty("labelFieldRotation", labelFieldRotation);

        xml.putProperty("useDefaultSymbol", useDefaultSymbol);
        xml.addChild(getDefaultSymbol().getXMLEntity());
        // xml.putProperty("isBWithHeightText", isBWithHeightText());
        xml.putProperty("numKeys", keys.size());

        if (keys.size() > 0) {
            xml.putProperty("tipoValueKeys", valueType);

            String[] sk = new String[keys.size()];
            String[] sv = new String[keys.size()];
            int[] stk = new int[keys.size()];
            int[] stv = new int[keys.size()];
            ISymbol[] fsymbols = getSymbols();
            Object[] values = getValues();

            for (int i = 0; i < keys.size(); i++) {
            	if (((Value) keys.get(i)).toString().equals("")) {
            		sk[i] =" ";
            	}else {
            		sk[i] = ((Value) keys.get(i)).toString();
            	}
            	if (((Value) values[i]).toString().equals("")) {
            		sv[i] =" ";
            	}else {
            		sv[i] = ((Value) values[i]).toString();
            	}
            	stk[i]= ((Value)keys.get(i)).getSQLType();
            	stv[i]= ((Value)values[i]).getSQLType();
                xml.addChild(fsymbols[i].getXMLEntity());

                ///System.out.println("get-----------"+sk[i]+"--"+fsymbols[i].getDescription()+"---"+fsymbols[i].getColor());
            }

            xml.putProperty("keys", sk);
            xml.putProperty("values", sv);
            xml.putProperty("typeKeys",stk);
            xml.putProperty("typeValues",stv);
        }

        return xml;
    }

    /**
     * Inserta el XMLEntity.
     *
     * @param xml XMLEntity.
     */
    public void setXMLEntity03(XMLEntity xml) {
        clear();
        setFieldName(xml.getStringProperty("fieldName"));
        setLabelField(xml.getStringProperty("labelfield"));

        int useDefaultSymbol = xml.getIntProperty("useDefaultSymbol");

        if (useDefaultSymbol == 1) {
            setDefaultSymbol(FSymbol.createFromXML03(xml.getChild(0)));
        } else {
            setDefaultSymbol(null);
        }

        int numKeys = xml.getIntProperty("numKeys");

        if (numKeys > 0) {
            String className = xml.getStringProperty("tipoValueKeys");
            String[] sk = xml.getStringArrayProperty("keys");
            String[] sv = xml.getStringArrayProperty("values");
            Value auxValue;
            Value auxValue2;

            for (int i = 0; i < numKeys; i++) {
                try {
                    auxValue = ValueFactory.createValue(sk[i], className);
                    auxValue2 = ValueFactory.createValue(sv[i], className);

                    ISymbol sym = FSymbol.createFromXML03(xml.getChild(i +
                                useDefaultSymbol));

                    ///addSymbol(auxValue, sym);
                    symbols.put(auxValue2, sym);
                    keys.add(auxValue);

                    ///System.out.println("---set------"+auxValue.toString());
                    /// System.out.println("set-----------"+sk[i]+"--"+sym.getDescription()+"---"+sym.getColor());
                } catch (SemanticException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * Inserta el XMLEntity.
     *
     * @param xml XMLEntity.
     */
    public void setXMLEntity(XMLEntity xml) {
        clear();
        setFieldName(xml.getStringProperty("fieldName"));
        setLabelField(xml.getStringProperty("labelfield"));

        if (xml.contains("labelFieldHeight")) {
            setLabelHeightField(xml.getStringProperty("labelFieldHeight"));
        }

        if (xml.contains("labelFieldRotation")) {
            setLabelRotationField(xml.getStringProperty("labelFieldRotation"));
        }

        useDefaultSymbol = xml.getBooleanProperty("useDefaultSymbol");
        setDefaultSymbol(FSymbol.createFromXML(xml.getChild(0)));

        // FJP: Esto no es necesario ya. Para comprobar si tenemos un campo de altura, miramos
        // si getLabelHeightField devuelve null o no.
        /* if (xml.contains("isBWithHeightText")) {
            setBWithHeightText(xml.getBooleanProperty("isBWithHeightText"));
        } */

        //addSymbol(new NullUniqueValue(),getDefaultSymbol());
        int numKeys = xml.getIntProperty("numKeys");

        if (numKeys > 0) {
            String className = xml.getStringProperty("tipoValueKeys");
            String[] sk = xml.getStringArrayProperty("keys");
            String[] sv = xml.getStringArrayProperty("values");
            Value auxValue = null;
            Value auxValue2 = null;
            int[] stk=null;
            if (xml.contains("typeKeys")) {
				stk = xml.getIntArrayProperty("typeKeys");
				int[] stv = xml.getIntArrayProperty("typeValues");
				for (int i = 0; i < numKeys; i++) {
					boolean isDefault = false;
					if (getValue(sk[i], stk[i]) == null) {
						isDefault = true;
					}

					if (className
							.equals("com.hardcode.gdbms.engine.values.NullUniqueValue")
							|| isDefault) {
						auxValue = new NullUniqueValue();
						auxValue2 = ValueFactory.createNullValue();
					} else {
						auxValue = getValue(sk[i], stk[i]); // ValueFactory.createValue(sk[i],
															// className);
						auxValue2 = getValue(sv[i], stv[i]); // ValueFactory.createValue(sv[i],
																// className);
					}
					ISymbol sym = FSymbol.createFromXML(xml.getChild(i + 1));
					symbols.put(auxValue2, sym);
					keys.add(auxValue);
				}
			} else {
				for (int i = 0; i < numKeys; i++) {
					boolean isDefault = false;
					if (getValue(sk[i]) == null) {
						isDefault = true;
					}
					if (className
							.equals("com.hardcode.gdbms.engine.values.NullUniqueValue")
							|| isDefault) {
						auxValue = new NullUniqueValue();
						auxValue2 = ValueFactory.createNullValue();
					} else {
						auxValue = getValue(sk[i]); // ValueFactory.createValue(sk[i],
													// className);
						auxValue2 = getValue(sv[i]); // ValueFactory.createValue(sv[i],
														// className);
					}
					ISymbol sym = FSymbol.createFromXML(xml.getChild(i + 1));
					symbols.put(auxValue2, sym);
					keys.add(auxValue);
				}
			}
        }
    }

    /**
	 * Devuelve el valor a partir de su valor en un string.
	 *
	 * @param s
	 *            String con el valor.
	 * @deprecated Mtodo utilizado hasta la 1.0 alpha 855 Debes utilizar a partir de ahora getValue(String s,int type);
	 * @return Value.
	 */
    private Value getValue(String s) {
        Value val = new NullUniqueValue();
        if (s.equals("Resto de Valores"))return val;
        try {
            try {
                val = ValueFactory.createValueByType(s, Types.INTEGER);

                return val;
            } catch (NumberFormatException e) {
            }

            try {
                val = ValueFactory.createValueByType(s, Types.BIGINT);

                return val;
            } catch (NumberFormatException e) {
            }

            try {
                val = ValueFactory.createValueByType(s, Types.FLOAT);

                return val;
            } catch (NumberFormatException e) {
            }

            try {
                val = ValueFactory.createValueByType(s, Types.DOUBLE);

                return val;
            } catch (NumberFormatException e) {
            }

            val = ValueFactory.createValueByType(s, Types.LONGVARCHAR);

        } catch (ParseException e) {
            e.printStackTrace();
        }

        return val;
    }
    /**
     * Devuelve el valor a partir de su valor en un string.
     *
     * @param s String con el valor.
     *
     * @return Value.
     */
    private Value getValue(String s,int type) {
        Value val = new NullUniqueValue();
        if (type==Types.OTHER)
        	return val;
        try {
        	val = ValueFactory.createValueByType(s, type);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return val;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#cloneLegend()
     */
    public Legend cloneLegend() throws XMLException {
        return LegendFactory.createFromXML(getXMLEntity());
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setDataSource(com.hardcode.gdbms.engine.data.DataSource)
     */
    public void setDataSource(DataSource ds)
        throws FieldNotFoundException, DriverException {
        try {
            dataSource = ds;
            ds.start();
            fieldId = ds.getFieldIndexByName(fieldName);
            ds.stop();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        }
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.UniqueValueLegend#getSymbolByValue(com.hardcode.gdbms.engine.values.Value)
     */
    public ISymbol getSymbolByValue(Value key) {
        if (symbols.containsKey(key)) {
            return (ISymbol) symbols.get(key);
        } else if (useDefaultSymbol) {
            return getDefaultSymbol();
        }

        return null;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getShapeType()
     */
    public int getShapeType() {
        return shapeType;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelHeightField()
     */
    public String getLabelHeightField() {
        return labelFieldHeight;
    }

    /**
     * Inserta el alto de campo.
     *
     * @param str alto.
     */
    public void setLabelHeightField(String str) {
        labelFieldHeight = str;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelRotationField()
     */
    public String getLabelRotationField() {
        return labelFieldRotation;
    }

    /**
     * Inserta rotacin.
     *
     * @param str Rotacin.
     */
    public void setLabelRotationField(String str) {
        labelFieldRotation = str;
    }

    /**
     * Introduce si se tiene que representar el resto de valores o no.
     *
     * @param b True si se utiliza el resto de valores.
     */
    public void useDefaultSymbol(boolean b) {
        useDefaultSymbol = b;
    }
    /**
     * Devuelve si se utiliza o no el resto de valores para representarse.
     *
     * @return True si se utiliza el resto de valores.
     */
    public boolean isUseDefaultSymbol() {
        return useDefaultSymbol;
    }
    /**
     * Devuelve true si el etiquetado de la capa se ha modificado.
     *
     * @return True si se ha modificado el etiquetado de la capa.
     */
    /* public boolean isBWithHeightText() {
        return bWithHeightText;
    } */

    /**
     * Introduce si se ha modificado el etiquetado de la capa.
     *
     * @param withHeightText True si se ha modificado el etiquetado de la capa.
     */
    /* public void setBWithHeightText(boolean withHeightText) {
        bWithHeightText = withHeightText;
    } */

    /**
     * Elimina el smbolo que tiene como clave el valor que se pasa como parmetro.
     *
     * @param key clave.
     */
    public void delSymbol(Object key) {
        keys.remove(key);
        symbols.remove(key);
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getUsedFields()
     */
    public String[] getUsedFields() {
        ArrayList usedFields = new ArrayList();
        if (getFieldName() != null)
            usedFields.add(getFieldName());
        if (getLabelField() != null)
            usedFields.add(getLabelField());
        if (getLabelHeightField() != null)
            usedFields.add(getLabelHeightField());
        if (getLabelRotationField() != null)
            usedFields.add(getLabelRotationField());

        return (String[]) usedFields.toArray(new String[0]);

    }
}
