/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools.Behavior;

import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.Events.MeasureEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PolylineListener;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;


/**
 * Behaviour que espera un listener de tipo MeasureListener.
 *
 * @author Vicente Caballero Navarro
 */
public class PolygonBehavior extends PolylineBehavior {
	/**
	 * Crea un nuevo PolygonBehavior.
	 *
	 * @param ali listener.
	 */
	public PolygonBehavior(PolylineListener ali) {
		super(ali);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		g.drawImage(getMapControl().getImage(), 0, 0, null);
		g.setColor(Color.black);

		if (!arrayX.isEmpty()) {
			drawPolyLine((Graphics2D) g);
		}
	}

	/**
	 * Reimplementacin del mtodo mousePressed de Behavior.
	 *
	 * @param E MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mousePressed(MouseEvent E) throws BehaviorException {
		if (E.getClickCount() == 2) {
			listener.polylineFinished(new MeasureEvent(
					(Double[]) arrayX.toArray(new Double[0]),
					(Double[]) arrayY.toArray(new Double[0]), E));
			arrayX.clear();
			arrayY.clear();
			isClicked = false;
		} else {
			isClicked = true;

			if (arrayX.size() == 0) {
				addPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));
				addPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));
				addPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));
			} else {
				addAntPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));
			}

			listener.pointFixed(new MeasureEvent(
					(Double[]) arrayX.toArray(new Double[0]),
					(Double[]) arrayY.toArray(new Double[0]), E));
		}
	}

	/**
	 * Aade el punto anterior al actual.
	 *
	 * @param p Punto a aadir.
	 */
	private void addAntPoint(Point2D p) {
		arrayX.add(arrayX.size() - 1, new Double(p.getX()));
		arrayY.add(arrayY.size() - 1, new Double(p.getY()));
	}

	/**
	 * Cambio de punto anterior.
	 *
	 * @param p Punto a cambiar.
	 */
	private void changeAntPoint(Point2D p) {
		if (arrayX.size() > 2) {
			arrayX.set(arrayX.size() - 2, new Double(p.getX()));
			arrayY.set(arrayY.size() - 2, new Double(p.getY()));
		}
	}

	/**
	 * Reimplementacin del mtodo mouseMoved de Behavior.
	 *
	 * @param E MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseMoved(MouseEvent E) throws BehaviorException {
		if (isClicked) {
			changeAntPoint(getMapControl().getViewPort().toMapPoint(E.getPoint()));

			MeasureEvent event = new MeasureEvent((Double[]) arrayX.toArray(
						new Double[0]),
					(Double[]) arrayY.toArray(new Double[0]), E);
			listener.points(event);
			getMapControl().repaint();
		}
	}

	/**
	 * Reimplementacin del mtodo mouseDragged de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Behavior.
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		mouseMoved(e);
	}
}
