/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools;

import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.fmap.tools.Listeners.PointListener;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.ImageIcon;


/**
 * Implementacin de la interfaz PointListener como herramienta para realizar
 * un zoom menos.
 *
 * @author Vicente Caballero Navarro
 */
public class ZoomOutListenerImpl implements PointListener {
	private final Image izoomout = new ImageIcon(MapControl.class.getResource(
				"images/ZoomOutCursor.gif")).getImage();
	private Cursor cur = Toolkit.getDefaultToolkit().createCustomCursor(izoomout,
			new Point(16, 16), "");
	private MapControl mapControl;

	/**
	 * Crea un nuevo ZoomOutListenerImpl.
	 *
	 * @param mapControl MapControl.
	 */
	public ZoomOutListenerImpl(MapControl mapControl) {
		this.mapControl = mapControl;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.PointListener#point(com.iver.cit.gvsig.fmap.tools.PointEvent)
	 */
	public void point(PointEvent event) {
		ViewPort vp = mapControl.getMapContext().getViewPort();
		Point2D p2 = vp.toMapPoint(event.getPoint());

		double nuevoX;
		double nuevoY;
		double factor = 1/MapContext.ZOOMOUTFACTOR;
		Rectangle2D.Double r = new Rectangle2D.Double();

		nuevoX = p2.getX() - ((vp.getExtent().getWidth() * factor) / 2.0);
		nuevoY = p2.getY() - ((vp.getExtent().getHeight() * factor) / 2.0);
		r.x = nuevoX;
		r.y = nuevoY;
		r.width = vp.getExtent().getWidth() * factor;
		r.height = vp.getExtent().getHeight() * factor;

		vp.setExtent(r);

		// mapControl.drawMap(false);
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#getCursor()
	 */
	public Cursor getCursor() {
		return cur;
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener#cancelDrawing()
	 */
	public boolean cancelDrawing() {
	    System.out.println("cancelDrawing del ZoomOutListenerImpl");
		return true;
	}

	public void pointDoubleClick(PointEvent event) {
		// TODO Auto-generated method stub

	}
}
