package org.gvsig.remoteClient.gml.utils;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferStrategy;
import java.io.IOException;

import org.gvsig.remoteClient.gml.GMLException;
import org.gvsig.remoteClient.gml.GMLTags;
import org.gvsig.remoteClient.gml.GMLFeaturesParser.Extent;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.schemas.IXMLType;
import org.gvsig.remoteClient.gml.schemas.XMLElement;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLUtilsParser.java,v 1.1 2006/08/10 12:00:49 jorpiell Exp $
 * $Log: GMLUtilsParser.java,v $
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLUtilsParser {

	/**
	 * Parses the boundedBy GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	public static Extent parseBoundedBy(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;
		Rectangle2D extent = null;	
		String srs = null;
		
		currentTag = parser.getEventType();	
		
		while (!end){
			switch(currentTag){
			case XMLSchemaParser.START_TAG:
				if (parser.getName().compareTo(GMLTags.GML_BOX)==0){
					for (int i=0 ; i<parser.getAttributeCount() ;i++){
						if (parser.getAttributeName(i).compareTo(GMLTags.GML_SRS_NAME) == 0){
							srs = parser.getAttributeValue(i);
							
						}
					}
					extent = parseBox(parser);					
				}   
				break;
			case XMLSchemaParser.END_TAG:
				if (parser.getName().compareTo(GMLTags.GML_BOUNDEDBY) == 0)
					end = true;
				break;
			case XMLSchemaParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}	
		return new Extent(extent,srs);
	}
	
	/**
	 * Parses the box GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Rectangle2D parseBox(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;
		Point2D ul = null;
		Point2D br = null;		
		Rectangle2D extent = null;
		
		currentTag = parser.getEventType();	
		
		while (!end){
			switch(currentTag){
			case XMLSchemaParser.START_TAG:
				if (parser.getName().compareTo(GMLTags.GML_COORDINATES)==0){
						extent = parseCoordinates(parser);
				}else if(parser.getName().compareTo(GMLTags.GML_COORD)==0){
					if (ul != null){
						ul = parsePoint(parser);
					}else{
						br = parsePoint(parser);
					}
				}   
				break;
			case XMLSchemaParser.END_TAG:
				if (parser.getName().compareTo(GMLTags.GML_BOX) == 0)
					end = true;
				break;
			case XMLSchemaParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}	
		if (extent == null){
			return new Rectangle2D.Double(ul.getX(),ul.getY(),br.getX(),br.getY());
		}else{
			return extent;
		}
	}

	/**
	 * Parses the coordinates GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Rectangle2D parseCoordinates(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;
		parser.next();
		
		for (int i=0 ; i<parser.getAttributeCount() ; i++){
    		if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_DECIMAL) == 0){
    			
    		}else if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_CS) == 0){
    			
    		}else if (parser.getAttributeName(i).compareTo(GMLTags.GML_COORDINATES_TS) == 0){
    			
    		}
    	}			
		
		String[] corners = parser.getText().split(" ");
		String[] ul = corners[0].split(",");
		String[] br = corners[1].split(",");
		
		return new Rectangle2D.Double(Double.parseDouble(ul[0]),
				Double.parseDouble(ul[1]),
				Double.parseDouble(br[0]),
				Double.parseDouble(br[1]));	
	}
	
	/**
	 * Parses the point GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Point2D parsePoint(XMLSchemaParser parser) throws XmlPullParserException, IOException {
		int currentTag;
		boolean end = false;		
		String x = "0";
		String y = "0";
		
		currentTag = parser.getEventType();		
		
		while (!end){
			switch(currentTag){
			case XMLSchemaParser.START_TAG:
				if (parser.getName().compareTo(GMLTags.GML_COORDINATES)==0){
					parseCoordinates(parser);					
				}   
				break;
			case XMLSchemaParser.END_TAG:
				if (parser.getName().compareTo(GMLTags.GML_X) == 0){
					x = parser.getText();
				}else if (parser.getName().compareTo(GMLTags.GML_Y) == 0){
					y = parser.getText();
					end = true;
				}					
				break;
			case XMLSchemaParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}
		}	
		return new Point2D.Double(Double.parseDouble(x),Double.parseDouble(y));
	}
	
	/**
	 * Parses a geometry GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	public static Object parseGeometry(XMLSchemaParser parser,String gmGeometryTag,IGeometriesFactory factory) throws GMLException {
		int currentTag;
		boolean end = false;		
		
		try{				
			currentTag = parser.next();				
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					try{
						if (parser.getName().compareTo(GMLTags.GML_POINT) == 0){
							Point2D point = parsePoint2D(parser);
							return factory.createPoint2D(point.getX(),point.getY());
						}else {
							return factory.createGeometry(getGeometryInGML(parser,parser.getName()));
						}
					}catch (Exception e){
						throw new GMLException(GMLException.EXC_NO_GEOMETRY);
					}
				case KXmlParser.END_TAG:
					if ((parser.getName().compareTo(gmGeometryTag) == 0))
						end = true;
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_READ_FILE);
		} 					
		return null;
	}
	
	/**
	 * Parses the point GML tag
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static Point2D parsePoint2D(XMLSchemaParser parser) throws GMLException{
		int currentTag;
		boolean end = false;		
		
		try{				
			currentTag = parser.getEventType();					
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					if ((parser.getName().compareTo(GMLTags.GML_COORDINATES) == 0)){
						parser.next();
						String sCoordinates = parser.getText();
						String[] coordinates = sCoordinates.split(",");
						return new Point2D.Double(Double.parseDouble(coordinates[0]),
								Double.parseDouble(coordinates[1]));
					}
					break;
				case KXmlParser.END_TAG:
					if ((parser.getName().compareTo(GMLTags.GML_POINT) == 0))
						end = true;
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_READ_FILE);
		} 					
		return null;	
	}
	
	/**
	 * Parses the geometry GML tag
	 * @return
	 * The geometry in GML
	 * @throws IOException 
	 * @throws XmlPullParserException 
	 *
	 */
	private static String getGeometryInGML(XMLSchemaParser parser,String endGeometryTag) throws GMLException{
		int currentTag;
		boolean end = false;
		StringBuffer gmlString = new StringBuffer();
		
		try{		
			currentTag = parser.getEventType();			
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					gmlString.append("<" + parser.getName());
					for (int i=0 ; i<parser.getAttributeCount() ; i++){
						gmlString.append(" " + parser.getAttributeName(i) + 
								"=\"" + parser.getAttributeValue(i) + "\"");
					}
					gmlString.append(">");
					break;
				case KXmlParser.END_TAG:
					if ((parser.getName().compareTo(endGeometryTag) == 0)){
						end = true;
					}
					gmlString.append("</" + parser.getName() + ">");
					break;
				case KXmlParser.TEXT:                 
					gmlString.append(parser.getText());
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}catch (XmlPullParserException e) {
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		} catch (IOException e) {
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_READ_FILE);
		} 							
		return gmlString.toString();	
	}		
	
}
