package org.gvsig.remoteClient.wfs.filters;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: BinaryTree.java,v 1.1 2006/10/05 10:25:15 jorpiell Exp $
 * $Log: BinaryTree.java,v $
 * Revision 1.1  2006/10/05 10:25:15  jorpiell
 * Implementado el filter encoding
 *
 *
 */
/**
 * This class implements a binary tree (not ordered) that is
 * used to build a sintactic tree for the SQL language.
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class BinaryTree {
	private Node root;
	private Node currentNode;
	
	public BinaryTree(){
		root = null;
		currentNode = null;
	}
	
	/**
	 * This method is called for each term that is on 
	 * the SQL query
	 * @param value
	 */
	public void addTerm(String value){
		if (value.equals("(")){
			if (root == null){
				root = new Node();
				currentNode = root;
			}else{
				if (currentNode.leftNode == null){
					currentNode.leftNode = new Node(currentNode);
					currentNode = currentNode.leftNode;
				}else if(currentNode.rigthNode == null){
					currentNode.rigthNode = new Node(currentNode);
					currentNode = currentNode.rigthNode;
				}
			}
		}else if(value.equals(")")){
			currentNode = currentNode.parentNode;
		}else{
			currentNode.value = value;
		}			
	}

	/**
	 * Print all the tree
	 *
	 */
	public void printTree(){
		printNode(root,0);
	}	
	
	/**
	 * Print one node
	 * @param node
	 * Node to print
	 * @param level
	 * Level node
	 */
	private void printNode(Node node,int level){
		if (node != null){
			String tab = "";
			for (int i=0 ; i<level ; i++){
				tab = tab + "\t";
			}
			level++;
			if (node.isField()){
				System.out.print(tab + node.value + "\n");
			}else{
				System.out.print(tab + node.value + "\n");
				printNode(node.leftNode,level);
				printNode(node.rigthNode,level);
				System.out.print(tab + "\\" + node.value + "\n");
			}
		}		
	}
	
	/**
	 * @return Returns the root.
	 */
	public Node getRoot() {
		return root;
	}	
	
	/**
	 * This class represents a binary tree node.
	 * @author Jorge Piera Llodr (piera_jor@gva.es)
	 *
	 */
	public class Node{
		private String value;
		private Node leftNode;
		private Node rigthNode;
		private Node parentNode;
		
		private Node(){
			leftNode = null;
			rigthNode = null;
			parentNode = null;
		}
		
		private Node(Node parentNode){
			leftNode = null;
			rigthNode = null;
			this.parentNode = parentNode;
		}
		
		public boolean isField(){
			if ((leftNode == null)&&
					(rigthNode == null)){
				return true;
			}
			return false;
		}

		/**
		 * @return Returns the leftNode.
		 */
		public Node getLeftNode() {
			return leftNode;
		}

		/**
		 * @return Returns the rigthNode.
		 */
		public Node getRigthNode() {
			return rigthNode;
		}

		/**
		 * @return Returns the value.
		 */
		public String getValue() {
			return value;
		}
	}

	
}

