package org.gvsig.gui.beans.filterPanel.filterButtons;

import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.text.DateFormat;
import java.util.Date;
import java.util.HashMap;

import javax.swing.JPanel;
import org.gvsig.gui.beans.textBoxWithCalendar.JCalendarDateDialog;

import com.iver.andami.PluginServices;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * This class is a panel with buttons for filter operations: AND, OR, NOT, >, <, ...
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class FilterButtonsJPanel extends JPanel implements Serializable {
	private static final long serialVersionUID = -8707991552239900454L;

	public static final int default_FilterButtonsJPanelWidth = 190;
	public static final int default_FilterButtonsJPanellHeight = 130;
	private final int buttonsGroupJPanelWidth = 190;
	private final int buttonsGroupJPanelHeight = 100;
	private final int buttonHeight = 20;
	private final int buttonWidthUnit = 40;

	private javax.swing.JButton btnEqual = null;
	private javax.swing.JButton btnDistinct = null;
	private javax.swing.JButton btnGreater = null;
	private javax.swing.JButton btnSmaller = null;
	private javax.swing.JButton btnEqualGreater = null;
	private javax.swing.JButton btnEqualSmaller = null;
	private javax.swing.JButton btnAnd = null;
	private javax.swing.JButton btnOr = null;
	private javax.swing.JButton btnNot = null;
	private javax.swing.JButton btnDate = null;
	private javax.swing.JButton btnParenthesis = null;
	private javax.swing.JButton btnDeleteText = null;
	private javax.swing.JPanel buttonsJPanel = null;	
	private javax.swing.JPanel referenceToFilterButtonsJPanel = null;
	
	// Last selected date
	private DateFormat dateFormat = DateFormat.getDateInstance();
	private Date lastSelectedDate = null;

	private JCalendarDateDialog jCalendarDateDialog = null;
	
	// Listener for property change support
	private PropertyChangeSupport changes = new PropertyChangeSupport(this);
	
	// Values of the events fired when has been clicked a button
	public static final int DEFAULT = 0;
	public static final int EQUAL = 1;
	public static final int DISTINCT = 2;
	public static final int GREATER = 3;
	public static final int SMALLER = 4;
	public static final int EQUALGREATER = 5;
	public static final int EQUALSMALLER = 6;
	public static final int AND = 7;
	public static final int OR = 8;
	public static final int NOT = 9;
	public static final int DATE = 10;
	public static final int PARENTHESIS = 11;
	public static final int DELETE_TEXT = 12;
	
	// Values of the type of event fired
	public static final int BUTTON_CLICKED_ACTION_ID = 13;
	public static final String BUTTON_CLICKED_ACTION_COMMAND = "Button Clicked";
	
	// Hash map for the items
	private HashMap map;
	
	// Action listener for notify (fire) some events that had happened to this component
	private ActionListener actionListener = null;

	/**
	 * @see JPanel#JPanel()
	 */
	public FilterButtonsJPanel() {
		super();
		map = new HashMap();
		referenceToFilterButtonsJPanel = this;
		initialize();
	}

	/**
	 * @see JPanel#JPanel(boolean)
	 */
	public FilterButtonsJPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
		map = new HashMap();
		referenceToFilterButtonsJPanel = this;
		initialize();
	}

	/**
	 * @see JPanel#JPanel(java.awt.LayoutManager, boolean)
	 */
	public FilterButtonsJPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
		map = new HashMap();
		referenceToFilterButtonsJPanel = this;
		initialize();
	}

	/**
	 * @see JPanel#JPanel(java.awt.LayoutManager)
	 */
	public FilterButtonsJPanel(LayoutManager layout) {
		super(layout);
		map = new HashMap();
		referenceToFilterButtonsJPanel = this;
		initialize();
	}
	
	/**
	 * This method initializes this
	 */
	private void initialize() {
		this.setPreferredSize(new Dimension(default_FilterButtonsJPanelWidth, default_FilterButtonsJPanellHeight));
		this.setLayout(new GridBagLayout());
		
		// Vertical center
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		gridBagConstraints.anchor = GridBagConstraints.CENTER;

		this.add(getButtonsJPanel(), gridBagConstraints);
	}
	
	/**
	 * This method initializes buttonsJPanel
	 *
	 * @return javax.swing.JPanel
	 */	
	private javax.swing.JPanel getButtonsJPanel() {
		if (buttonsJPanel == null) {
			buttonsJPanel = new JPanel();
			
			buttonsJPanel.setPreferredSize(new Dimension(buttonsGroupJPanelWidth, buttonsGroupJPanelHeight));
			buttonsJPanel.add(getBtnEqual());
			buttonsJPanel.add(getBtnDistinct());
			buttonsJPanel.add(getBtnSmaller());
			buttonsJPanel.add(getBtnGreater());
			buttonsJPanel.add(getBtnEqualSmaller());
			buttonsJPanel.add(getBtnEqualGreater());
			buttonsJPanel.add(getBtnAnd());
			buttonsJPanel.add(getBtnOr());
			buttonsJPanel.add(getBtnNot());
			buttonsJPanel.add(getBtnParenthesis());
			buttonsJPanel.add(getBtnDate());
			buttonsJPanel.add(getBtnDeleteText());
		}
		
		return buttonsJPanel;
	}
	
	/**
	 * This method initializes btnAnd
	 *
	 * @return javax.swing.JButton
	 */	
	private javax.swing.JButton getBtnAnd() {
		if (btnAnd == null) {
			btnAnd = new javax.swing.JButton();
			btnAnd.setText("And");
			btnAnd.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnAnd.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("And", new Integer(FilterButtonsJPanel.AND));
			
			btnAnd.addActionListener(this.getActionListener());
		}

		return btnAnd;
	}

	/**
	 * This method initializes btnDistinct
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnDistinct() {
		if (btnDistinct == null) {
			btnDistinct = new javax.swing.JButton();
			btnDistinct.setText("!=");
			btnDistinct.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnDistinct.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("!=", new Integer(FilterButtonsJPanel.DISTINCT));
			
			btnDistinct.addActionListener(this.getActionListener());
		}
		
		return btnDistinct;
	}
	
	
	/**
	 * This method initializes btnEqua
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnEqual() {
		if (btnEqual == null) {
			btnEqual = new javax.swing.JButton();
			btnEqual.setText("=");
			btnEqual.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnEqual.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("=", new Integer(FilterButtonsJPanel.EQUAL));

			btnEqual.addActionListener(this.getActionListener());
		}

		return btnEqual;
	}

	/**
	 * This method initializes btnGreater
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnGreater() {
		if (btnGreater == null) {
			btnGreater = new javax.swing.JButton();
			btnGreater.setText(">");
			btnGreater.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnGreater.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put(">", new Integer(FilterButtonsJPanel.GREATER));
			
			btnGreater.addActionListener(this.getActionListener());
		}

		return btnGreater;
	}

	/**
	 * This method initializes btnEqualGreater
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnEqualGreater() {
		if (btnEqualGreater == null) {
			btnEqualGreater = new javax.swing.JButton();
			btnEqualGreater.setText(">=");
			btnEqualGreater.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnEqualGreater.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put(">=", new Integer(FilterButtonsJPanel.EQUALGREATER));
			
			btnEqualGreater.addActionListener(this.getActionListener());
		}

		return btnEqualGreater;
	}

	/**
	 * This method initializes btnSmaller
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnSmaller() {
		if (btnSmaller == null) {
			btnSmaller = new javax.swing.JButton();
			btnSmaller.setText("<");
			btnSmaller.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnSmaller.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("<", new Integer(FilterButtonsJPanel.SMALLER));
			
			btnSmaller.addActionListener(this.getActionListener());
		}

		return btnSmaller;
	}

	/**
	 * This method initializes btnEqualSmaller
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnEqualSmaller() {
		if (btnEqualSmaller == null) {
			btnEqualSmaller = new javax.swing.JButton();
			btnEqualSmaller.setText("<=");
			btnEqualSmaller.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnEqualSmaller.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("<=", new Integer(FilterButtonsJPanel.EQUALSMALLER));
			
			btnEqualSmaller.addActionListener(this.getActionListener());
		}

		return btnEqualSmaller;
	}

	/**
	 * This method initializes btnDate
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnDate() {
		if (btnDate == null) {
			btnDate = new javax.swing.JButton();
			btnDate.setText("Date");
			btnDate.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnDate.setPreferredSize(new java.awt.Dimension(86, buttonHeight));
			map.put("Date", new Integer(FilterButtonsJPanel.DATE));
			
			btnDate.addMouseListener(new MouseAdapter() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
				 */
				public void mouseClicked(MouseEvent e) {
					getCDD().show();
				}				
			});			
		}

		return btnDate;
	}

	/**
	 * This method initializes a JCalendarDateDialog	
	 * 	
	 * @return 	
	 */
	protected JCalendarDateDialog getCDD() {
		if (jCalendarDateDialog == null) {
			jCalendarDateDialog = new JCalendarDateDialog(350, 230);
			jCalendarDateDialog.setModal(true);
			jCalendarDateDialog.setLocationRelativeTo(btnDate);
	
			// Adds a listener for get the date when the 
			jCalendarDateDialog.addComponentListener(new ComponentAdapter() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
				 */
				public void componentHidden(ComponentEvent e) {
					lastSelectedDate = jCalendarDateDialog.getDate();
					
					actionListener.actionPerformed(new ActionEvent(btnDate, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_ID, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND));

				}			
			});
		
		}
		
		return jCalendarDateDialog;
	}	
	
	/**
	 * This method initializes btnNot
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnNot() {
		if (btnNot == null) {
			btnNot = new javax.swing.JButton();
			btnNot.setText("Not");
			btnNot.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnNot.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));			
			map.put("Not", new Integer(FilterButtonsJPanel.NOT));
			
			btnNot.addMouseListener(new MouseAdapter() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
				 */
				public void mouseClicked(MouseEvent e) {
					new ActionEvent(this, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_ID, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND);
				}				
			});
			
			btnNot.addActionListener(this.getActionListener());

		}

		return btnNot;
	}

	/**
	 * This method initializes btnOr
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnOr() {
		if (btnOr == null) {
			btnOr = new javax.swing.JButton();
			btnOr.setText("Or");
			btnOr.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnOr.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("Or", new Integer(FilterButtonsJPanel.OR));
			
			btnOr.addMouseListener(new MouseAdapter() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
				 */
				public void mouseClicked(MouseEvent e) {
					new ActionEvent(this, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_ID, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND);
				}				
			});
			
			btnOr.addActionListener(this.getActionListener());
		}

		return btnOr;
	}

	/**
	 * This method initializes btnParenthesis
	 *
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnParenthesis() {
		if (btnParenthesis == null) {
			btnParenthesis = new javax.swing.JButton();
			btnParenthesis.setText("()");
			btnParenthesis.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnParenthesis.setPreferredSize(new java.awt.Dimension(buttonWidthUnit, buttonHeight));
			map.put("()", new Integer(FilterButtonsJPanel.PARENTHESIS));
			
			btnParenthesis.addMouseListener(new MouseAdapter() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
				 */
				public void mouseClicked(MouseEvent e) {
					new ActionEvent(this, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_ID, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND);
				}				
			});
			
			btnParenthesis.addActionListener(this.getActionListener());
		}

		return btnParenthesis;
	}
	
	/**
	 * This method initializes btnDeleteText
	 * 
	 * @return javax.swing.JButton
	 */
	private javax.swing.JButton getBtnDeleteText() {
		if (btnDeleteText == null) {
			btnDeleteText = new javax.swing.JButton();
			btnDeleteText.setText(PluginServices.getText(this, "deleteText"));
			btnDeleteText.setMargin(new java.awt.Insets(2, 2, 2, 2));
			btnDeleteText.setPreferredSize(new java.awt.Dimension(176, buttonHeight));
			map.put(PluginServices.getText(this, "deleteText"), new Integer(FilterButtonsJPanel.DELETE_TEXT));
			
			btnDeleteText.addMouseListener(new MouseAdapter() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
				 */
				public void mouseClicked(MouseEvent e) {
					new ActionEvent(this, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_ID, FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND);
				}				
			});
			
			btnDeleteText.addActionListener(this.getActionListener());
		}
			
		return btnDeleteText;	
	}
	
	/**
	 * This method initializes the "actionListener" ActionListener
	 * 
	 * @return ActionListener
	 */
	private ActionListener getActionListener() {
		if (actionListener == null) {
			actionListener = new ActionListener() {
				/*
				 *  (non-Javadoc)
				 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
				 */
				public void actionPerformed(ActionEvent event) {
				   	// Notifies that has been clicked a button
		       		changes.firePropertyChange(FilterButtonsJPanel.BUTTON_CLICKED_ACTION_COMMAND, FilterButtonsJPanel.DEFAULT, ((Integer) map.get( ((javax.swing.JButton)event.getSource()).getText())).intValue());
		        }
		    };
		}
		return actionListener;
	}
	
	/**
	 * Returns the las selected date, formatted
	 * 
	 * @return A formatted date
	 */
	public String getLastSelectedDate() {
		if (lastSelectedDate == null)
			return "";
		else
			return "Date(" + dateFormat.format(lastSelectedDate) + ")"; 
	}
	
	/**
	 * Returns the 'DateFormat' private attribute
	 * 
	 * @return A 'DateFormat' reference
	 */
	public DateFormat getDateFormat() {
		return dateFormat;
	}

    /**
     * Adds a "Property Change Listener"
     */
    public void addPropertyChangeListener(PropertyChangeListener l) {
    	changes.addPropertyChangeListener(l);
    }

    /**
     * Removes a "Property Change Listener"
     */
    public void removePropertyChangeListener(PropertyChangeListener l) {
    	changes.removePropertyChangeListener(l);
    }
}
