package org.gvsig.gui.beans.textBoxWithCalendar;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JPanel;

import org.freixas.jcalendar.*;

import com.iver.andami.PluginServices;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/**
 * Creates a Panel for include in other panels -> this panel allows users to set the date they want
 * The difference from this class to the JCalendarDatePanel is that in this class the user can move
 *    the calendar with the mouse
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class JCalendarCDatePanel extends JPanel implements java.io.Serializable{
	private static final long serialVersionUID = 8509184989434648171L;
	private int defaultWidth = 120;
	private int defaultHeight = 19;
	private JCalendarCombo calendar = null;
	
	/**
	 * This is the default constructor: creates the panel with a JCalendar object
	 */
	public JCalendarCDatePanel() {
			
		try
		{
			// Set properties to the current panel
			this.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
			this.setLayout(new BorderLayout());
			
		    // Adds the JCalendar to the current panel
		    this.add(this.getJCalendarCombo());
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * This is the default constructor with 2 parameters: creates the panel with a JCalendar object
	 */
	public JCalendarCDatePanel(int width, int height) {
			
		try
		{			
			defaultWidth = width;
			defaultHeight = height;
			
			// Set properties to the current panel
			this.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
			this.setLayout(new BorderLayout());			
		    
		    // Adds the JCalendar to the current panel
		    this.add(this.getJCalendarCombo());
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Returns a reference to the JCalendarCombo private attribute
	 * 
	 * @return A reference to the calendar
	 */
	private JCalendarCombo getJCalendarCombo() {
		if (this.calendar == null) {
			// Create a JCalendar
			calendar = new JCalendarCombo(Calendar.getInstance(), Locale.getDefault(), JCalendarCombo.DISPLAY_DATE, true);
			calendar.setDateFormat(new SimpleDateFormat("yyyy-MM-dd"));
			calendar.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
		    calendar.setEditable(false);		    
		    calendar.setBackground(Color.WHITE);
		    calendar.setToolTipText(PluginServices.getText(this, "calendarSelectDate"));
		    calendar.setToolTipTextToMonthDecrButton(PluginServices.getText(null, "calendarBackOneMonth"));
		    calendar.setToolTipTextToMonthIncrButton(PluginServices.getText(null, "calendarForwardOneMonth"));
		    calendar.setToolTipTextToYearDecrButton(PluginServices.getText(null, "calendarBackOneYear"));
		    calendar.setToolTipTextToYearIncrButton(PluginServices.getText(null, "calendarForwardOneYear"));
		}
		
		return this.calendar;
	}
	
	/**
	 * Sets the size of the CalendarDatePanel for resize the JCalendarCommbo and this panel
	 * 
	 * @param width (the new Width for the panel)
	 * @param height (the new Height for the panel)
	 */
	public void setPreferredSizeResize(int width, int height)
	{
		defaultWidth = width;
		defaultHeight = height;
		this.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
		calendar.setPreferredSize(new Dimension(defaultWidth, defaultHeight));		   
		calendar.setSize(new Dimension(defaultWidth, defaultHeight));
		this.revalidate();
	}
	
	/**
	 * Get the height of this panel
	 */
	public int getHeight()
	{
		return this.defaultHeight;
	}
	
	/**
	 * Get the width of this panel
	 */
	public int getWidth()
	{
		return this.defaultWidth;
	}
	
	/**
	 * Sets the date to the calendar
	 * 
	 * @param date
	 */
	public void setDate(Date date)
	{
		calendar.setDate(date);
	}
	
	/**
	 * Gets the date of the calendar 
	 * 
	 * @return Date
	 */
	public Date getDate()
	{
		return calendar.getDate();
	}
	
	/**
	 * Allows select the date
	 */
	public void enableCalendar()
	{
		calendar.setEnabled(true);
	}
	
	/**
	 * Don't allow select the date
	 */
	public void disableCalendar()
	{
		calendar.setEnabled(false);
	}	
	
	/**
	 * Returns the date selected, formatted
	 * 
	 * @return String The formatted date
	 */
	public String getFormattedDate() {
		return new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(this.getDate());
	}
}
