package org.gvsig.gui.beans.textBoxWithCalendar;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.freixas.jcalendar.DateEvent;
import org.freixas.jcalendar.DateListener;
import org.freixas.jcalendar.JCalendar;

import com.iver.andami.PluginServices;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ComponentListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * Creates a Panel for include in other panels -> this panel allows users to set the date they want
 * The difference from this class to the JCalendarDatePanel is that in this class the user can move
 *    the calendar with the mouse
 * 
 * @author Pablo Piqueras Bartolom (p_queras@hotmail.com)
 */
public class JCalendarDatePanel extends JPanel implements Serializable{
	private static final long serialVersionUID = 6851067300202797515L;
	private JButton jButton = null;
	private JCalendar jCalendar = null;
	private JDialog jDialogCalendar = null;
	private int defaultWidth = 120;
	private int defaultHeight = 19;
	private MouseListener mouseListener;	

	/**
	 * Default Constructor
	 */
	public JCalendarDatePanel() {
	
		try
		{
			this.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
			initialize();			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * This is the default constructor with 2 parameters for set the size
	 */
	public JCalendarDatePanel(int width, int height) {
			
		try
		{			
			defaultWidth = width;
			defaultHeight = height;
			
			this.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
			initialize();			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * This method initializes this 
	 */
	private void initialize() {
		// Set properties to the current panel
		GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 0;		
		
		GridBagLayout gridBagLayout = new GridBagLayout();
		this.setLayout(gridBagLayout);
		
		// Add components to this panel:
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.gridx = 0;
		
		// Create the JCalendar Object
		this.getJCalendar();
		
		// Create the JDialog Object
		this.getJDialogCalendar();

		// Add the JButton
		this.add(getJButton(), gridBagConstraints1);
		
		// Defines the mouseLIstener
		this.createMouseListener();
		
		// Adds the mouseListener to the jTextField and the jButton
		jButton.addMouseListener(mouseListener);
	}

	
	/**
	 * Sets the size of this panel
	 * 
	 * @param width (the new Width for the panel)
	 * @param height (the new Height for the panel)
	 */
	public void setPreferredSizeResize(int width, int height)
	{
		defaultWidth = width;
		defaultHeight = height;
		this.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
		this.revalidate();
	}
	
	/**
	 * Get the height of this panel
	 */
	public int getHeight()
	{
		return this.defaultHeight;
	}
	
	/**
	 * Get the width of this panel
	 */
	public int getWidth()
	{
		return this.defaultWidth;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setPreferredSize(new Dimension(defaultWidth, defaultHeight));
			jButton.setHorizontalTextPosition(SwingConstants.LEFT);
			jButton.setBackground(Color.WHITE);
			jButton.setText(this.getFormattedDate(jCalendar.getDate()));
		}
		return jButton;
	}
	
	/**
	 * This method initializes jCalendar	
	 * 	
	 * @return javax.swing.JCalendar	
	 */
	private JCalendar getJCalendar() {
		if (jCalendar == null) {
			jCalendar = new JCalendar();
			jCalendar.setToolTipText(PluginServices.getText(this, "calendarSelectDate"));
			
			try
			{
				jCalendar.setToolTipTextToMonthDecrButton(PluginServices.getText(null, "calendarBackOneMonth"));
				jCalendar.setToolTipTextToMonthIncrButton(PluginServices.getText(null, "calendarForwardOneMonth"));
				jCalendar.setToolTipTextToYearDecrButton(PluginServices.getText(null, "calendarBackOneYear"));
				jCalendar.setToolTipTextToYearIncrButton(PluginServices.getText(null, "calendarForwardOneYear"));

				// Adds a date listener calendar
				jCalendar.addDateListener(new DateListener(){
					/*
					 *  (non-Javadoc)
					 * @see org.freixas.jcalendar.DateListener#dateChanged(org.freixas.jcalendar.DateEvent)
					 */
					public void dateChanged(DateEvent arg0) {
						// TODO Auto-generated method stub
						jButton.setText(getFormattedDate(arg0.getSelectedDate().getTime()));
						jDialogCalendar.hide();
					}				
				});
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		return jCalendar;
	}
	
	/**
	 * This method initializes jDialogCalendar	
	 * 	
	 * @return javax.swing.JDialog	
	 */
	private JDialog getJDialogCalendar() {
		if (jDialogCalendar == null) {
			jDialogCalendar = new JDialog();
			jDialogCalendar.setSize(jCalendar.getPreferredSize());
			this.setDefaultTitle();
			
			// Adds the jCalendar to this dialog
			jDialogCalendar.getContentPane().add(jCalendar);
		}
		return jDialogCalendar;
	}
	
	
	/**
	 * Gets the date of the calendar 
	 * 
	 * @return Date
	 */
	public Date getDate() {		
		return jCalendar.getDate();
	}
	
	/**
	 * Sets the date to the calendar
	 * 
	 * @param Date
	 */
	public void setDate(Date date)
	{
		jCalendar.setDate(date);
	}
	
	/**
	 * Allows select the date
	 */
	public void enableCalendar() {
		jButton.setEnabled(true);
	}
	
	/**
	 * Don't allow select the date
	 */
	public void disableCalendar() {
		jButton.setEnabled(false);
	}

	/**
	 * Returns the date formatted
	 * 
	 * @param Date
	 * @return String The formatted date
	 */
	private String getFormattedDate(Date d) {
		return new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(d);
	}
	
	/**
	 * Returns the date selected, formatted
	 * 
	 * @return String The formatted date
	 */
	public String getFormattedDate() {
		return new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(this.getDate());
	}
	
	/**
	 * Gets the title of the JDialog with the calendar component. The title is displayed in the JDialog's border.
	 * @return Title
	 */
	public String getTitle() {
		return this.jDialogCalendar.getTitle();
	}
		
	/**
	 * Sets the title of the JDialog with the calendar component. 
	 * 
	 * @param String
	 */
	public void setTitle(String title) {
		this.jDialogCalendar.setTitle(title);
	}

	/**
	 * Sets the default title of the JDialog with the calendar component.
	 */
	public void setDefaultTitle() {
		this.jDialogCalendar.setTitle(PluginServices.getText(this, "calendarTitle"));
	}	
	
	/**
	 * @see java.awt.Dialog#setModal(boolean)
	 */
	public void setModal(boolean b) {
		jDialogCalendar.setModal(b);
	}
	
	/**
	 * Adds a component listener for the inner JDialog that cointins the calendar interface
	 * 
	 * @param componentListener
	 */
	public void addComponentListenerForJDialogCalendar(ComponentListener componentListener) {
		jDialogCalendar.addComponentListener(componentListener);
	}	
	
	/**
	 * This method creates a Mouse Listener object
	 */
	private void createMouseListener() {
		
		mouseListener = new MouseAdapter() {

			/*
			 *  (non-Javadoc)
			 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
			 */
			public void mouseClicked(MouseEvent e) {
				// TODO Auto-generated method stub
				
				// Show the JDialog
				if (getJButton().isEnabled())
				{
					jDialogCalendar.setLocationRelativeTo(jButton);
					jDialogCalendar.show();
				}				
			}	
		};
	}
	
}  //  @jve:decl-index=0:visual-constraint="10,10"
