package org.agil.core.spatialindex;

/**
 *  A point in the n-dimensional space. All dimensions are stored in an array
 *  of floats. <p>
 *
 *  Created: Tue May 18 15:01:38 1999 <p>
 *
 *
 *
 *@author     Marios Hadjieleftheriou
 *@version    2
 */
public class Point extends Object implements Cloneable {
	private float[] data;


	/**
	 *  Constructor for the Point object
	 *
	 *@param  x  Description of Parameter
	 *@param  y  Description of Parameter
	 */
	public Point(double x, double y) {
		data = new float[2];
		data[0] = (float) x;
		data[1] = (float) y;
	}


	/**
	 *  Constructor for the Point object
	 *
	 *@param  d  Description of Parameter
	 */
	public Point(float[] d) {
		if (d == null) {
			throw new IllegalArgumentException("Coordinates cannot be null.");
		}
		if (d.length < 2) {
			throw new IllegalArgumentException("Point dimension should be greater than 1.");
		}
		data = new float[d.length];
		System.arraycopy(d, 0, data, 0, d.length);
	}


	/**
	 *  Constructor for the Point object
	 *
	 *@param  d  Description of Parameter
	 */
	public Point(int[] d) {
		if (d == null) {
			throw new IllegalArgumentException("Coordinates cannot be null.");
		}
		if (d.length < 2) {
			throw new IllegalArgumentException("Point dimension should be greater than 1.");
		}
		data = new float[d.length];
		for (int i = 0; i < d.length; i++) {
			data[i] = (float) d[i];
		}
	}


	/**
	 *  Gets the FloatCoordinate attribute of the Point object
	 *
	 *@param  i  Description of Parameter
	 *@return    The FloatCoordinate value
	 */
	public float getFloatCoordinate(int i) {
		return data[i];
	}


	/**
	 *  Gets the IntCoordinate attribute of the Point object
	 *
	 *@param  i  Description of Parameter
	 *@return    The IntCoordinate value
	 */
	public int getIntCoordinate(int i) {
		return (int) data[i];
	}


	/**
	 *  Gets the Dimension attribute of the Point object
	 *
	 *@return    The Dimension value
	 */
	public int getDimension() {
		return data.length;
	}


	/**
	 *  Compares this to the parameter.
	 *
	 *@param  p  the reference object with which to compare.
	 *@return    <tt>true</tt> if this object is the same as the obj argument;
	 *      <tt>false</tt> otherwise.
	 */
	public boolean equals(Point p) {
		if (p.getDimension() != getDimension()) {
			throw new IllegalArgumentException("Points must be of equal dimensions to be compared.");
		}
		for (int i = 0; i < getDimension(); i++) {
			// if (getFloatCoordinate(i) != p.getFloatCoordinate(i)) {
//			if (!com.soluzionasf.onis.geometria.GMath.eq(getFloatCoordinate(i),p.getFloatCoordinate(i))){
			if (Math.abs((getFloatCoordinate(i) - p.getFloatCoordinate(i))) > 0.001d) {
				return false;
			}
		}
		return true;
	}


	/**
	 *  Creates an exact copy of this object.
	 *
	 *@return    Description of the Returned Value
	 */
	public Object clone() {
		return new Point(data);
	}


	/**
	 *  Converts to a String representation of the object.
	 *
	 *@return    A string representation of the object.
	 */
	public String toString() {
		String s = new String();
		for (int i = 0; i < data.length; i++) {
			s += data[i] + " ";
		}
		return s;
	}
}
// Point
